      PROGRAM APRGINFO
C-----------------------------------------------------------------------
C     DONALD E. AMOS, MAY 1, 2002
C
C     THIS FILE CONTAINS THE DRIVERS WHICH WILL EXERCIZE SUBROUTINES IN
C     FILE AMOSSUBS.FOR. THEY SERVE TO DEMOMSTRATE CALL PROCEDURES, A
C     NOMINAL USE OF ERROR FLAGS AND TYPICAL RELATIVE ERRORS. 
C
C     USAGE:
C
C     EXTRACT ONE OF THE PROGRAMS BELOW ALONG WITH ANY SUCCEEDING
C     DOUBLE PRECISION FUNCTION(S) OR SUBROUTINES, COMPILE, LINK
C     COMPILED FILE AMOSSUBS.FOR, AND THEN EXECUTE THE LINKED FILES.
C     OUTPUT IS WRITTEN TO A TEXT FILE(*.TXT) WITH THE NAME OF THE
C     PROGRAM BEING EXECUTED.
C
C     PROGRAMS:                 OUTPUT FILE:                 FOLDER
C              PROGRAM GECOMP               GECOMP.TXT         18
C              PROGRAM GHECOMP              GHECOMP.TXT        18
C              PROGRAM DFCOMP               DFCOMP.TXT         16
C              PROGRAM DGCOMP               DGCOMP.TXT         16
C              PROGRAM HERFCOMP             HERFCOMP.TXT       23
C-----------------------------------------------------------------------
      END
      PROGRAM GECOMP
C
C     COMPARES INTEGER ORDER ROUTINES DGEXINT AND DEXINT
C     AGAINST A QUADRATURE. OUTPUT IS WRITTEN TO FILE GECOMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ENI(100),EN(100)
      COMMON /CQFUN/AX, NU, KTEST
      EXTERNAL DQFUN
      OPEN(UNIT=7,FILE="GECOMP.TXT")
      WRITE (7,110)
 110  FORMAT('     X           QUAD          RELERR1  IERR1 IERR2')
 111  CONTINUE
      PRINT *,'INPUT 1 TO TEST DGEXINT; 2 TO TEST DEXINT: '
      READ *,ICASE
      IF((ICASE.LT.1).OR.(ICASE.GT.2)) THEN
        GOTO 111
      ENDIF
      IF(ICASE.EQ.1) THEN
      PRINT *,'INPUT 1 TO TEST G SEQUENCE; INPUT 2 TO TEST E SEQUENCE: '
      READ *, ITEST
      ELSE
        ITEST=2
      ENDIF        
      PRINT *,'INPUT STARTING VALUE OF ORDER (INTEGER > 0): '
      READ *, NIN
      IF((ITEST.LT.1).OR.(ITEST.GT.2)) GOTO 111
      IF(NIN.LT.1) GOTO 111
      DO 3 KODE=1,2
        WRITE(7,308)KODE
 308    FORMAT(/'                       KODE = ',I3/)
      KTEST=ITEST
      N=NIN
      DO 5 MM=1,5
        IF(ICASE.EQ.1) THEN
          IF(ITEST.EQ.1) THEN
            WRITE(7,305) MM,N,KODE
 305    FORMAT(/'TESTING G SEQ IN DGEXINT',' MM = ',I3,' N = ',I3,
     &' KODE = ',I3)
          ELSE
            WRITE(7,306) MM,N,KODE
 306    FORMAT(/'TESTING E SEQ IN DGEXINT',' MM = ',I3,' N = ',I3,
     &' KODE = ',I3)
          ENDIF
        ELSE
          WRITE(7,307) MM,N,KODE
 307    FORMAT(/'TESTING E SEQ IN DEXINT',' MM = ',I3,' N = ',I3,
     &' KODE = ',I3)
        ENDIF
        NIX=50
        DO 10 IX=1,NIX
          X=DBLE(FLOAT(IX))/5.0D0
C         KODE IS DEFINED ABOVE
          REL=0.5D-15
          TOL=REL
          IF(ICASE.EQ.1) THEN
            CALL DGEXINT(X, N, KODE, MM, TOL, ENI, EN, NZ, IERR)
          ELSE
            CALL DEXINT(X, N, KODE, MM, TOL,  EN, NZ, IERR)
          ENDIF
          DEX=DEXP(X)
        DO 50 J=1,MM
          IF(ITEST.EQ.1) THEN
            Y=ENI(J)
          ELSE
            Y=EN(J)
          ENDIF
          INIT=0
          TOL=0.5D-14
          AX=X
          NU=N+J-1
          X1=1.0D0
          SIG=3.0D0/X
          CALL DQUAD8(DQFUN,INIT,X1,SIG,TOL,X2,QANS,IERR1)
C         CONTINUE QUADRATURE PAST X2 TO BE SURE THE TRUNCATION IS
C         ADEQUATE
          SIG=6.0D0/X
          CALL DQUAD8(DQFUN,INIT,X1,SIG,TOL,X2,QANS,IERR2)
          IF(KODE.EQ.2) QANS=QANS*DEX
          ERR=DABS(Y-QANS)
          IF (QANS.NE.0.0D0) THEN
            ERR1=ERR/DABS(QANS)
          ENDIF
          IF (J.EQ.1) THEN
            WRITE (7,200) X,QANS,ERR1,IERR1,IERR
 200        FORMAT(D11.4,2D15.5,2I4)
          ELSE
            WRITE (7,201) QANS,ERR1,IERR1,IERR
 201        FORMAT(11X,2D15.5,2I4)
          ENDIF
 50     CONTINUE 
 10   CONTINUE
 5    CONTINUE
 3    CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQFUN(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      COMMON /CQFUN/ AX,NU,KTEST
      N=NU
      IF (KTEST.EQ.1) THEN
        KODE=1
        MM=1
        TOL=0.5D-15
        ARG=AX*T
        CALL DEXINT(ARG, N, KODE, MM, TOL, EN, NZ, IERR)
        DQFUN=EN(1)/(T**N)
      ELSE
        DQFUN=DEXP(-AX*T)/(T**N)
      ENDIF
      RETURN
      END
      PROGRAM GHECOMP
C
C     COMPARES HALF ODD INTEGER ORDER ROUTINES DGHEXINT AND
C     DHEXINT AGAINST A QUADRATURE. OUTPUT IS WRITTEN TO FILE
C     GHECOMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ENI(100),EN(100)
      COMMON /CQFUN/AX, FNU, KTEST
      EXTERNAL DQFUN
      OPEN(UNIT=7,FILE="GHECOMP.TXT")
      WRITE (7,110)
 110  FORMAT('     X           QUAD          RELERR1  IERR1 IERR2')
 111  CONTINUE
      PRINT *,'INPUT 1 TO TEST DGHEXINT; 2 TO TEST DHEXINT: '
      READ *,ICASE
      IF((ICASE.LT.1).OR.(ICASE.GT.2)) THEN
        GOTO 111
      ENDIF
      IF(ICASE.EQ.1) THEN
      PRINT *,'INPUT 1 TO TEST G SEQUENCE; INPUT 2 TO TEST E SEQUENCE: '
      READ *, ITEST
      ELSE
        ITEST=2
      ENDIF        
      PRINT *,'INPUT STARTING VALUE OF ORDER (DP HALF ODD INTEGER > 0):
     &'
      READ *, FNHI
      IF((ITEST.LT.1).OR.(ITEST.GT.2)) GOTO 111
      IF((FNHI-DBLE(FLOAT(INT(FNHI)))).NE.0.5D0) GOTO 111
      DO 3 KODE=1,2
        WRITE(7,308)KODE
 308    FORMAT(/'                       KODE = ',I3/)
      KTEST=ITEST
      FNH=FNHI
      DO 5 MM=1,5
        IF(ICASE.EQ.1) THEN
          IF(ITEST.EQ.1) THEN
            WRITE(7,305) MM,FNH,KODE
 305    FORMAT(/'TESTING G SEQ IN DGHEXINT',' MM = ',I3,' FNH = ',D10.3,
     &' KODE = ',I3)
          ELSE
            WRITE(7,306) MM,FNHI,KODE
 306    FORMAT(/'TESTING E SEQ IN DGHEXINT',' MM = ',I3,' FNH = ',D10.3,
     &' KODE = ',I3)
          ENDIF
        ELSE
          WRITE(7,307) MM,FNHI,KODE
 307    FORMAT(/'TESTING E SEQ IN DHEXINT',' MM = ',I3,' FNH = ',D10.3,
     &' KODE = ',I3)
        ENDIF
        NIX=50
        DO 10 IX=1,NIX
          X=DBLE(FLOAT(IX))/5.0D0
C         KODE IS DEFINED ABOVE
          REL=0.5D-15
          TOL=REL
          FNH=FNHI
          IF(ICASE.EQ.1) THEN
            CALL DGHEXINT(X, FNH, KODE, MM, TOL, ENI, EN, NZ, IERR)
          ELSE
            CALL DHEXINT(X, FNH, KODE, MM, TOL,  EN, NZ, IERR)
          ENDIF
          DEX=DEXP(X)
        DO 50 J=1,MM
          IF(ITEST.EQ.1) THEN
            Y=ENI(J)
          ELSE
            Y=EN(J)
          ENDIF
          INIT=0
          TOL=0.5D-14
          AX=X
          FNU=FNHI+DBLE(FLOAT(J-1))
          X1=1.0D0
          SIG=3.0D0/X
          CALL DQUAD8(DQFUN,INIT,X1,SIG,TOL,X2,QANS,IERR1)
C         CONTINUE QUADRATURE PAST X2 TO BE SURE THE TRUNCATION IS
C         ADEQUATE
          SIG=6.0D0/X
          CALL DQUAD8(DQFUN,INIT,X1,SIG,TOL,X2,QANS,IERR2)
          IF(KODE.EQ.2) QANS=QANS*DEX
          ERR=DABS(Y-QANS)
          IF (QANS.NE.0.0D0) THEN
            ERR1=ERR/DABS(QANS)
          ENDIF
          IF (J.EQ.1) THEN
            WRITE (7,200) X,QANS,ERR1,IERR1,IERR
 200        FORMAT(D11.4,2D15.5,2I4)
          ELSE
            WRITE (7,201) QANS,ERR1,IERR1,IERR
 201        FORMAT(11X,2D15.5,2I4)
          ENDIF
 50     CONTINUE 
 10   CONTINUE
 5    CONTINUE
 3    CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQFUN(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      COMMON /CQFUN/ AX,FNU,KTEST
      FNH=FNU
      IF (KTEST.EQ.1) THEN
        KODE=1
        MM=1
        TOL=0.5D-15
        ARG=AX*T
        CALL DHEXINT(ARG, FNH, KODE, MM, TOL, EN, NZ, IERR)
        DQFUN=EN(1)/(T**FNH)
      ELSE
        DQFUN=DEXP(-AX*T)/(T**FNH)
      ENDIF
      RETURN
      END
      PROGRAM DFCOMP
C
C     COMPARES THE COMPUTATION OF
C         DFERF=  INTEGRAL ON [0,X] OF ERF(T)/T
C     WITH A QUADRATURE.  SEE FOLDER 16
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      EXTERNAL DQFUN
      OPEN(UNIT=7,FILE="DFCOMP.TXT")
      WRITE (7,110)
 110  FORMAT('     X           QUAD           RELERR1   IERR')
      DO 10 IX=1,1000
        X=DBLE(FLOAT(IX))/100.0D0
        REL=0.5D-15
        Y=DFERF(X,REL,IERR)
        X1=0.0D0
        X2=X
        TOL=0.5D-14
        CALL DGAUS8(DQFUN,X1,X2,TOL,QANS,IERR)
        ERR=DABS(Y-QANS)
        IF (QANS.NE.0.0D0) THEN
          ERR1=ERR/DABS(QANS)
        ENDIF
        WRITE (7,200) X,QANS,ERR1,IERR
 200    FORMAT(D11.4,2D15.5,I4)
 10   CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQFUN(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      KODE=1
      NZ=0
      DQFUN=DRERF(T,KODE,NZ)/T
      RETURN
      END
      PROGRAM DGCOMP
C
C     COMPARES G FUNCTION OF FOLDER 16 WITH A QUADRATURE
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      EXTERNAL DQFUN
      OPEN(UNIT=7,FILE="DGCOMP.TXT")
      WRITE (7,110)
 110  FORMAT('     X           QUAD          RELERR1    IERR IERR1 IERR2
     &')
      DO 5 KODE=1,2
      WRITE(7,120)KODE
 120  FORMAT(/'            KODE = ',I5/)
      DO 10 IX=1,200
        X=DBLE(FLOAT(IX))/10.0D0
        REL=0.5D-15
        Y=DGERFC(X,KODE,REL,IERR)
        INIT=0
        TOL=REL*10.0D0
        X1=X
        SIG=3.0D0
        CALL DQUAD8(DQFUN,INIT,X1,SIG,TOL,X2,QANS,IERR1)
        SIG=6.0D0
        CALL DQUAD8(DQFUN,INIT,X1,SIG,TOL,X2,QANS,IERR2)
        IF(KODE.EQ.2) THEN
          QANS=QANS*DEXP(X*X)
        ENDIF
        ERR=DABS(Y-QANS)
        IF (QANS.NE.0.0D0) THEN
          ERR1=ERR/DABS(QANS)
        ENDIF
        WRITE (7,200) X,QANS,ERR1,IERR,IERR1,IERR2
 200    FORMAT(D11.4,2D15.5,3I5)
 10   CONTINUE
 5    CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQFUN(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      KODE=2
      NZ=0
      DQFUN=DRERF(T,KODE,NZ)/T
      RETURN
      END
      PROGRAM HERFCOMP
C
C     DONALD E. AMOS, MODIFIED MAY,2005
C
C     COMPARES H23 SERIES OF FOLDER 23 AND THE ASYMPTOTIC EXPANSION
C     OF FOLDER 23 WITH A QUADRATURE ON
C
C     HANS=DHERFC(X)= INT ON (0,X) OF EXP(W*W)*ERFC(W). 
C
C     COMPARISONS ARE WRITTEN TO FILE HERFCOMP.TXT IN TERMS OF RELATIVE
C     ERRORS
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      EXTERNAL DQHERFC
      OPEN(UNIT=7,FILE="HERFCOMP.TXT")
      WRITE(7,201)
 201  FORMAT(2X,'COMPARISON OF SERIES AND ASYMPTOTIC EXPANSION AGAINST A
     & QUADRATURE FOR H23'// 
     &       '  ERRS =RELATIVE ERROR FOR THE SERIES',/
     &       '  ERRA =RELATIVE ERROR FOR THE ASYMPTOTIC EXPANSION',/
     &       '  IERR =1, A NORMAL RETURN FROM DGAUS8',/
     &       '  TRMSA=NUMBER OF TERMS OF THE ASYMPTOTIC EXPANSION'/)
      WRITE(7,200)
 200  FORMAT(7X,"X",11X,'QUAD',9X,'ERRS',9X,'ERRA',5X,'IERR',2X,'TRMSA')
      DO 10 IX=1,1000
        X=DBLE(FLOAT(IX))/10.0D0
        NSIG=INT(X/3.0D0)+1
        SIG=X/DBLE(FLOAT(NSIG))
        QSUM=0.0D0
        X2=0.0D0
        REL=0.5D-14
        DO 20 J=1,NSIG
          X1=X2
          X2=X2+SIG
          TOL=REL
          CALL DGAUS8(DQHERFC,X1,X2,TOL,QANS,IERR)
          QSUM=QSUM+QANS
 20     CONTINUE
        HANS=DHERFC(X)
        ERR1=DABS(HANS-QSUM)
        IF(QANS.NE.0.0D0) THEN
          ERR1=ERR1/DABS(QANS)
        ENDIF
        IF(X.GE.4.0D0) THEN
          HASY=DHASY(X,NTRMS)
          ERR2=DABS(HASY-QSUM)
          IF(QANS.NE.0.0D0) THEN
            ERR2=ERR2/DABS(QANS)
          ENDIF
        ENDIF
        IF(X.LT.4.0D0) THEN
          WRITE(7,100) X, QSUM, ERR1,IERR
 100      FORMAT(3D13.5,13X,I5)
        ELSE
          WRITE(7,101) X, QSUM, ERR1,ERR2,IERR,NTRMS
 101      FORMAT(4D13.5,2I5)
        ENDIF
 10   CONTINUE
      END 
      DOUBLE PRECISION FUNCTION DQHERFC(W)
C     INTEGRAND FOR DGAUS8 IN DHERFCOMP
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      KODE=3
      NZ=0
      DQHERFC=DRERF(W,KODE,NZ)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DHASY(X,NLAST)
C     ASYMPTOTIC EXPANSION FOR X TO INFINITY FOR H23
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI  /1.772453850905516D0/
      DATA EULER /5.77215664901532861D-01/
      DATA TOL   /0.5D-14/
      NLAST=0
      S0=EULER+2.0D0*DLOG(X+X)
C     K=1 TERM
      X2=X*X
      TM=-0.5D0/X2
      HN=1.5D0
      SS=TM
      AN=2.0D0
      DO 10 K=2,20
        TM=-TM*HN/X2
        TRM=TM/AN
        IF(DABS(TRM).LE.DABS(SS)*TOL) GOTO 20
        SS=SS+TRM
        HN=HN+1.0D0
        AN=AN+1.0D0
  10  CONTINUE
  20  CONTINUE
      NLAST=K
      DHASY=0.5D0*(S0-SS)/RTPI
      RETURN
      END

