      PROGRAM BPRGINFO
C-----------------------------------------------------------------------
C     DONALD E. AMOS, MAY 1, 2002,   FEBRUARY, 2006
C
C     THIS FILE CONTAINS THE DRIVERS WHICH WILL EXERCIZE SUBROUTINES IN
C     FILE BECKSUBS.FOR. THEY SERVE TO DEMOMSTRATE CALL PROCEDURES, A
C     NOMINAL USE OF ERROR FLAGS AND TYPICAL RELATIVE ERRORS. FILE
C     AMOSSUBS.FOR MUST ALSO BE ATTACHED TO EXECUTE THESE DRIVERS.
C
C     USAGE:
C
C     EXTRACT ONE OF THE PROGRAMS BELOW ALONG WITH ANY SUCCEEDING
C     DOUBLE PRECISION FUNCTION(S) OR SUBROUTINES, COMPILE, LINK
C     COMPILED FILES BECKSUBS.FOR AND AMOSSUBS.FOR, AND THEN EXECUTE THE
C     LINKED FILES. OUTPUT IS WRITTEN TO A TEXT FILE (*.TXT) WITH THE
C     NAME OF THE PROGRAM BEING EXECUTED.
C                                                           
C     PROGRAMS:                      OUTPUT:                 FOLDER
C              PROGRAM I1COMP               I1COMP.TXT       1,2,10
C              PROGRAM I2COMP               I2COMP.TXT         9
C              PROGRAM I9COMP               I9COMP.TXT         9
C              PROGRAM I3COMP               I3COMP.TXT         7
C              PROGRAM I5COMP               I5COMP.TXT         5
C              PROGRAM J5COMP               J5COMP.TXT         5
C              PROGRAM V5COMP               V5COMP.TXT         5             
C              PROGRAM I6COMP               I6COMP.TXT       3,6,15
C              PROGRAM W3COMP               W3COMP.TXT         10
C              PROGRAM PCOMP                PCOMP.TXT          11
C              PROGRAM QCOMP                QCOMP.TXT          11
C              PROGRAM I21COMP              I21COMP.TXT        21
C              PROGRAM J21COMP              J21COMP.TXT        21
C              PROGRAM I22COMP              I22COMP.TXT        22
C              PROGRAM J22COMP              J22COMP.TXT        22
C              PROGRAM I29COMP              I29COMP.TXT        29
C              PROGRAM GNCOMP               GNCOMP.TXT         21
C              PROGRAM VTCOMP               VTCOMP.TXT         21
C-----------------------------------------------------------------------
      END
      PROGRAM I1COMP
C
C     DONALD E. AMOS   JANUARY, 2003; APRIL, 2006
C
C     REF: FOLDER 1, FOLDER 2, FOLDER 10
C
C     I1COMP COMPARES THE CLOSED FORM OF I1 OF FOLDERS 1, 2 AND 10
C     WITH A QUADRATURE FOR I1 FOR KODE=1:
C
C          I1=INT ON (T,INF) OF EXP(-A^2*W^2)*ERF(B*W)/W^2
C
C     AND THE COMPLEMETARY FORM FOR KODE=2:
C
C         I1C=INT ON (T,INF) OF EXP(-A^2*W^2)*ERFC(B*W)/W^2
C
C     WHERE A.GE.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0. THE CLOSED FORM IS
C     IMPLEMENTED IN DOUBLE PRECISION SUBROUTINE INTEGI1.
C
C     OUTPUT IS WRITTEN TO FILE I1COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I1COMP.TXT")
      DO 5 KODE=2,2
      WRITE(7,115) KODE
 115  FORMAT(/,'                          KODE = ',I3/)
      WRITE (7,110)
 110  FORMAT('       A          B           T          QUAD         ERR
     *     IERR IERR1')
      DO 10 IT=1,101,10
        T=DBLE(FLOAT(IT))/40.0D0
        DO 20 IA=1,101,10
          A=DBLE(FLOAT(IA-1))/40.0D0
          DO 30 IB=1,101,10
            B=DBLE(FLOAT(IB-1))/40.0D0
            REL=0.50D-14
            CALL INTEGI1(A,B,T,KODE,REL,ANS1,IERR1)
            CALL I1QUAD(A,B,T,KODE,QANS,IERR)
            ERR=DABS(QANS-ANS1)
            IF(QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE(7,100) A,B,T,QANS,ERR,IERR,IERR1
 100        FORMAT(3D12.4,D13.5,D12.4,2I5)
 101        FORMAT(3I5)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
   5  CONTINUE
      END
      SUBROUTINE I1QUAD(A,B,T,KODE,QANS,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      COMMON/ CQFI1/ CA2,CB,CB2,KKODE
      DATA RTPI       /1.772453850905516D0/
      DATA REL /0.50D-14/
      EXTERNAL DQI1
      IERR=0
      IF(B.EQ.0.0D0) THEN
        IF(KODE.EQ.1) THEN
          QANS=0.0D0
          GOTO 40
        ELSE
          IF(A.EQ.0.0D0) THEN
            QANS=1.0D0/T
            GOTO40
          ENDIF
        ENDIF
      ENDIF
      IF (A.EQ.0.0D0) THEN
        IF(KODE.EQ.1) THEN
          BT=B*T
          IF(BT.GE.6.0D0) THEN
            QANS=1.0D0/T
          ELSE
            ERFB=DRERF(BT,KODE,NZ)
            BT2=BT*BT
            N=1
            IKODE=1
            M=1
            TOL=REL
            NZ=0
            IERR=0
            CALL DEXINT(BT2, N, IKODE, M, TOL, EN, NZ, IERR)
            QANS=ERFB/T+B*EN(1)/RTPI
          ENDIF
          GOTO 40
        ELSE
          XXA=B
        ENDIF
      ELSE
        XXA=DSQRT(A*A+B*B) 
      ENDIF
C     SET COMMON VARIABLES FOR QUADRATURE
      CA2=A*A
      CB=B
      CB2=CB*CB
      KKODE=KODE
      SIG=3.0D0/XXA
      X1=T
      TOL=REL
      INIT=0
      CALL DQUAD8(DQI1,INIT,X1,SIG,TOL,X2,QANS,IERR)
C     GET BETTER ESTIMATE OF THE TRUNCATION ERROR
      TOL=REL
      CALL DQUAD8(DQI1,INIT,X1,SIG,TOL,X2,QANS,IERR)
 40   CONTINUE
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQI1(T)
C
C     DQI1 COMPUTES THE INTEGRAND FOR THE QUADRATURE OF I1 AND I1C
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CQFI1/ CA2,CB,CB2,KKODE
      T2=T*T
      ARG=CB*T
      IF(KKODE.EQ.1) THEN
        ERFX=DRERF(ARG,KKODE,NZ)
        DQI1=ERFX*DEXP(-CA2*T2)/T2        
      ELSE
        IKODE=3
        ERFX=DRERF(ARG,IKODE,NZ)
        ARG=(CA2+CB2)*T2
        DQI1=ERFX*DEXP(-ARG)/T2
      ENDIF
      RETURN
      END
      PROGRAM I2COMP
C
C     DONALD E. AMOS    DECEMBER, 2005; APRIL, 2006
C
C     REF: FOLDER 9, FOLDER 7
C
C     ON KODE=1, I2COMP COMPUTES THE INTEGRAL
C
C                I2=INT ON (0,T) OF ERF(A*W)*ERF(B*W);
C
C     ON KODE=2, I2COMP COMPUTES  THE INTEGRAL
C
C                I2C=INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)
C
C     AND EACH INTEGRAL IS COMPARED WITH ITS RESPECTIVE QUADRATURE,
C     WHERE     A.GT.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0.
C
C     OUTPUT IS WRITTEN TO FILE I2COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA REL/0.5D-14/
      OPEN(UNIT=7,FILE="I2COMP.TXT")
      DO 5 KODE=1,2
        KKODE=KODE
        WRITE(7,109) KODE
 109    FORMAT(/,27X,"KODE = ",I1)
        WRITE (7,110)
 110    FORMAT('       A          B           T          QUAD       REL
     &ERR     IERR  IERR2')
        DO 10 IT=1,21,2
          T=DBLE(FLOAT(IT))/5.0D0
          DO 20 IA=1,21,2
            A=DBLE(FLOAT(IA-1))/1.0D0
            DO 30 IB=3,21,2
              B=DBLE(FLOAT(IB-1))/1.0D0+REL
              CALL INTEGI2(A,B,T,KODE,ANSI2,IERR2)
              CALL I2QUAD(A,B,T,KODE,QANS,IERR)
              ERR=DABS(QANS-ANSI2)
              IF(QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE(7,100) A,B,T,QANS,ERR,IERR,IERR2
 100          FORMAT(3D12.4,D13.5,D12.4,2I6)
   30       CONTINUE
   20     CONTINUE
   10   CONTINUE
   5  CONTINUE
      END
      SUBROUTINE I2QUAD(A,B,T,KODE,QANS,IERR)
C     I2 BY QUADRATURE
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI2/ CA,CB,KKODE
      DATA TOL/0.5D-14/
      EXTERNAL DQI2
      CA=A
      CB=B
      KKODE=KODE
      IF(KODE.EQ.1) THEN
        XA=MIN(A,B)
        IF((XA.EQ.0.0D0).OR.(T.EQ.0.0D0)) THEN
          QANS=0.0D0
          IERR=1
          RETURN
        ENDIF      
        WSM=MIN(6.0D0/A,6.0D0/B)
        WBM=MAX(6.0D0/A,6.0D0/B)
        IF(T.LE.WSM) THEN
C         INTERVAL 0.LE.T.LE.WSM
          MMQ=INT(SNGL(T))+2
          MQ=MIN(MMQ,20)
          SIG=T/DBLE(FLOAT(MQ))
          INIT=-MQ
          X1=0.0D0
          REL=TOL
          CALL DQUAD8(DQI2,INIT,X1,SIG,REL,X2,QANS,IERR)
        ELSE
          IF(T.LE.WBM) THEN
C           INTERVAL (0,WSM]
C           ADJUST THE NUMBER OF SIG INTERVALS ACCORDING TO THE LENGTH
C           OF THE QUADRATURE INTERVAL
            MMQ=INT(SNGL(WSM))+2
            MQ=MIN(MMQ,20)
            SIG=WSM/DBLE(FLOAT(MQ))
            INIT=-MQ
            X1=0.0D0
            REL=TOL
            CALL DQUAD8(DQI2,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=ANS
C           INTERVAL (WSM,WBM]
            XL=T-WSM
            MMQ=INT(SNGL(XL))+2
            MQ=MIN(MMQ,20)
            SIG=XL/DBLE(FLOAT(MQ))
C           TEST TO DETERMINE IF T IS TOO CLOSE TO WSM TO GET AN
C           ACCURATE QUADRATURE (SUPPRESS AN ERROR MESSAGE FROM DGAUS8)
            XMID=0.5D0*(T+WSM)
            FI2=DQI2(XMID)
            IF(FI2*SIG.LE.2.0D0*REL*QANS) GOTO 51
            INIT=-MQ
            X1=WSM
            REL=TOL
            CALL DQUAD8(DQI2,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=QANS+ANS
 51         CONTINUE
          ELSE
C           INTERVAL (0,WSM]
            MMQ=INT(SNGL(WSM))+2
            MQ=MIN(MMQ,20)
            SIG=WSM/DBLE(FLOAT(MQ))
            INIT=-MQ
            X1=0.0D0
            REL=TOL
            CALL DQUAD8(DQI2,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=ANS
C           INTERVAL (WSM,WBM]
            XL=WBM-WSM
            MMQ=INT(SNGL(XL))+2
            MQ=MIN(MMQ,20)
            SIG=XL/DBLE(FLOAT(MQ))
C           SEE IF WSM AND WBM ARE TOO CLOSE TOGETHER TO GET AN ACCURATE
C           ANSWER FROM DGAUS8
            XMID=0.5D0*(WSM+WBM)
            FI2=DQI2(XMID)
            IF(FI2*SIG.LE.2.0D0*REL*QANS) GOTO 52
            INIT=-MQ
            X1=WSM
            REL=TOL
            CALL DQUAD8(DQI2,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=QANS+ANS
 52         CONTINUE
C           INTERVAL (WBM,T]
            XL=T-WBM
            MMQ=INT(SNGL(XL))+2
            MQ=MIN(MMQ,20)
            SIG=XL/DBLE(FLOAT(MQ))
C           SEE IF T AND WBM ARE TOO CLOSE TOGETHER TO GET AN ACCURATE
C           ANSWER FROM DGAUS8
            XMID=0.5D0*(T+WBM)
            FI2=DQI2(XMID)
            IF(FI2*SIG.LE.2.0D0*REL*QANS) GOTO 53
            INIT=-MQ
            X1=WBM
            REL=TOL
            CALL DQUAD8(DQI2,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=QANS+ANS
 53         CONTINUE
          ENDIF
        ENDIF
      ELSE
        INIT=0
        X1=T
        A2=A*A
        B2=B*B
        SIG=6.0D0/DSQRT(A2+B2)
        REL=TOL
        CALL DQUAD8(DQI2,INIT,X1,SIG,REL,X2,QANS,IERR)
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQI2(T)
C
C     DQI2 COMPUTES THE INTEGRAND FOR FUNCTIONS I2 AND I2C
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI2/ CA,CB,KKODE
      ARG=CA*T
      ERFA=DRERF(ARG,KKODE,NZ)
      ARG=CB*T
      ERFB=DRERF(ARG,KKODE,NZ)
      DQI2=ERFA*ERFB
      RETURN
      END
      PROGRAM I9COMP
C
C     DONALD E. AMOS    DECEMBER, 2005; APRIL, 2006
C
C     REF: FOLDER 9, FOLDER 7
C
C     ON KODE=1, I9COMP COMPUTES THE INTEGRAL
C
C                I9=INT ON (0,T) OF W*ERF(A*W)*ERF(B*W);
C
C     ON KODE=2, I9COMP COMPUTES THE COMPLEMENTARY INTEGRAL
C
C                I9C=INT ON (T,INF) OF W*ERFC(A*W)*ERFC(B*W)
C
C     AND EACH INTEGRAL IS COMPARED WITH ITS RESPECTIVE QUADRATURE,
C     WHERE     A.GT.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0.
C
C     OUTPUT IS WRITTEN TO FILE I9COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA REL/0.50D-14/
      OPEN(UNIT=7,FILE="I9COMP.TXT")
      DO 5 KODE=1,2
        WRITE(7,109) KODE
 109    FORMAT(/,27X,"KODE = ",I1)
        WRITE (7,110)
 110    FORMAT('       A          B           T          QUAD       REL
     &ERR     IERR  IERR9')
        DO 10 IT=1,21,2
          T=DBLE(FLOAT(IT))/5.0D0
          DO 20 IA=1,21,2
            A=DBLE(FLOAT(IA-1))/1.0D0
            DO 30 IB=3,21,2
              B=DBLE(FLOAT(IB-1))/1.0D0+REL
              CALL INTEGI9(A,B,T,KODE,ANSI9,IERR9)
              CALL I9QUAD(A,B,T,KODE,QANS,IERR)
              ERR=DABS(QANS-ANSI9)
              IF(QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE(7,100) A,B,T,QANS,ERR,IERR,IERR9
 100          FORMAT(3D12.4,D13.5,D12.4,2I6)
   30       CONTINUE
   20     CONTINUE
   10   CONTINUE
   5  CONTINUE
      END
      SUBROUTINE I9QUAD(A,B,T,KODE,QANS,IERR)
C     I9 BY QUADRATURE
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI9/ CA,CB,KKODE
      DATA TOL/0.50D-14/
      EXTERNAL DQI9
      CA=A
      CB=B
      KKODE=KODE
      IF(KODE.EQ.1) THEN
        XA=MIN(A,B)
        IF((XA.EQ.0.0D0).OR.(T.EQ.0.0D0)) THEN
          QANS=0.0D0
          IERR=1
          RETURN
        ENDIF      
        WSM=MIN(6.0D0/A,6.0D0/B)
        WBM=MAX(6.0D0/A,6.0D0/B)
        IF(T.LE.WSM) THEN
C         INTERVAL (0,T]
          MMQ=INT(SNGL(T))+2
          MQ=MIN(MMQ,20)
          SIG=T/DBLE(FLOAT(MQ))
          INIT=-MQ
          X1=0.0D0
          REL=TOL
          CALL DQUAD8(DQI9,INIT,X1,SIG,REL,X2,QANS,IERR)
        ELSE
          IF(T.LE.WBM) THEN
C           INTERVAL (0,WSM]
C           MAKE NUMBER OF INTERVALS OF LENGTH SIG INCREASE WITH TOTAL
C           QUADRATURE LENGTH
            MMQ=INT(SNGL(WSM))+2
            MQ=MIN(MMQ,20)
            SIG=WSM/DBLE(FLOAT(MQ))
            INIT=-MQ
            X1=0.0D0
            REL=TOL
            CALL DQUAD8(DQI9,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=ANS
C           INTERVAL (WSM,WBM]
            XL=T-WSM
            MMQ=INT(SNGL(XL))+2
            MQ=MIN(MMQ,20)
            SIG=XL/DBLE(FLOAT(MQ))
C           SKIP QUADRATURE IF INTERVAL IS TOO SMALL FOR AN ACCURATE
C           QUADRATURE. AVOIDS AN ERROR MESSAGE FROM DGAUS8
            XMID=0.5D0*(T+WSM)
            FI2=DQI9(XMID)
            IF(FI2*SIG.LE.2.0D0*REL*QANS) GOTO 51
            INIT=-MQ
            X1=WSM
            REL=TOL
            CALL DQUAD8(DQI9,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=QANS+ANS
 51         CONTINUE
          ELSE
C           INTERVAL (0,WSM]
            MMQ=INT(SNGL(WSM))+2
            MQ=MIN(MMQ,20)
            SIG=WSM/DBLE(FLOAT(MQ))
            INIT=-MQ
            X1=0.0D0
            REL=TOL
            CALL DQUAD8(DQI9,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=ANS
C           INTERVAL (WSM,WBM]
            XL=WBM-WSM
            MMQ=INT(SNGL(XL))+2
            MQ=MIN(MMQ,20)
            SIG=XL/DBLE(FLOAT(MQ))
            XMID=0.5D0*(WSM+WBM)
            FI2=DQI9(XMID)
            IF(FI2*SIG.LE.2.0D0*REL*QANS) GOTO 52
            INIT=-MQ
            X1=WSM
            REL=TOL
            CALL DQUAD8(DQI9,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=QANS+ANS
 52         CONTINUE
C           INTERVAL (WBM,T]
            XL=T-WBM
            MMQ=INT(SNGL(XL))+2
            MQ=MIN(MMQ,20)
            SIG=XL/DBLE(FLOAT(MQ))
            XMID=0.5D0*(T+WBM)
            FI2=DQI9(XMID)
            IF(FI2*SIG.LE.2.0D0*REL*QANS) GOTO 53
            INIT=-MQ
            X1=WBM
            REL=TOL
            CALL DQUAD8(DQI9,INIT,X1,SIG,REL,X2,ANS,IERR)
            QANS=QANS+ANS
 53         CONTINUE
          ENDIF
        ENDIF
      ELSE
        INIT=0
        X1=T
        A2=A*A
        B2=B*B
        SIG=6.0D0/DSQRT(A2+B2)
        REL=TOL
        CALL DQUAD8(DQI9,INIT,X1,SIG,REL,X2,QANS,IERR)
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQI9(T)
C
C     DQI9 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI9/ CA,CB,KKODE
      ARG=CA*T
      ERFA=DRERF(ARG,KKODE,NZ)
      ARG=CB*T
      ERFB=DRERF(ARG,KKODE,NZ)
      DQI9=T*ERFA*ERFB
      RETURN
      END      
      PROGRAM I3COMP
C
C     DONALD E. AMOS    AUGUST, 2000;   MARCH, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 7
C
C     ON KODE=1, I3COMP COMPARES SERIES FROM INTEGI3 AND DGAUS8
C     QUADRATURE EVALUATIONS OF
C
C     I3=INT ON (T,INF) OF EXP(-C^2*W^2)*ERF(A*W)*ERF(B*W). 
C
C     ON KODE=2, I3COMP COMPARES SERIES FROM INTEGI3 AND DQUAD8
C     QUADRATURE EVALUATIONS OF
C
C     I3C=INT ON (T,INF) OF EXP(-C^2*W^2)*ERFC(A*W)*ERFC(B*W) 
C
C     WHERE C>0 AND PARAMETERS A AND B ARE NON-NEGATIVE.
C
C     OUTPUT IS WRITTEN TO FILE I3COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I3COMP.TXT")
C
      DO 5 KODE=1,2
      WRITE(7,105)KODE
 105  FORMAT('KODE = ',I2)
      WRITE (7,110)
 110  FORMAT('      A         B          C          T         QUAD
     &REL ERR  IERR KFORM')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/4.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/4.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC))/2.0D0
              CALL INTEGI3(A,B,C,T,KODE,ANS3,I3ERR,KFORM)
              IF((I3ERR.NE.0)) THEN
                WRITE (7,405) A,B,C,T,I3ERR,KFORM
 405            FORMAT( 4D11.4,1X,'IN INTEGI3, I3ERR=',I1,' KFORM=',I1)
                GOTO 40
              ENDIF
C----------------------------------------------------------------------
C     KFORM IS A RETURN VARIABLE TELLING WHICH FORMULA WAS USED IN THE
C     COMPUTATION OF ANSI3 WHEN IERR=0:
C   
C     KFORM=0 MEANS EITHER A OR B WAS ZERO. ANSI3=0.0D0 RETURNED
C     KFORM=1 MEANS THE ENHANCEMENT (A)- ERF(*)=1, BOTH AT AND BT.GE.6
C     KFORM=2 MEANS THE ENHANCEMENT (B)- POWER SERIES AT,BT,CT.LE.2.0D0
C     KFORM=3 MEANS THE ENHANCEMENT (C)- LARGE C FORMULA
C     KFORM=4,5 MEANS MAIN FORMULA CASE I  FOR C.LE.XA.LE.XB
C     KFORM=6,7 MEANS MAIN FORMULA CASE I  FOR XA.LE.C.LE.XB
C     KFORM=8   MEANS MAIN FORMULA CASE II FOR XA.LE.XB.LE.C
C               WHERE XA=MIN(A,B) AND XB=MAX(A,B)
C-----------------------------------------------------------------------
            CALL I3QUAD(A,B,C,T,KODE,QANS,IERR)
            ERR=DABS(ANS3-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,C,T,QANS,ERR,IERR,KFORM
 200        FORMAT(4D11.4,D12.5,D12.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
   5  CONTINUE
      END
      SUBROUTINE I3QUAD(A,B,C,T,KODE,QANSI3,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFI3/ AA,BB,CC,C2,KKODE
      DATA RTPI       /1.772453850905516D0/
      EXTERNAL DQI3
      IERR=0
      AA=A
      BB=B
      CC=C
      C2=C*C
      KKODE=KODE
      IF(KODE.EQ.1) THEN
        JD=50
        XA=MIN(A,B)
        QANS=0.0D0
        IF(XA.EQ.0.0D0) THEN           
           QANSI3=0.0D0
           RETURN
        ENDIF
C       SEE FOLDER 7d FOR THE METHOD OF INTEGRATION
        WSM=MIN(6.0D0/A,6.0D0/B)
        WBM=MAX(6.0D0/A,6.0D0/B)
        IF(T.LE.WSM) THEN
          SIGC=3.0D0/C
          SIG=MIN(SIGC,0.455D0*WSM)
          IFLAG=0
          X2=T
          DO 50 J=1,JD
            X1=X2
            X2=X1+SIG
            REL=0.50D-14
            IF(X2.GE.WSM) THEN
              X2=WSM
              IFLAG=1
            ENDIF
            CALL DGAUS8(DQI3,X1,X2,REL,ANS,IERRQ)
            QANS=QANS+ANS
            IF(IERRQ.EQ.1)THEN
              IERR=0
            ELSE
              IERR=IERRQ
            ENDIF
            IF(DABS(ANS).LE.REL*DABS(QANS)) THEN
              QANSI3=QANS
              RETURN
            ENDIF
            IF(IFLAG.EQ.1) GOTO 55
 50       CONTINUE
          WRITE (7,300) JD
 300      FORMAT('NO CONVERGENCE FROM DGAUS8 LOOP IN I3COMP AFTER',I3,
     &' STEPS')        
  55      CONTINUE
          CALL INTEGJ5(C,XA,WSM,ANS1,IERR5)
          CALL INTEGI5(C,XA,WBM,ANS2,IERR5)
          SS=ANS1+ANS2
          QANSI3=QANS+SS
        ELSE
          IF(T.LE.WBM) THEN
            CALL INTEGJ5(C,XA,T,ANS1,IERR5)
            CALL INTEGI5(C,XA,WBM,ANS2,IERR5)
            SS=ANS1+ANS2
            QANSI3=SS
          ELSE
            IKODE=2
            CT=C*T
            ERFC=DRERF(CT,IKODE,NZ)
            QANSI3=0.5D0*RTPI*ERFC/C
          ENDIF
        ENDIF
      ELSE
        INIT=0
        X1=T
        SIG=3.0D0/DSQRT(A*A+B*B+C*C)
        REL=0.5D-14
        CALL DQUAD8(DQI3,INIT,X1,SIG,REL,X2,QANS,IERR)
        QANSI3=QANS
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQI3(T)
C
C     DQI3 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I3 OR I3C IN I3COMP
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI3/ AA,BB,CC,C2,KKODE
      ARG=BB*T
      NZ=0
      ERFB=DRERF(ARG,KKODE,NZ)
      ARG=AA*T
      ERFA=DRERF(ARG,KKODE,NZ)
      T2=T*T
      ARG=C2*T2
      DQI3=ERFA*ERFB*DEXP(-ARG)
      RETURN
      END
      PROGRAM I5COMP
C
C     DONALD E. AMOS    AUGUST, 2000
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------

C     REF: FOLDER 5
C
C     I5COMP COMPARES THE SERIES FROM INTEGI5 AND DGAUS8 QUADRATURE
C     EVALUATIONS OF
C
C            I5=INT ON (X,INF) OF  EXP(-A^2*T^2)*ERFC(B*T)
C
C     WHERE ALL PARAMETERS ARE NONNEGATIVE WITH A AND B NOT ZERO AT THE
C     SAME TIME.
C
C     OUTPUT IS WRITTEN TO FILE I5COMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFI5/ AA,BB
      EXTERNAL DQI5
      OPEN(UNIT=7,FILE="I5COMP.TXT")
      WRITE (7,110)
 110  FORMAT('      A           B           X         SERIES        QUAD
     &        REL ERR IERR')
      DO 10 IX=1,20,2
        X=DBLE(FLOAT(IX-1))/1.00D0 
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/10.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/5.0D0
            CALL INTEGI5(A,B,X,ANS5,I5ERR)
            IF ((I5ERR.NE.0)) THEN
              WRITE (7,405) A,B,X,I5ERR
 405          FORMAT(3D12.4,'  FROM INTEGI5, ERROR FLAG = ',I3)
              GOTO 30
            ENDIF
            X1=X
            SIG=1.5D0/MAX(A,B)
            AA=A*A
            BB=B
            REL=1.0D-14
            INIT=0
            CALL DQUAD8(DQI5,INIT,X1,SIG,REL,X2,QANS,IERR)
            ERR=DABS(ANS5-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,X,ANS5,QANS,ERR,IERR
  200       FORMAT(3D12.4,2D13.5,D12.4,I3)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQI5(T)
C
C     DQI5 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I5 IN I5COMP
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI5/ AA,BB
      ARG=BB*T
      KODE=2
      NZ=0
      ERF2=DRERF(ARG,KODE,NZ)
      ARG=-AA*T*T
      DQI5=ERF2*DEXP(ARG)
      RETURN
      END
      PROGRAM J5COMP
C
C     DONALD E. AMOS  AUGUST, 2000
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 5
C
C     J5COMP COMPARES THE SERIES FROM INTEGJ5 AND DGAUS8 QUADRATURE
C     EVALUATIONS OF
C
C        INT ON (X,INF) OF  EXP(-A*A*T*T)*ERF(B*T)
C
C     WHERE A.GT.0.0D0, B.GE.0.0D0 AND T.GE.0.0D0
C
C     OUTPUT IS WRITTEN TO FILE J5COMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFJ5/ AA,BB
      EXTERNAL DQJ5
      OPEN(UNIT=7,FILE="J5COMP.TXT")
      WRITE (7,110)
 110  FORMAT('      A           B           X         SERIES        QUAD
     &        REL ERR JERR')
      DO 10 IX=1,10
        X=DBLE(FLOAT(IX-1))/1.0D0
        DO 20 IA=1,21,2
          A=DBLE(FLOAT(IA))/5.0D0
          DO 30 IB=1,21,2
            B=DBLE(FLOAT(IB))/5.0D0
            CALL INTEGJ5(A,B,X,ANS5,JERR)
            JD=30
            X2=X
            SIG=3.0D0/MAX(A,B)
            AA=A*A
            BB=B
            QANS=0.0D0
            REL=1.0D-14
            IFLAG=0
            DO 50 J=1,JD
              X1=X2
              X2=X2+SIG
              CALL DGAUS8(DQJ5,X1,X2,REL,ANS,IERR)
              QANS=QANS+ANS
              ERR=DABS(ANS)
              IF (ERR.LE.REL*DABS(QANS)) GOTO 55
              IF (IFLAG.EQ.0) THEN
                IF(B*X2.GT.3.0D0) THEN
                  SIG=3.0D0/A
                  IFLAG=1
                ENDIF
              ENDIF
  50        CONTINUE
            WRITE (7,300)
 300        FORMAT('NO ANSWER FROM DGAUS8 AFTER 30 STEPS')
  55        CONTINUE
            IF (QANS.NE.0.0D0) THEN
              ERR=DABS(QANS-ANS5)/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,X,ANS5,QANS,ERR,JERR
 200        FORMAT(3D12.4,2D13.5,D12.4,I3)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQJ5(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ5/ AA,BB
      ARG=BB*T
      KODE=1
      NZ=0
      ERF2=DRERF(ARG,KODE,NZ)
      ARG=-AA*T*T
      DQJ5=ERF2*DEXP(ARG)
      RETURN
      END
      PROGRAM V5COMP
C
C     DONALD E. AMOS      JANUARY, 2003; APRIL 2006
C
C     REF: FOLDER 5
C
C     V5COMP COMPARES THE SERIES FROM INTEGV5 AND DGAUS8 QUADRATURE
C     EVALUATIONS OF
C
C            V5 = INT ON (0,X) OF  EXP(-A^2*W^2)*ERF(B*W)
C
C     WHERE A.GE.0 , B.GE.0, X.GE.0
C
C     OUTPUT IS WRITTEN TO FILE V5COMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFV5/ AA,BB
      OPEN(UNIT=7,FILE="V5COMP.TXT")
      WRITE (7,110)
 110  FORMAT('      A           B           X          QUAD        REL E
     &RR   IERR IERR5')
      DO 10 IX=1,41,2
        X=DBLE(FLOAT(IX-1))/5.0D0
        DO 20 IA=1,81,8
          A=DBLE(FLOAT(IA-1))/10.0D0
          DO 30 IB=1,81,8
            B=DBLE(FLOAT(IB-1))/2.0D0
            CALL INTEGV5(A,B,X,ANS5,IERR5)
            CALL V5QUAD(A,B,X,QANS,IERR)
            ERR=DABS(ANS5-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,X,QANS,ERR,IERR,IERR5
 200        FORMAT(3D12.4,D13.5,D12.4,2I5)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      SUBROUTINE V5QUAD(A,B,T,QANS,IERR)
C     V5 BY QUADRATURE
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION MQ(2),XX(2),SS(2)
      COMMON/ CFV5/ AA,BB
      DATA TOL/0.5D-14/
      EXTERNAL DQV5
      IERR=0
      IF(B.EQ.0.0D0) THEN
        QANS=0.0D0
        IERR=1
        RETURN
      ENDIF
C     SET COMMON VARIABLES
      AA=A*A
      BB=B
      ABM=MAX(A,B)
      IF(ABM.NE.0.0D0) THEN
        SIGM=6.0D0/ABM
C       IF SIGM IS SMALL, MOST OF THE AREA COULD BE NEAR T=0 OR A LARGE
C       GRADIENT COULD OCCUR AND A REFINEMENT NEAR T=0 MAKES
C       THE COMPUTATION BETTER OVER A WIDER RANGE OF VARIABLES.
        IF(SIGM.LT.0.25D0*T) THEN
          JD=2
          MMQ=2
          MQ(1)=MMQ
          XX(1)=0.0D0
          SS(1)=SIGM/DBLE(FLOAT(MMQ))
          TL=T-SIGM
          MMQ=INT(SNGL(TL/2.0D0))+2
          MQ(2)=MMQ
          XX(2)=SIGM
          SS(2)=TL/DBLE(FLOAT(MMQ))
        ELSE
          JD=1
          MMQ=INT(SNGL(T/2.0D0))+2
          MQ(1)=MMQ
          XX(1)=0.0D0
          SS(1)=T/DBLE(FLOAT(MMQ))          
        ENDIF
      ELSE
C       THE INTEGRAND IS ZERO HERE FOR MAX(A,B)=0.0D0
        QANS=0.0D0
        IERR=1
        RETURN
      ENDIF
      QANS=0.0D0
      DO 10 J=1,JD
        INIT=-MQ(J)
        X1=XX(J)
        SIG=SS(J)
        REL=TOL
        CALL DQUAD8(DQV5,INIT,X1,SIG,REL,X2,ANS,IERR)
        QANS=QANS+ANS
 10   CONTINUE
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQV5(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFV5/ AA,BB
      ARG=BB*T
      KODE=1
      NZ=0
      ERF2=DRERF(ARG,KODE,NZ)
      ARG=-AA*T*T
      DQV5=ERF2*DEXP(ARG)
      RETURN
      END
      PROGRAM I6COMP
C
C     DONALD E. AMOS    DECEMBER, 2001; FEBRUARY, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     ON KODE=1, I6COMP COMPUTES THE INTEGRAL
C
C            INT ON (T,INF) OF ERF(A*W)*ERF(B*W)/W**2,
C
C     ON KODE=2, I6COMP COMPUTES THE INTEGRAL
C
C            INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)/W**2
C
C     FOR A, B, T NON-NEGATIVE. T=0 IS NOT PERMITTED FOR KODE=2.
C
C     BY QUADRATURE (I6QUAD) AND BY SERIES (INTEGI6) AND PRINTS
C     THE RELATIVE ERROR. FORMULAE ARE CONTAINED IN FOLDER 3 FOR
C     THE (REFINED) QUADRATURE PROCEDURE AND FOLDERS 6 AND 15 FOR THE
C     SERIES.
C
C     OUTPUT IS WRITTEN TO FILE I6COMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      EXTERNAL FINTI6
      OPEN(UNIT=7,FILE="I6COMP.TXT")
      DO 5 KODE=1,2
        WRITE(7,201) KODE
 201    FORMAT('                           KODE = ', I2)
        WRITE(7,200)
 200    FORMAT('     A          B          T         QUAD       SERIES 
     &    REL ERR   IERR')
        DO 10 IA=6,41,5
          A=DBLE(FLOAT(IA-1))/10.0D0
C          A=0.001D0+0.05D0*DBLE(FLOAT(IA-1))
C          A=100.0D0/10.0D0**(IA-1)
          DO 20 IB=6,41,5
            B=DBLE(FLOAT(IB-1))/10.0D0
C         B=0.001D0+0.05D0*DBLE(FLOAT(IB-1))
C         B=100.0D0/10.0D0**(IB-1)
            DO 30 IT=1,26
              T=0.0D0+0.2D0*DBLE(FLOAT(IT-1))
              CALL INTEGI6(A,B,T,KODE,SS,JERR)
              REL=0.5D-14
              CALL I6QUAD(A,B,T,REL,KODE,ANS,IERR)
              ERR=DABS(SS-ANS)
              IF(ANS.NE.0.0D0) THEN
                ERR=ERR/DABS(ANS)
              ENDIF
              WRITE(7,100) A,B,T,ANS,SS,ERR,JERR
  100         FORMAT(3D11.4,3D12.5,I3)
  30        CONTINUE
  20      CONTINUE
  10    CONTINUE
  5   CONTINUE
      END
      SUBROUTINE I6QUAD(A,B,X,REL,KODE,ANS,IERR)
C
C     REF.: REFINED QUADRATURE OF FOLDER 3     
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      EXTERNAL FINTI6
      COMMON /CFINTI6/AA,BB,KKODE
      AA=A
      BB=B
      KKODE=KODE
      IF(KODE.EQ.1) THEN
        XAB=MIN(A,B)
        WSM=MIN(6.0D0/A,6.0D0/B)
        WLM=MAX(6.0D0/A,6.0D0/B)
        JL=1
        SS=0.0D0
        IF (X.LE.WSM) THEN
          W2=X
          JL=5
          DW=(WSM-X)/DBLE(FLOAT(JL))
          DO 42 L=1,JL
            W1=W2
            W2=W1+DW
            TOL=REL
            CALL DGAUS8(FINTI6,W1,W2,TOL,QANS,IERR)
            SS=SS+QANS
   42     CONTINUE
          SS=SS+QPI6(XAB,WSM,WLM,REL)
        ELSE
          IF (X.LE.WLM)  THEN
            SS=QPI6(XAB,X,WLM,REL)
          ELSE
            SS=1.0D0/X
          ENDIF
        ENDIF
        ANS=SS
        RETURN
      ELSE
        IF(X.EQ.0.0D0) THEN
          ANS=0.0D0
          IERR=2
          RETURN
        ENDIF
        INIT=0
        X1=X
        SIG=3.0D0/DSQRT(A*A+B*B)
        TOL=REL
        CALL DQUAD8(FINTI6,INIT,X1,SIG,TOL,X2,QANS,IERR)
        CALL DQUAD8(FINTI6,INIT,X1,SIG,TOL,X2,QANS,IERR)
        ANS=QANS
        RETURN
      ENDIF
      END
      DOUBLE PRECISION FUNCTION FINTI6(W)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFINTI6/AA,BB,KKODE
      ARG=AA*W
      ERF1=DRERF(ARG,KKODE,NZ)/W
      ARG=BB*W
      ERF2=DRERF(ARG,KKODE,NZ)/W
      FINTI6=ERF1*ERF2
      RETURN
      END
      PROGRAM W3COMP
C
C     DONALD E. AMOS   OCTOBER, 2001; MARCH, 2003 
C
C     REF: FOLDER1, FOLDER2, FOLDER 10
C
C     W3COMP COMPARES THE CLOSED FORM OF W3 OF FOLDER 10 WITH A
C     REFINED QUADRATURE FOR W3 IN THE MANNER OF FOLDER 3. ON KODE=1,
C
C          W3=INT ON (T,INF) OF ERF(A*W)*ERF(B*W)/W**3.
C
C     ON KODE=2, THE COMPLEMENTARY FUNCTION IS COMPUTED
C
C          W3C=INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)/W**3
C
C     WHERE A.GE.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0. THE CLOSED FORMS ARE
C     IMPLEMENTED IN DOUBLE PRECISION SUBROUTINE INTEGW3.
C
C     OUTPUT IS WRITTEN TO FILE W3COMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="W3COMP.TXT")
      WRITE (7,110)
 110  FORMAT('       A          B           T         QUAD          ERR'
     *)
      KODE=1
      WRITE(7,115) KODE
 115  FORMAT(/,2X,'THE FOLLOWING COMPARISONS ARE FOR KODE = ',I3/) 
      DO 10 IT=1,111,10
        T=DBLE(FLOAT(IT))/25.0D0
        DO 20 IA=1,111,10
          A=DBLE(FLOAT(IA-1))/25.0D0
          DO 30 IB=1,111,10
            B=DBLE(FLOAT(IB-1))/25.0D0
            REL=1.0D-14
            CALL INTEGW3(A,B,T,KODE,REL,ANS1,IERR2)
            REL=1.0D-14
            IF(KODE.EQ.1) THEN
              CALL W3BYQ1(A,B,T,REL,QANS,IERR)
            ELSE
              CALL W3BYQ2(A,B,T,REL,QANS,IERR)
            ENDIF
  40        CONTINUE
            ERR=DABS(QANS-ANS1)
            IF(QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE(7,100) A,B,T,QANS,ERR
 100        FORMAT(3D12.4,D13.5,D12.4)
 101        FORMAT(3I5)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      SUBROUTINE W3BYQ1(A,B,X,REL,ANS,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CQFW3/AA,BB,KKODE
      EXTERNAL DQFW3
C     REFINED QUADRATURE OF FOLDER 3
      IF((A.EQ.0.0D0).OR.(B.EQ.0.0D0)) THEN
        ANS=0.0D0
        RETURN
      ENDIF
      AA=A
      BB=B
      KKODE=1
      XAB=MIN(A,B)
      WSM=MIN(6.0D0/A,6.0D0/B)
      WLM=MAX(6.0D0/A,6.0D0/B)
      JL=1
      SS=0.0D0
      IF (X.LE.WSM) THEN
        W2=X
        JL=5
        DW=(WSM-X)/DBLE(FLOAT(JL))
        DO 42 L=1,JL
          W1=W2
          W2=W1+DW
          CALL DGAUS8(DQFW3,W1,W2,REL,QANS,IERR)
          SS=SS+QANS
   42   CONTINUE
        SS=SS+PW3(XAB,WSM,WLM)+0.50D0/(WLM*WLM)
      ELSE
        IF (X.LE.WLM)  THEN
          SS=PW3(XAB,X,WLM)+0.50D0/(WLM*WLM)
        ELSE
          SS=0.50D0/(X*X)
        ENDIF
      ENDIF
      ANS=SS
      RETURN
      END
      DOUBLE PRECISION FUNCTION PW3(XAB,FL,FU)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      T1=XAB*FL
      T2=XAB*FU
      KODE=1
      NZ=0
      TRM1=DRERF(T1,KODE,NZ)/FL
      KODE=1
      NZ=0
      TRM2=DRERF(T2,KODE,NZ)/FU
      S=0.5D0*(TRM1/FL-TRM2/FU)
      KODE=1
      CALL DIERFC(T1,KODE,ANSL,IERR)
      CALL DIERFC(T2,KODE,ANSU,IERR)
      PW3=S+(ANSL/FL-ANSU/FU)*XAB
      RETURN
      END
      SUBROUTINE W3BYQ2(A,B,X,REL,ANS,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CQFW3/AA,BB,KKODE
      EXTERNAL DQFW3
C     REFINED QUADRATURE OF FOLDER 3
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
        ANS=0.50D0/(X*X)
        RETURN
      ENDIF
      KODE=2
      AS=A*A
      BS=B*B
C     SET COMMON PARAMETERS
      AA=A
      BB=B
      KKODE=KODE
      INIT=0
      X1=X
      SIG=2.0D0/DSQRT(AS+BS)
      TOL=REL
      CALL DQUAD8(DQFW3,INIT,X1,SIG,TOL,X2,QANS,IERR)
      ANS=QANS
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQFW3(W)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CQFW3/AA,BB,KKODE
      ARG=AA*W
      ERF1=DRERF(ARG,KKODE,NZ)/W
      ARG=BB*W
      ERF2=DRERF(ARG,KKODE,NZ)/W
      DQFW3=ERF1*ERF2/W
      RETURN
      END
      PROGRAM PCOMP
C
C     DONALD E. AMOS    AUGUST, 2000;  JANUARY, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 11
C
C     PCOMP COMPUTES THE INTEGRAL
C
C       P  =  INT ON (T,INF) OF EXP(-(A*W)^2)*ERF(B*W)/W   ON KODE=1
C
C     AND THE COMPLEMENTARY INTEGRAL
C
C       PC =  INT ON (T,INF) OF EXP(-(A*W)^2)*ERFC(B*W)/W  ON KODE=2
C 
C     BY QUADRATURE AND BY SERIES AND PRINTS THE RELATIVE ERROR.
C     OUTPUT IS WRITTEN TO FILE PCOMP.TXT 
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="PCOMP.TXT")
C
      DO 5 KODE=1,2
      WRITE(7,201) KODE
 201  FORMAT(/,32X,'  KODE = ',I3)
      WRITE(7,200)
 200  FORMAT('     A          B          T         QUAD       SERIES   
     C  REL ERR JERR IERR')
      DO 10 IA=1,31,5
         A=DBLE(FLOAT(IA-1))/10.0D0
        DO 20 IB=1,31,5
          B=DBLE(FLOAT(IB-1))/10.0D0
          DO 30 IT=1,31,2
            T=0.0D0+0.10D0*DBLE(FLOAT(IT-1))
            REL=0.5D-14
            CALL INTEGP(A,B,T,KODE,REL,SS,JERR)
            IF((JERR.NE.0).AND.(JERR.NE.2)) THEN
              SS=0.0D0
              QANS=0.0D0
              IERR=8
              GOTO 40
            ENDIF
            CALL PQUAD(A,B,T,REL,KODE,QANS,IERR)
  40        CONTINUE
            ERR=DABS(SS-QANS)
            IF(QANS.NE.0.0D0)THEN
              ERR=ERR/QANS
            ENDIF
            WRITE(7,100) A,B,T,QANS,SS,ERR,JERR,IERR
  100       FORMAT(3D11.4,3D12.5,2I3)
  30      CONTINUE
  20    CONTINUE
  10  CONTINUE
  5   CONTINUE
      END
      SUBROUTINE PQUAD(A,B,T,REL,KODE,ANSP,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFINTP/AA,BB,KKODE
      EXTERNAL FINTP
      IERR=1
      AA=A
      BB=B
      KKODE=KODE
      IF(KODE.EQ.1) THEN
        IF(A.EQ.0.0D0) THEN
          ANSP=0.0D0
          IERR=6
          RETURN
        ENDIF
        IF(B.EQ.0.0D0) THEN
          ANSP=0.0D0
          RETURN
        ENDIF
        WSM=6.0D0/B
        X2=T
        QANS=0.0D0
        DO 10 K=1,50
          X1=X2
          IF(X1.LT.WSM) THEN
            SIG=0.25D0*WSM
          ELSE
            SIG=3.0D0/A
          ENDIF
          X2=X1+SIG
          TOL=REL
          CALL DGAUS8(FINTP,X1,X2,TOL,ANS,IERR)
          QANS=QANS+ANS
          IF(DABS(ANS).LT.REL*DABS(QANS)) GOTO 15
  10    CONTINUE
  15    CONTINUE
        ANSP=QANS
      ELSE
        IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
          ANSP=0.0D0
          IERR=7
          RETURN
        ENDIF
        IF(T.EQ.0.0D0) THEN
          IERR=9
          ANSP=0.0D0
          RETURN
        ENDIF
        A2=A*A
        B2=B*B
        INIT=0
        X1=T
        SIG=6.0D0/DSQRT(A2+B2)
        PREL=REL
        CALL DQUAD8(FINTP,INIT,X1,SIG,PREL,X2,QANS,IERR)
        ANSP=QANS
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION FINTP(W)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFINTP/AA,BB,KKODE
      ARG=BB*W
      ERFB=DRERF(ARG,KKODE,NZ)
      ARG=AA*W
      ARG2=ARG*ARG
      FINTP=DEXP(-ARG2)*ERFB/W
      RETURN
      END
      PROGRAM QCOMP
C
C     DONALD E. AMOS    AUGUST, 2000; JANUARY, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF. FOLDER 11, FOLDER 6
C
C     QCOMP COMPUTES THE INTEGRAL
C
C       Q =  INT ON (T,INF) OF EXP(-(A*W)^2)*E(1,(B*W)^2)    
C
C     BY QUADRATURE AND BY SERIES AND PRINTS THE RELATIVE ERROR.
C     OUTPUT IS WRITTEN TO FILE QCOMP.TXT.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="QCOMP.TXT")
      WRITE(7,200)
 200  FORMAT('     A          B          T         QUAD       SERIES   
     C  REL ERR JERR IERR')
      DO 10 IA=1,31,5
        A=DBLE(FLOAT(IA-1))/10.0D0
        DO 20 IB=1,31,5
          B=DBLE(FLOAT(IB-1))/10.0D0
          DO 30 IT=1,40,2
            T=0.0D0+0.10D0*DBLE(FLOAT(IT))
            REL=0.5D-14
            CALL INTEGQ(A,B,T,REL,SS,JERR)
            IF((JERR.NE.0).AND.(JERR.NE.2)) THEN
              SS=0.0D0
              QANS=0.0D0
              IERR=8
              GOTO 40
            ENDIF
            CALL QQUAD(A,B,T,REL,QANS,IERR)
  40        CONTINUE
            IF(IERR.NE.1) THEN
              QANS=0.0D0
            ENDIF
            ERR=DABS(SS-QANS)
            IF(QANS.NE.0.0D0)THEN
              ERR=ERR/QANS
            ENDIF
            WRITE(7,100) A,B,T,QANS,SS,ERR,JERR,IERR
  100       FORMAT(3D11.4,3D12.5,2I3)
  30      CONTINUE
  20    CONTINUE
  10  CONTINUE
      END
      SUBROUTINE QQUAD(A,B,T,REL,ANSQ,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFINTQ/AA,BB,CREL
      EXTERNAL FINTQ
      AA=A
      BB=B
      CREL=REL
      A2=A*A
      B2=B*B
      INIT=0
      X1=T
      SIG=4.0D0/DSQRT(A2+B2)
      QREL=REL
      CALL DQUAD8(FINTQ,INIT,X1,SIG,QREL,X2,QANS,IERR)
      SIG=SIG+SIG
      CALL DQUAD8(FINTQ,INIT,X1,SIG,QREL,X2,QANS,IERR)
      ANSQ=QANS
      RETURN
      END
      DOUBLE PRECISION FUNCTION FINTQ(W)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      COMMON /CFINTQ/AA,BB,CREL
      ARG=BB*W
      ARG2=ARG*ARG
      N=1
      KKODE=1
      M=1
      TOL=CREL
      CALL DEXINT(ARG2, N, KKODE, M, TOL, EN, NZ, IERR)
      ARG=AA*W
      ARG2=ARG*ARG
      FINTQ=DEXP(-ARG2)*EN(1)
      RETURN
      END
      PROGRAM I21COMP
C
C     DONALD E. AMOS   OCTOBER, 2002; MAY, 2006
C
C     REF: FOLDER 21
C
C     ON KODE=1, I21COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGI21
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          I21=INT ON (0,T) OF EXP(A^2*W+2*A*B)*ERFC(X)*ERF(C/RW)
C
C     ON KODE=2, I21COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGI21
C     AND DGAUS8 QUADRATURE EVALUATIONS OF THE COMPLEMENTARY FUNCTION
C
C          I21=INT ON (0,T) OF EXP(A^2*W+2*A*B)*ERFC(X)*ERFC(C/RW)
C              WITH X=A*RW+B/RW AND RW=SQRT(W)
C
C     AND A, B AND C ARE POSITIVE.
C
C     OUTPUT IS WRITTEN TO FILE I21COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I21COMP.TXT")
      WRITE (7,109)
 109  FORMAT('----------------------------------------------------------
     &-----------------'/
     &'LEGENDS FOR KFORM:'//
     &'KFORM DIGIT=0 MEANS ONE OR MORE PARMETERS ARE ZERO--A SPECIAL FOR
     &MULA IS USED'/
     &'KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0'/
     &'KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT'/
     &'KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3'/
     &'KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21'/
     &'KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3'//
     &'EXAMPLE:'/
     &'KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED'/
     &'-----------------------------------------------------------------
     &----------')
C
      DO 5 KODE=1,2
C
      WRITE (7,105) KODE
 105  FORMAT(/'THE FOLLOWING TABLE OF COMPARISONS IS FOR KODE = ',I3/)
      WRITE (7,110)
 110  FORMAT('      A         B         C          T         QUAD
     &REL ERR  IERR KFORM')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/10.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC-1))/5.0D0
              CALL INTEGI21(A,B,C,T,KODE,ANS2,I2ERR,KFORM)
              IF((I2ERR.NE.0)) THEN
                WRITE (7,405) A,B,C,T,I2ERR,KFORM
 405            FORMAT( 4D11.4,1X,'IN INTEGI21, I2ERR=',I2,' KFORM=',I2)
                GOTO 40
              ENDIF
C-----------------------------------------------------------------------
C     LEGENDS FOR KFORM:
C
C     KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0
C     KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3
C     KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3
C
C     EXAMPLE:
C     KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED
C-----------------------------------------------------------------------
              CALL I21QUAD(A,B,C,T,KODE,QANS,IERR)              
              ERR=DABS(ANS2-QANS)
              IF (QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE (7,200) A,B,C,T,QANS,ERR,IERR,KFORM
 200          FORMAT(4D11.4,D12.5,D12.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
   5  CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQI21(X)
C
C     DQI21 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I21 WITH INTEGI21
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI21/ CA,CB,CC,IKODE
C     A CHANGE OF VARIABLE T=X*X REMOVES THE SQRT FROM THE INTEGRAND
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T)
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFX=ERFX*DEXP(-ARG)
      ARG=CC/X
      NZ=0
      ERFA=DRERF(ARG,IKODE,NZ)
      DQI21=2.0D0*X*ERFX*ERFA
      RETURN
      END
      SUBROUTINE I21QUAD(A,B,C,T,KODE,QANS21,IERR)
C     QUADRATURE FOR THE I21 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI21/ CA,CB,CC,IKODE
      DATA TOL /0.5D-14/
      EXTERNAL DQI21
      IERR=0
      IF((C.EQ.0.0D0).AND.(KODE.EQ.1)) THEN
        QANS21=0.0D0
        IERR=1
        RETURN
      ENDIF
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
C       KODE=2 HERE BECAUSE C=0 AND THE PREVIOUS TEST FAILED
        QANS21=T
        IERR=1
        RETURN
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANS21=0.0D0
        IERR=1
        RETURN
      ENDIF
      IF(A.EQ.0.0D0) THEN
        IF(KODE.EQ.1) THEN
          XLIM=MIN(B,C)/6.0D0
        ELSE
          XLIM=DSQRT(B*B+C*C)/6.0D0
        ENDIF
      ELSE
        IF(B.EQ.0.0D0) THEN
          XLIM=C/6.0D0
        ELSE
          IF(KODE.EQ.1) THEN
            XLIM=MIN(B,C)/6.0D0
          ELSE
            XLIM=DSQRT(B*B+C*C)/6.0D0
          ENDIF
        ENDIF
      ENDIF
      CA=A
      CB=B
      CC=C
      IKODE=KODE
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI21,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI21,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS        
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQI21(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQI21,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS21=SS
      RETURN
      END
      PROGRAM J21COMP
C
C     DONALD E. AMOS   OCTOBER, 2002; MAY, 2006
C
C     REF: FOLDER 21
C
C     J21COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGJ21
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          J21=INT ON (0,T) OF EXP(A^2*W+2*A*B)*EXP(-C*C/W)/(W**3/2)
C
C     AND A IS NON-NGATIVE AND B AND C ARE NON-NEGATIVE, BUT NOT ZERO AT
C     THE SAME TIME.
C
C     OUTPUT IS WRITTEN TO FILE J21COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="J21COMP.TXT")
      WRITE (7,109)
 109  FORMAT('----------------------------------------------------------
     &-----------------'/
     &'LEGENDS FOR KFORM:'//
     &'KFORM DIGIT=0 MEANS ONE OR MORE PARMETERS ARE ZERO--A SPECIAL FOR
     &MULA IS USED'/
     &'KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0'/
     &'KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT'/
     &'KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3'/
     &'KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21'/
     &'KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3'//
     &'EXAMPLE:'/
     &'KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED'/
     &'-----------------------------------------------------------------
     &----------'/)
C
      WRITE (7,110)
 110  FORMAT('      A         B         C          T         QUAD
     &REL ERR  IERR KFORM')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/10.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC-1))/5.0D0
              CALL INTEGJ21(A,B,C,T,ANS2,J2ERR,KFORM)
              IF((J2ERR.NE.0)) THEN
                WRITE (7,405) A,B,C,T,J2ERR,KFORM
 405            FORMAT( 4D11.4,1X,'IN INTEGJ21, J2ERR=',I2,' KFORM=',I2)
                GOTO 40
              ENDIF
C-----------------------------------------------------------------------
C     LEGENDS FOR KFORM:
C
C     KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0
C     KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3
C     KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3
C
C     EXAMPLE:
C     KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED
C-----------------------------------------------------------------------
              CALL J21QUAD(A,B,C,T,QANS,IERR)              
              ERR=DABS(ANS2-QANS)
              IF (QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE (7,200) A,B,C,T,QANS,ERR,IERR,KFORM
 200          FORMAT(4D11.4,D12.5,D12.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQJ21(X)
C
C     DQJ21 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF J21 WITH INTEGJ21
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ21/ CA,CB,CC
C     A CHANGE OF VARIABLE T=X*X REMOVES THE SQRT FROM THE INTEGRAND
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T)
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFX=ERFX*DEXP(-ARG)
      ARG=CC*CC/X2
      DQJ21=2.0D0*ERFX*DEXP(-ARG)/X2
      RETURN
      END
      SUBROUTINE J21QUAD(A,B,C,T,QANS21,IERR)
C     QUADRATURE FOR THE J21 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ21/ CA,CB,CC
      DATA TOL /0.5D-14/
      EXTERNAL DQJ21
      IERR=0
      IF((B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
C       THE INTEGRAL DOES NOT EXIST--WRONG INPUT
        QANS21=0.0D0
        IERR=2
        RETURN
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANS21=0.0D0
        IERR=1
        RETURN
      ENDIF
      CA=A
      CB=B
      CC=C
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      XLIM=DSQRT(B*B+C*C)/6.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQJ21,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQJ21,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQJ21(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQJ21,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS21=SS
      RETURN
      END
      PROGRAM I22COMP
C
C     DONALD E. AMOS   NOVENBER, 2002; MAY, 2006
C
C     REF: FOLDER 22
C
C     I22COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGI22
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          I22=INT ON (0,T) OF EXP(A^2*W+2*A*B)*EXP(-C*C/W)/SQRT(W)
C
C     AND A IS NON-NGATIVE AND B AND C ARE NON-NEGATIVE, BUT NOT ZERO AT
C     THE SAME TIME.
C
C     OUTPUT IS WRITTEN TO FILE I22COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I22COMP.TXT")
      WRITE (7,109)
 109  FORMAT('----------------------------------------------------------
     &-----------------'/
     &'LEGENDS FOR KFORM:'//
     &'KFORM DIGIT=0 MEANS ONE OR MORE PARMETERS ARE ZERO--A SPECIAL FOR
     &MULA IS USED'/
     &'KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0'/
     &'KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT'/
     &'KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3'/
     &'KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21'/
     &'KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3'//
     &'EXAMPLE:'/
     &'KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED'/
     &'-----------------------------------------------------------------
     &----------'/)
C
      WRITE (7,110)
 110  FORMAT('      A         B         C          T         QUAD
     &REL ERR  IERR KFORM')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/10.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC-1))/5.0D0
              CALL INTEGI22(A,B,C,T,ANS2,I2ERR,KFORM)
              IF((I2ERR.NE.0)) THEN
                WRITE (7,405) A,B,C,T,I2ERR,KFORM
 405            FORMAT( 4D11.4,1X,'IN INTEGI22, I2ERR=',I2,' KFORM=',I2)
                GOTO 40
              ENDIF
C-----------------------------------------------------------------------
C     LEGENDS FOR KFORM:
C
C     KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0
C     KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3
C     KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3
C
C     EXAMPLE:
C     KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED
C-----------------------------------------------------------------------
              CALL I22QUAD(A,B,C,T,QANS,IERR)              
              ERR=DABS(ANS2-QANS)
              IF (QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE (7,200) A,B,C,T,QANS,ERR,IERR,KFORM
 200          FORMAT(4D11.4,D12.5,D12.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQI22(X)
C
C     DQI22 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I22 WITH INTEGI22
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI22/ CA,CB,CC
C     A CHANGE OF VARIABLE T=X*X REMOVES THE SQRT FROM THE INTEGRAND.
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T).
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFX=ERFX*DEXP(-ARG)
      ARG=CC*CC/X2
      DQI22=2.0D0*ERFX*DEXP(-ARG)
      RETURN
      END
      SUBROUTINE I22QUAD(A,B,C,T,QANS22,IERR)
C     QUADRATURE FOR THE I22 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI22/ CA,CB,CC
      DATA TOL /0.5D-14/
      EXTERNAL DQI22
      IERR=0
      IF((B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
        XLIM=4.0D0
      ELSE
        XLIM=DSQRT(B*B+C*C)/6.0D0
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANS21=0.0D0
        IERR=1
        RETURN
      ENDIF
      CA=A
      CB=B
      CC=C
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI22,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI22,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQI22(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQI22,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS22=SS
      RETURN
      END
      PROGRAM J22COMP
C
C     DONALD E. AMOS   NOVEMBER, 2002; MAY, 2006
C
C     REF: FOLDER 22
C
C     J22COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGJ22
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          J22=INT ON (0,T) OF EXP(A^2*W+2*A*B)*EXP(-C*C/W)*SQRT(W)
C
C     AND A IS NON-NGATIVE AND B AND C ARE NON-NEGATIVE, BUT NOT ZERO AT
C     THE SAME TIME.
C
C     OUTPUT IS WRITTEN TO FILE J22COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="J22COMP.TXT")
      WRITE (7,109)
 109  FORMAT('----------------------------------------------------------
     &-----------------'/
     &'LEGENDS FOR KFORM:'//
     &'KFORM DIGIT=0 MEANS ONE OR MORE PARMETERS ARE ZERO--A SPECIAL FOR
     &MULA IS USED'/
     &'KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0'/
     &'KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT'/
     &'KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3'/
     &'KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21'/
     &'KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3'//
     &'EXAMPLE:'/
     &'KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED'/
     &'-----------------------------------------------------------------
     &----------'/)
C
      WRITE (7,110)
 110  FORMAT('      A         B         C          T         QUAD
     &REL ERR  IERR KFORM')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/10.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC-1))/5.0D0
              CALL INTEGJ22(A,B,C,T,ANS2,J2ERR,KFORM)
              IF((J2ERR.NE.0)) THEN
                WRITE (7,405) A,B,C,T,J2ERR,KFORM
 405            FORMAT( 4D11.4,1X,'IN INTEGJ22, J2ERR=',I2,' KFORM=',I2)
                GOTO 40
              ENDIF
C-----------------------------------------------------------------------
C     LEGENDS FOR KFORM:
C
C     KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0
C     KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3
C     KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3
C
C     EXAMPLE:
C     KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED
C-----------------------------------------------------------------------
              CALL J22QUAD(A,B,C,T,QANS,IERR)              
              ERR=DABS(ANS2-QANS)
              IF (QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE (7,200) A,B,C,T,QANS,ERR,IERR,KFORM
 200          FORMAT(4D11.4,D12.5,D12.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQJ22(X)
C
C     DQJ22 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF J22 WITH INTEGJ22
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ22/ CA,CB,CC
C     A CHANGE OF VARIABLE T=X*X REMOVES THE SQRT FROM THE INTEGRAND
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T)
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFX=ERFX*DEXP(-ARG)
      ARG=CC*CC/X2
      DQJ22=2.0D0*ERFX*DEXP(-ARG)*X2
      RETURN
      END
      SUBROUTINE J22QUAD(A,B,C,T,QANS21,IERR)
C     QUADRATURE FOR THE J22 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ22/ CA,CB,CC
      DATA TOL /0.5D-14/
      EXTERNAL DQJ22
      IERR=0
      IF((B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
        XLIM=4.0D0
      ELSE
        XLIM=DSQRT(B*B+C*C)/6.0D0
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANS21=0.0D0
        IERR=1
        RETURN
      ENDIF
      CA=A
      CB=B
      CC=C
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQJ22,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQJ22,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQJ22(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQJ22,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS21=SS
      RETURN
      END
      PROGRAM I29COMP
C
C     DONALD E. AMOS   AUGUST, 2006
C
C     REF: FOLDER 29, FOLDER 28
C
C     I29COMP COMPARES THE N MEMBER SEQUENCE GENERATED BY INTEGI29 FOR
C
C        INT ON (T,INF) OF EXP(-A^2*W^2)*I(N0+2*K-2)ERFC(B*W), K=1,N
C
C     WITH QUADRATURES ON (T,INF). THE N MEMBERS OF THE SEQUENCE START
C     WITH ORDER N0 AND CONSECUTIVE MEMBERS HAVE ORDERS 2 APART, ENDING
C     AT ORDER N0+2*N-2. IF N0 IS EVEN, INTEGI29 GENERATES EVEN ORDERS;
C     IF N0 IS ODD, INTEGI29 GENERATES ODD ORDERS. HERE I(M)ERFC(X) IS
C     THE ITERATED COERROR FUNCTION OF ORDER M.
C
C     OUTPUT IS WRITTEN TO FILE I29COMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION YN(10)
      COMMON /CQI29/AA,BB,KK
      DATA RTPI /1.772453850905516D0/
      DATA REL  /0.5D-14/
      EXTERNAL QI29
      OPEN(FILE='I29COMP.TXT',UNIT=7)
      DO 2 NSEQ=1,3
      WRITE(7,50)NSEQ
 50   FORMAT(25X,'SEQUENCE LENGTH = ',I3)
      WRITE(7,100)
 100  FORMAT(5X,"A",10X,"B",10X,'T',7X,'N0',3X,'QANS',7X,'YANS',6X,'REL
     &ERR',3X,'IRY',1X,'IRQ')
      DO 5 NI=0,3
        N0=NI
        DO 10 IB=3,9,2
          B=DBLE(FLOAT(IB))/2.0D0
          DO 20 IA=3,9,2
            A=DBLE(FLOAT(IA))/2.0D0
            DO 30 IT=1,11,2
              T=DBLE(FLOAT(IT-1))/5.0D0
              CALL INTEGI29(A,B,T,N0,NSEQ,YN,IERRY)
C             COMPARE A QUADRATURE WITH EACH MEMBER OF THE SEQUENCE
C             YN(K),K=1,NSEQ
              DO 40 K=1,NSEQ
              AA=A
              BB=B
              KK=N0+2*K-2
              INIT=0
              X1=T
              SIG=6.0D0/DSQRT(A*A+B*B)
              TOL=REL
              CALL DQUAD8(QI29,INIT,X1,SIG,TOL,X2,QANS,IERRQ)
              ERR=DABS(QANS-YN(K))
              IF(DABS(QANS).NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              IF(K.EQ.1)THEN
                WRITE (7,101) A,B,T,N0,QANS,YN(K),ERR,IERRY,IERRQ
 101            FORMAT(3D11.4,I3,3D11.4,2I4)
              ELSE
                WRITE(7,102)QANS,YN(K),ERR,IERRY,IERRQ
 102            FORMAT(36X,3D11.4,2I4)
              ENDIF
 40           CONTINUE 
 30         CONTINUE
 20       CONTINUE
 10     CONTINUE
 5    CONTINUE
 2    CONTINUE
      END
      DOUBLE PRECISION FUNCTION QI29(W)
C-----------------------------------------------------------------------
C     COMPUTATION OF THE INTEGRAND FOR THE QUADRATURE ON
C
C        INT ON (T,INF) OF EXP(-AA^2*W^2)*I(KK)ERFC(BB*W)
C
C     AA, BB, KK ARE THE PARAMETERS IN COMMON/CQI29/
C-----------------------------------------------------------------------     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION YY(1)
      COMMON /CQI29/AA,BB,KK
      DATA RTPI /1.772453850905516D0/
      DATA REL /0.5D-14/
      AW=AA*W
      BW=BB*W
      ARGA=AW*AW
      EX=DEXP(-ARGA)
      X=BW
      N0=KK
      KODE=1
      NI=1
      TOL=REL
      CALL DINERFC(X,N0,KODE,NI,TOL,YY,NZ)
      QI29=EX*YY(1)
      RETURN
      END
      PROGRAM GNCOMP
C
C     DONALD E. AMOS,   SEPTEMBER, 2002
C
C     REF: FOLDER 21
C
C     GNCOMP STUDIES THE FORWARD AND BACKWARD 3-TERM RECURRENCE FOR
C     EVALUATIONS OF
C
C      G(N)=INT ON (CAPT,INF) OF EXP(-A^2*X^2)*I(N)ERFC(B*X)/X**N
C                        CAPT=1/DSQRT(T)
C
C     THE RECURRENCE IS N(N+1)*Y(N+1)+2*N*B*CAPT*Y(N)+ABTRM*Y(N-1)
C                       =[I(N-1)ERFC(B*CAPT)*EXP(B^2*CAPT^2)]
C
C     WHERE Y(N)=2.0D0*(CAPT**(N-1))*EXP(ABTRM)*G(N), WHERE
C     I(N)ERFC IS THE ITERATED COERROR FUNCTION OF ORDER N, AND
C     ABTRM=(A^2+B^2)/T=(A^2+B^2)*CAPT*CAPT.
C
C     Y(N) CAN BE ESTIMATED FOR AN ORDER OF MAGNITUDE RESULT BY THE
C     UPPER BOUND
C
C             [EXP(ABTRM)*I(N)ERFC(B*CAPT)]*E((N+1)/2,C^2*T^2)
C
C     A STABILITY ANALYSIS INDICATES THAT FORWARD AND BACKWARD FROM
C     INDEX NFB=INT(2.0*ABTRM) IS STABLE. THE NUMERICAL RESULTS
C     SEEM TO CONFIRM THIS RESULT.
C
C     THE CODE SETS A VALUE OF M=50 FOR THE UPPER INDEX OF THE SEQUENCE
C     AND COMPUTES LOWER INDICES BY RECURRING UP AND DOWN FROM INDEX
C     NFB IN SUBROUTINE RECURR. THE RESULTS ARE COMPARED TO A QUADRATURE
C     ON THE ORIGINAL INTEGRAL USING SUBROUTINE DQUAD8.
C
C     OUTPUT IS WRITTEN TO FILE GNCOMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION YN(100),SY(1)
      COMMON /CGSEQ/ KN,CREL,CABTRM,CBT
      EXTERNAL DGNFUN
      OPEN(UNIT=7,FILE="GNCOMP.TXT")
      WRITE(7,109)
 109  FORMAT('  NFB=INDEX WHICH SEPARATES FORWARD AND BACKWARD RECURRENC
     *E')
      WRITE (7,110)
 110  FORMAT('      A          B         T     N  NFB     QANS      RELE
     &RR  IERR UPPER BOUND')
      DO 5 IT=1,11,2
      T=DBLE(FLOAT(IT))/5.0D0
      RT=DSQRT(T)
      CAPT=1.0D0/RT
      DO 10 IA=1,11,2
        A=DBLE(FLOAT(IA-1))/5.00D0
        DO 20 IB=1,11,2
          B=DBLE(FLOAT(IB))/5.0D0
          M=40
C          MM=51    
          REL=0.50D-13
          DREL=REL
C         YN(K+1) IS G(K)*(CAPT**(K-1))*DEXP(ABTRM), K=0,1,...
C         ABTRM=(A*A+B*B)*CAPT*CAPT, CAPT=1/DSQRT(T)
          CALL GNSEQ(A,B,CAPT,M,DREL,YN)
          DO 30 IN=1,M
C           SET COMMON PARAMETERS FOR DGFUN
            A2=A*A
            B2=B*B
            CBT=B*CAPT
            ABTRM=(A2+B2)/T
            CABTRM=ABTRM
            CREL=REL
            ARG=ABTRM+ABTRM
            NFB=INT(ARG)
            KN=IN-1
            ABSUM=A2+B2
            DSIG=RT/DSQRT(ABSUM)
            X1=0.0D0
            INIT=0
            DREL=REL
            CALL DQUAD8(DGNFUN,INIT,X1,DSIG,DREL,X2,QANS,IERR)
            GANS=YN(KN+1)
            ERR=DABS(GANS-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
C       COMPUTE AN ESTIMATE OF G(KN)*(CAPT**KN)*[EXP(ABTRM)]
            N0=KN             
            IF((A.EQ.0.0D0).AND.(N0.LT.2)) THEN
              WRITE (7,200) A,B,T,KN,NFB,QANS,ERR,IERR
              GO TO 30
            ENDIF
            KODE=2
            MM=1
            BT=B*CAPT
            CALL DINERFC(BT,N0,KODE,MM,REL,SY,NZ)
            EST=SY(1)
            AT=A*CAPT
            ARG=AT*AT
            KODE=2
            MM=1
            TOL=REL
            IF(MOD(N0+1,2).EQ.0) THEN
              IK=(N0+1)/2
              CALL DEXINT(ARG, IK, KODE, MM, TOL, SY, NZ, IERR)
            ELSE
              FNH=DBLE(FLOAT(N0/2))+0.5D0
              CALL DHEXINT(ARG, FNH, KODE, MM, TOL, SY, NZ, IERR)
            ENDIF
            EST=EST*SY(1)
            WRITE (7,200) A,B,T,KN,NFB,QANS,ERR,IERR,EST
 200        FORMAT(3D10.3,2I4,2D12.4,I4,D12.4)
   30     CONTINUE
          WRITE (7,111)
  111     FORMAT(/)
          PAUSE
   20   CONTINUE
   10 CONTINUE
   5  CONTINUE
      END
      PROGRAM VTCOMP
C
C     DONALD E. AMOS   DECEMBER, 2002; MAY, 2006
C
C     REF: FOLDER 21
C
C     VTCOMP COMPARES THE PROCEDURE FROM DOUBLE PRECISION FUNCTION DVOFT
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          DVOFT=INT ON (0,T) OF U(A,B,W)
C
C     WHERE  U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     AND A, B ARE NON-NGATIVE.
C
C     OUTPUT IS WRITTEN TO FILE VTCOMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="VTCOMP.TXT")
C
      WRITE (7,110)
 110  FORMAT('      A         B          T         QUAD        REL ERR
     &IERR')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/5.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,15,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            REL=0.5D-14
            ANS2=DVOFT(A,B,T,REL,IVERR,KFORM)
            IF((IVERR.NE.0)) THEN
              WRITE (7,405) A,B,T,IVERR
 405          FORMAT( 3D11.4,1X,'IN DVOFT, IVERR=',I2)
              GOTO 30
            ENDIF
            CALL VTQUAD(A,B,T,QANS,IERR)              
            ERR=DABS(ANS2-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,T,QANS,ERR,IERR
 200        FORMAT(3D11.4,D12.5,D12.4,I4)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQVT(X)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFVT/ CA,CB
C     A CHANGE OF VARIABLE TAU=X*X REMOVES THE SQRT FROM THE INTEGRAND
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T)
      ARG=CA*X+CB/X
      KODE=3
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      DQVT=2.0D0*X*ERFX*DEXP(-ARG)
      RETURN
      END
      SUBROUTINE VTQUAD(A,B,T,QANSVT,IERR)
C     QUADRATURE FOR THE VOFT FUNCTION. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFVT/ CA,CB
      DATA TOL /0.50D-14/
      EXTERNAL DQVT
      IERR=0
      IF(B.EQ.0.0D0) THEN
        XLIM=4.0D0
      ELSE
        XLIM=B/6.0D0
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANSVT=0.0D0
        IERR=1
        RETURN
      ENDIF
      CA=A
      CB=B
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQVT,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQVT,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQVT(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQVT,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANSVT=SS
      RETURN
      END
