      PROGRAM RPRGINFO
C-----------------------------------------------------------------------
C     DONALD E. AMOS, MAY 1, 2002
C
C     THIS FILE CONTAINS RESEARCH PROGRAMS WHICH EVALUATE FUNCTIONS
C     BEING STUDIED. MOST FUNCTIONS ARE INTEGRALS WITH SERIES
C     REPRESENTATIONS AND ARE COMPARED TO A DIRECT QUADRATURE. FILES
C     BECKSUBS.FOR AND AMOSSUBS.FOR MUST BE LINKED TO THE COMPILED FILES
C     FOR EXECUTION.
C
C     USAGE:
C
C     EXTRACT ONE OF THE PROGRAMS BELOW ALONG WITH ANY SUCCEEDING
C     DOUBLE PRECISION FUNCTION(S) OR SUBROUTINES, COMPILE, LINK
C     COMPILED FILES BECKSUBS.FOR AND AMOSSUBS.FOR, AND THEN EXECUTE THE
C     LINKED FILES. OUTPUT IS WRITTEN TO A TEXT FILE (*.TXT) WITH THE
C     NAME OF THE PROGRAM BEING EXECUTED.
C
C     PROGRAMS:                      OUTPUT:                  FOLDER
C              PROGRAM I1COMPB              I1COMPB.TXT        1,2
C              PROGRAM I4COMP               I4COMP.TXT          8
C              PROGRAM J4COMP               J4COMP.TXT          8
C              PROGRAM ERFINT               ERFINT.TXT         12
C              PROGRAM I13COMP              I13COMP.TXT        13
C              PROGRAM I14COMP              I14COMP.TXT        14
C              PROGRAM I19COMP              I19COMP.TXT        19
C              PROGRAM I20COMP              I20COMP.TXT        20
C              PROGRAM I24COMP              I24COMP.TXT        24
C              PROGRAM J24COMP              J24COMP.TXT        24
C              PROGRAM V24COMP              V24COMP.TXT        24
C              PROGRAM I25COMP              I25COMP.TXT        25
C              PROGRAM I26COMP              I26COMP.TXT        26
C              PROGRAM I26ACOMP             I26ACOMP.TXT       26
C              PROGRAM DGSCOMP              DGSCOMP.TXT        16
C-----------------------------------------------------------------------
      END
      PROGRAM I1COMPB
C
C     DONALD E. AMOS   AUGUST, 2003
C
C     REF: FOLDER 1, FOLDER 2, SECTION 2.6 OF "INTEGRALS RELATED TO
C          HEAT CONDUCTION AND DIFFUSION"
C
C     I1COMPB COMPARES THE CLOSED FORM OF I1 OF FOLDERS 1 & 2
C     WITH A QUADRATURE FOR I1 FOR A.LE.B:
C
C          I1=INT ON (T,INF) OF EXP(-A^2*W^2)*ERF(B*W)/W^2
C
C     WHERE A.GE.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0. THE CLOSED FORM IS
C     IMPLEMENTED IN DOUBLE PRECISION SUBROUTINE INTEGI1B WHERE THE SUM
C     FOR A.LE.B IN FOLDER 2 IS IMPLEMENTED.
C
C     PROGRAM I1COMPB IS THE SAME AS I1COMP, BUT RESTRICTED TO KODE=1
C     AND A.LE.B. SUBROUTINE INTEGI1B TAKES THE PLACE OF INTEGI1.

C     OUTPUT IS WRITTEN TO FILE I1COMPB.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      COMMON /CFI1/ CA2,CB,KKODE
      DATA RTPI       /1.772453850905516D0/
      EXTERNAL DQI1
      OPEN(UNIT=7,FILE="I1COMPB.TXT")
      WRITE (7,110)
 110  FORMAT('       A          B           T         QUAD          ERR
     *     INIT IERR2')
      KODE=1
      WRITE(7,115) KODE
 115  FORMAT(/,2X,'THE FOLLOWING COMPARISONS ARE FOR KODE = ',I3/) 
      DO 10 IT=1,101,10
        T=DBLE(FLOAT(IT))/40.0D0
        DO 20 IA=1,101,10
          A=DBLE(FLOAT(IA-1))/40.0D0
          DO 30 IB=1,101,10
            B=DBLE(FLOAT(IB-1))/40.0D0
            IF(B.LT.A) GOTO 30
            REL=0.50D-14
            CALL INTEGI1B(A,B,T,KODE,REL,ANS1,IERR2)            
            IF(B.EQ.0.0D0) THEN
              IF(KODE.EQ.1) THEN
                QANS=0.0D0
                GOTO 40
              ELSE
                IF(A.EQ.0.0D0) THEN
                  QANS=1.0D0/T
                  GOTO40
                ENDIF
              ENDIF
            ENDIF
            XXA=A
            IF (A.EQ.0.0D0) THEN
              IF(KODE.EQ.1) THEN
                BT=B*T
                IF(BT.GE.6.0D0) THEN
                  QANS=1.0D0/T
                ELSE
                  ERFB=DRERF(BT,KODE,NZ)
                  BT2=BT*BT
                  N=1
                  IKODE=1
                  M=1
                  TOL=REL
                  NZ=0
                  IERR=0
                  CALL DEXINT(BT2, N, IKODE, M, TOL, EN, NZ, IERR)
                  QANS=ERFB/T+B*EN(1)/RTPI
                  GOTO 40
                ENDIF
                GOTO 40
              ELSE
                XXA=B
              ENDIF
            ENDIF
C           SET COMMON VARIABLES
            CA2=A*A
            CB=B
            KKODE=KODE
            SIG=3.0D0/XXA
            X1=T
            QREL=0.50D-14
            INIT=0
            CALL DQUAD8(DQI1,INIT,X1,SIG,QREL,X2,QANS,IERR)
C           GET BETTER ESTIMATE OF THE TRUNCATION ERROR      
C            SIG=SIG+SIG
            CALL DQUAD8(DQI1,INIT,X1,SIG,QREL,X2,QANS,IERR)
  40        CONTINUE
            ERR=DABS(QANS-ANS1)
            IF(QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE(7,100) A,B,T,QANS,ERR,INIT,IERR2
 100        FORMAT(3D12.4,D13.5,D12.4,2I5)
 101        FORMAT(3I5)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQI1(T)
C
C     DQI1 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     WITH THE CLOSED FORM OF I1
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI1/ CA2,CB,KKODE
      T2=T*T
      ARG=CB*T
      IF(KKODE.EQ.1) THEN
        ERFX=DRERF(ARG,KKODE,NZ)
        DQI1=ERFX*DEXP(-CA2*T2)/T2        
      ELSE
        IKODE=3
        ERFX=DRERF(ARG,IKODE,NZ)
        ARG=(CA2+CB*CB)*T2
        DQI1=ERFX*DEXP(-ARG)/T2
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGI1B(A,B,T,KODE,REL,ANS1,IERR2)
C
C     INTEGI1B COMPUTES I1 FROM THE FORMULA OF FOLDER2 FOR A.LE.B WHERE
C     SQRT(t)=1/T
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      DATA RTPI                  /1.772453850905516D0/
      IF(B.EQ.0.0D0) THEN
        ANS1=0.0D0
        RETURN
      ENDIF
      A2B2=A*A+B*B
      ARG=A2B2*T*T 
      N=1
      IKODE=1
      M=1
      TOL=REL
      NZ=0
      IERR=0
      CALL DEXINT(ARG, N, IKODE, M, TOL, EN, NZ, IERR)
      X=A*T
      CALL DIERFC(X,IKODE,ANS,IERR)
      SS=RTPI*ANS/T+B*EN(1)/RTPI
      X=T
      CALL SER26(A,B,X,SUM,IERR)
      ANS1=SS-SUM
      RETURN
      END
      SUBROUTINE SER26(A,B,X,SUM,IERR)
C
C     SER26 COMPUTES THE SUM OF FOLDER 2 FOR A.LE.B.
C
C     IERR=0 NORMAL RETURN
C     IERR=1 UNDERFLOW, SUM=0.0D0
C     IERR=2 NO CONVERGENCE IN 50 TERMS OF THE SERIES
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(52)
      DATA RTPI                  /1.772453850905516D0/
      IERR=0
      ARG=(A*A+B*B)
      DEN=ARG
      ARG=ARG*X*X
      FNH=0.5D0
      KODE=1
      M=52
      TOL=1.0D-14
      NZ=0
      KERR=0
      CALL DHEXINT(ARG, FNH, KODE, M, TOL, EN, NZ, KERR)
      IF(NZ.NE.0) THEN
        IERR=1
        SUM=0.0D0
        RETURN
      ENDIF
      AK=1.0D0
      S=EN(1)
      DH=-0.5D0
      Z=A*A/DEN
      FAC=1.0D0
      DO 10 K=1,M-1
        AK=AK*DH/FAC
        AK=AK*Z
        TRM=AK*EN(K+1)
        S=S+TRM
        IF (DABS(TRM).LE.TOL*DABS(S)) GOTO 15        
        DH=DH+1.0D0
        FAC=FAC+1.0D0
  10  CONTINUE
       IERR=2
C      WRITE (7,300)
C 300  FORMAT('SER26-SERIES NOT CONVERGED IN 52 TERMS')
  15  CONTINUE
      SUM=DSQRT(DEN)*S/RTPI
      RETURN
      END
      PROGRAM I4COMP
C     
C     ON KODE=1, I4COMP COMPUTES
C
C     I4=INT(T,INF) OF W*W*EXP(-C*C*W*W)*ERF(A*W)*ERF(B*W)
C
C     AND ON KODE=2, I4COMP COMPUTES
C
C     I4C=INT(T,INF) OF W*W*EXP(-C*C*W*W)*ERFC(A*W)*ERFC(B*W)
C
C                      A.GE.0,  B.GE.0,  C.GT.0
C
C     BY SERIES AND QUADRATURE DEVELOPED IN FOLDERS 8a AND 8b AND
C     COMPARES THE RESULTS.
C
C     OUTPUT IS WRITTEN TO FILE I4COMP.TXT
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI /1.772453850905516D0/
      OPEN(UNIT=7,FILE="I4COMP.TXT")
C
      DO 5 KODE=1,2      
C
      WRITE(7,105)KODE
 105  FORMAT('KODE = ',I2)
      WRITE (7,110)
 110  FORMAT('      A          B          C          T         QUAD
     &REL ERR IERR IERR4')
      DO 10 IT=1,9,2
        T=DBLE(FLOAT(IT-1))/1.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/2.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC))/2.0D0
              CALL I4SER(A,B,C,T,KODE,ANS4,IERR4)
              CALL I4QUAD(A,B,C,T,KODE,QANS4,IERR)
              ERR=DABS(ANS4-QANS4)
              IF (QANS4.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS4)
              ENDIF
              WRITE (7,200) A,B,C,T,QANS4,ERR,IERR,IERR4
 200          FORMAT(4D11.4,2D11.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
   5  CONTINUE
      END
      SUBROUTINE I4SER(A,B,C,T,KODE,ANSI4,IERR)
C     I4SER EVALUATES THE SERIES FORM OF I4=INT (T,INF) OF
C     W*W*EXP(-C*C*W*W)*ERF(A*W)*ERF(B*W), DEVELOPED IN FOLDER 8
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI /1.772453850905516D0/
      IERR=0
      AT=A*T
      NZ=0
      ERFA=DRERF(AT,KODE,NZ)
      BT=B*T
      NZ=0
      ERFB=DRERF(BT,KODE,NZ)
      A2=A*A
      B2=B*B
      C2=C*C
      X2=A2+B2+C2
      X=DSQRT(X2)
      XT=X*T
      KKODE=2
      NZ=0
      ERFC=DRERF(XT,KKODE,NZ)
      SAC2=A2+C2
      SBC2=B2+C2
      CT=C*T
      CT2=CT*CT
      AT2=AT*AT
      BT2=BT*BT
      ECT2=DEXP(-CT2)
      S=T*ERFA*ERFB*ECT2
      TERM=(A*ERFB*ECT2*DEXP(-AT2)/SAC2+B*ERFA*ECT2*DEXP(-BT2)/SBC2)/
     *RTPI
      IF(KODE.EQ.1) THEN
        S=S+TERM
      ELSE
        S=S-TERM
      ENDIF
      PAB=A*B
      S=S+(PAB/SAC2+PAB/SBC2)*ERFC/(X*RTPI)
      CALL INTEGI3(A,B,C,T,KODE,ANSI3,IERR,KFORM)
      S=S+ANSI3
      TC2=C2+C2
      ANSI4=S/TC2
      RETURN
      END
      SUBROUTINE I4QUAD(A,B,C,T,KODE,ANS,IERR)
C     I4 BY QUADRATURE
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFI4/AA,BB,CC,CC2,KKODE
      DATA RTPI /1.772453850905516D0/
      DATA TOL /0.5D-14/
      EXTERNAL DQI4
      IERR=1
      AA=A 
      BB=B
      CC=C
      C2=C*C
      CC2=C2
      KKODE=KODE
      IF(KODE.EQ.1) THEN
C       REFINED QUADRATURE OF LIKE THAT OF FOLDERS 3 AND 6
        XAB=MIN(A,B)
        IF(XAB.EQ.0.0D0) THEN
          ANS=0.0D0
          RETURN
        ENDIF
        WSM=MIN(6.0D0/A,6.0D0/B)
        WLM=MAX(6.0D0/A,6.0D0/B)
        SS=0.0D0
        IF (T.LE.WSM) THEN
          W2=T
          SIGC=3.0D0/C
          SIG=MIN(SIGC,0.5D0*WSM)
          LN=50
          IFLAG=0
          DO 42 L=1,LN
            W1=W2
            W2=W1+SIG
            IF(W2.GT.WSM) THEN
              W2=WSM
              IFLAG=1
C             TEST FOR SMALL ERROR DUE TO A SMALL INTERVAL TO AVOID AN
C             ERROR FLAG FROM DGAUS8
              WMID=0.5D0*(W1+W2)
              DTEMP=DQI4(WMID)*DABS(W2-W1)
              IF(DTEMP.LT.2.0D0*TOL*DABS(SS)) GOTO 43
            ENDIF
            REL=TOL
            CALL DGAUS8(DQI4,W1,W2,REL,QANS,IERR)
            SS=SS+QANS
            IF(DABS(QANS).LT.TOL*DABS(SS)) GOTO 43
            IF(IFLAG.EQ.1) GOTO 43
   42     CONTINUE
   43     CONTINUE
          SS=SS+DQPI4(C,XAB,WSM,WLM)
        ELSE
          IF (T.LE.WLM)  THEN
            SS=DQPI4(C,XAB,T,WLM)
          ELSE
            C2=C*C
            ARG1=C2*T*T
            TC=C+C
            TC2=C2+C2
            KKODE=2
            NZ=0
            ARG2=C*T
            ERFD=DRERF(ARG2,KKODE,NZ)          
            SS=(T*DEXP(-ARG1)+RTPI*ERFD/TC)/TC2
          ENDIF
        ENDIF
        ANS=SS
        RETURN
      ELSE
        INIT=0
        X1=T
        SIG=3.0D0/DSQRT(A*A+B*B+C*C)
        REL=TOL
        CALL DQUAD8(DQI4,INIT,X1,SIG,REL,X2,QANS,IERR)
        ANS=QANS
      ENDIF
      END
      DOUBLE PRECISION FUNCTION DQI4(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI4/ AA,BB,CC,C2,KKODE
      ARG=BB*T
      ERFB=DRERF(ARG,KKODE,NZ)
      ARG=AA*T
      KODE=1
      ERFA=DRERF(ARG,KKODE,NZ)
      T2=T*T
      ARG=C2*T2
      DQI4=T2*ERFA*ERFB*DEXP(-ARG)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQPI4(C,X,FL,FU)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI/1.77245385090552D0/
C     X=MIN(A,B) OF FOLDER8
      T1=X*FL
      T2=X*FU
      TC=C+C
      KODE=1
      NZ=0
      ERFA=DRERF(T1,KODE,NZ)
      NZ=0
      KODE=2
      ERFB=DRERF(T2,KODE,NZ)
      C2=C*C
      TC2=C2+C2
      ARG1=C2*FL*FL
      ARG2=C2*FU*FU
      S=FL*DEXP(-ARG1)*ERFA+FU*DEXP(-ARG2)*ERFB
      X2=X*X
      SXC2=X2+C2
      ARG1=SXC2*FL*FL
      ARG2=SXC2*FU*FU
      S=S+((DEXP(-ARG1)-DEXP(-ARG2))*(X/SXC2))/RTPI
      CALL INTEGJ5(C,X,FL,ANS1,IERR)
      CALL INTEGI5(C,X,FU,ANS2,IERR)
      S=S+ANS1+ANS2
      DQPI4=S/TC2
      RETURN
      END
      PROGRAM J4COMP
C     
C     ON KODE=1, J4COMP COMPUTES
C
C     J4=INT(T,INF) OF W*EXP(-C*C*W*W)*ERF(A*W)*ERF(B*W)
C
C     AND ON KODE=2, J4COMP COMPUTES
C
C     J4C=INT(T,INF) OF W*EXP(-C*C*W*W)*ERFC(A*W)*ERFC(B*W)
C
C                      A.GE.0,  B.GE.0, C.GT.0
C
C     BY SERIES AND QUADRATURE DEVELOPED IN FOLDERS 8c AND 8d AND
C     COMPARES THE RESULTS.
C
C     OUTPUT IS WRITTEN TO FILE J4COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C     DATA RTPI /1.772453850905516D0/
      OPEN(UNIT=7,FILE="J4COMP.TXT")
C
      DO 5 KODE=1,2
C
      WRITE(7,105) KODE
 105  FORMAT('KODE = ',I2)
      WRITE (7,110)
 110  FORMAT('      A          B          C          T        QUAD
     &REL ERR  IERR IERR4')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT-1))/5.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/2.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC))/2.0D0
              CALL J4SER(A,B,C,T,KODE,ANS4,IERR4)
              CALL J4QUAD(A,B,C,T,KODE,QANS4,IERR)
              ERR=DABS(ANS4-QANS4)
              IF (QANS4.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS4)
              ENDIF
              WRITE (7,200) A,B,C,T,QANS4,ERR,IERR,IERR4
 200          FORMAT(4D11.4,2D11.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
   5  CONTINUE
      END
      SUBROUTINE J4SER(A,B,C,T,KODE,ANSJ4,IERR)
C     J4SER EVALUATES THE SERIES FORM OF J4=INT (T,INF) OF
C              W*EXP(-C*C*W*W)*ERF(A*W)*ERF(B*W)   ON KODE=1
C     OR
C              W*EXP(-C*C*W*W)*ERFC(A*W)*ERFC(B*W) ON KODE=2
C     DEVELOPED IN FOLDER 8C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI /1.772453850905516D0/
      IERR=0
      AT=A*T
      NZ=0
      ERFA=DRERF(AT,KODE,NZ)
      BT=B*T
      NZ=0
      ERFB=DRERF(BT,KODE,NZ)
      A2=A*A
      B2=B*B
      C2=C*C
      CT=C*T
      CT2=CT*CT
      ECT2=DEXP(-CT2)
      S=0.5D0*ERFA*ERFB*ECT2
      RTB=DSQRT(B2+C2)
      RTA=DSQRT(A2+C2)
      IF(KODE.EQ.1) THEN
        CALL INTEGJ5(RTB,A,T,SUMB,IERR)
        CALL INTEGJ5(RTA,B,T,SUMA,IERR)
        S=S+(B*SUMB+A*SUMA)/RTPI
      ELSE
        CALL INTEGI5(RTB,A,T,SUMB,IERR)
        CALL INTEGI5(RTA,B,T,SUMA,IERR)
        S=S-(B*SUMB+A*SUMA)/RTPI
      ENDIF
      ANSJ4=S/C2
      RETURN
      END
      SUBROUTINE J4QUAD(A,B,C,T,KODE,ANS,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFJ4/AA,BB,CC,CC2,KKODE
      EXTERNAL DQJ4
C      DATA RTPI /1.772453850905516D0/
      DATA TOL /0.5D-14/
C     REFINED QUADRATURE LIKE THAT OF FOLDERS 3 AND 6
      IERR=1
      AA=A
      BB=B
      CC=C
      C2=C*C
      CC2=C2
      KKODE=KODE
      IF(KODE.EQ.1) THEN
        XAB=MIN(A,B)
        IF(XAB.EQ.0.0D0) THEN
          ANS=0.0D0
          RETURN
        ENDIF
        WSM=MIN(6.0D0/A,6.0D0/B)
        WLM=MAX(6.0D0/A,6.0D0/B)
        SS=0.0D0
        TC2=C2+C2
        IF (T.LE.WSM) THEN
          W2=T
          SIGC=3.0D0/C
          SIG=MIN(SIGC,0.5D0*WSM)
          LN=50
          IFLAG=0
          DO 42 L=1,LN
            W1=W2
            W2=W1+SIG
            IF(W2.GT.WSM) THEN
              W2=WSM
              IFLAG=1
C             TEST FOR SMALL ERROR DUE TO A SMALL INTERVAL TO AVOID AN
C             ERROR FLAG FROM DGAUS8
              WMID=0.5D0*(W1+W2)
              DTEMP=DQJ4(WMID)*DABS(W2-W1)
              IF(DTEMP.LT.2.0D0*TOL*DABS(SS)) GOTO 43
            ENDIF
            REL=TOL
            CALL DGAUS8(DQJ4,W1,W2,REL,QANS,IERR)
            SS=SS+QANS
            IF(IFLAG.EQ.1) GOTO 43          
            IF(DABS(QANS).LT.TOL*DABS(SS)) GOTO 43
   42     CONTINUE
   43     CONTINUE   
          SS=SS+DQPJ4(C,XAB,WSM,WLM)
        ELSE
          IF (T.LE.WLM)  THEN
            SS=DQPJ4(C,XAB,T,WLM)
          ELSE
            ARG1=C2*T*T
            SS=DEXP(-ARG1)/TC2
          ENDIF
        ENDIF
        ANS=SS
        RETURN
      ELSE
        INIT=0
        X1=T
        SIG=3.0D0/DSQRT(A*A+B*B+C*C)
        REL=TOL
        CALL DQUAD8(DQJ4,INIT,X1,SIG,REL,X2,ANS,IERR)
      ENDIF
      END
      DOUBLE PRECISION FUNCTION DQPJ4(C,X,FL,FU)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C      DATA RTPI/1.77245385090552D0/
C     X=MIN(A,B) OF FOLDER8
      T1=X*FL
      T2=X*FU
      X2=X*X
      KODE=1
      NZ=0
      ERFA=DRERF(T1,KODE,NZ)
      NZ=0
      KODE=2
      ERFB=DRERF(T2,KODE,NZ)
      C2=C*C
      TC2=C2+C2
      ARG1=C2*FL*FL
      ARG2=C2*FU*FU
      S=DEXP(-ARG1)*ERFA+DEXP(-ARG2)*ERFB
      RT=DSQRT(C2+X2)
      T1=FL*RT
      KODE=2
      NZ=0
      ERFCL=DRERF(T1,KODE,NZ)
      T2=FU*RT
      KODE=2
      NZ=0
      ERFCU=DRERF(T2,KODE,NZ)
      S=S+X*(ERFCL-ERFCU)/RT
      DQPJ4=S/TC2
      RETURN
      END
      DOUBLE PRECISION FUNCTION DQJ4(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ4/ AA,BB,CC,C2,KKODE
      ARG=BB*T
      ERFB=DRERF(ARG,KKODE,NZ)
      ARG=AA*T
      ERFA=DRERF(ARG,KKODE,NZ)
      T2=T*T
      ARG=C2*T2
      DQJ4=T*ERFA*ERFB*DEXP(-ARG)
      RETURN
      END
      PROGRAM ERFINT
C
C     DONALD E. AMOS   JULY, 2001
C
C     REF: FOLDER 12
C          INTEGRAL  X*EXP((A^2-B^2)*X)*ERFC(A*DX+C/DX), DX=SQRT(X)
C          BECK E-MAIL DATED 7/12/01
C
C     OUTPUT IS WRITTEN TO FILE ERFINT.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFI3/ CA,CA2,CB2,CC
      EXTERNAL DGFUN
      OPEN(UNIT=7,FILE="ERFINT.TXT")
      X1=1.772D0
      X2=4.14159D0
      WRITE (7,109) X1,X2
 109  FORMAT(' X1 = ',D12.4,'  X2 = ',D12.4)
      WRITE (7,110)
 110  FORMAT('      A         B         C        QANS       RELERR   IER
     *R')
      DO 10 IA=1,10,2
        A=DBLE(FLOAT(IA))/2.00D0
        DO 20 IB=1,11,2
          B=DBLE(FLOAT(IB))/2.0D0
          IF (B.EQ.A) GOTO 20
          DO 30 IC=1,5
            C=DBLE(FLOAT(IC))/1.0D0
            CALL ERFINTX(X2,A,B,C,GANS2)
            CALL ERFINTX(X1,A,B,C,GANS1)
            GANS=GANS2-GANS1
C           SET COMMON PARAMETERS FOR DGFUN
            CA=A
            CA2=A*A
            CB2=B*B
            CC=C
            REL=0.50D-12
            CALL DGAUS8(DGFUN, X1, X2, REL, QANS, IERR)
            ERR=DABS(GANS-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,C,QANS,ERR,IERR
 200        FORMAT(3D10.3,2D12.4,I4)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DGFUN(T)
C
C     DGFUN COMPUTES THE INTEGRAND T*EXP((A^2-B^2)*T)*ERFC(A*RT+C/RT)
C     WHERE RT=SQRT(T)
C
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      COMMON/ CFI3/ CA,CA2,CB2,CC
      DT=DSQRT(T)
      ARG=CA*DT+CC/DT
      KODE=2
      Y=DRERF(ARG,KODE,NZ)
      DGFUN=T*Y*DEXP((CA2-CB2)*T)
      RETURN
      END
      SUBROUTINE ERFINTX(X,A,B,C,ANS)
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      A2=A*A
      B2=B*B
      A2MB2=A2-B2
      T1=-DEXP(A2MB2*X)/A2MB2
      T2=-X+1.0D0/A2MB2
      KODE=2
      DX=DSQRT(X)
      ARG=A*DX+C/DX
      Y=DRERF(ARG,KODE,NZ)
      TRM1=T1*T2*Y
      CALL FN(DX,A,B,C,FANS1)
      CALL FN(DX,-A,B,-C,FANS2)
      ARG=-2.0D0*A*C
      ANS=TRM1+0.5D0*DEXP(ARG)*(FANS1-FANS2)
      RETURN
      END
      SUBROUTINE FN(DX,A,B,C,FANS)
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      DATA RTPI       /1.772453850905516D0/
      ARG=2.0D0*B*C
      A2MB2=A*A-B*B
      T1=DEXP(ARG)/A2MB2
      ARG=B*DX+C/DX
      KODE=2
      Y=DRERF(ARG,KODE,NZ)
      T2=(-DX/B)*(1.0D0+A/B)/RTPI
      T3=(C/B+1.0D0/A2MB2)*(1.0D0+A/B)-0.5D0*(A/B)/(B*B)
      FANS=T1*(T2*DEXP(-ARG*ARG)+T3*Y)
      RETURN
      END
      PROGRAM I13COMP
C
C     DONALD E. AMOS   DECEMBER, 2001; DECEMBER, 2005
C
C     REF: FOLDER 13, FOLDER 10, FOLDER 11
C
C     ON KODE=1, I13COMP COMPUTES THE INTEGRAL
C
C          I13=INT ON (T,INF) OF EXP(-A^2*W^2)ERF(B*W)/(W**3);
C
C     ON KODE=2 I13COMP COMPUTES THE COMPLEMENTARY INTEGRAL
C
C         I13C=INT ON (T,INF) OF EXP(-A^2*W^2)ERFC(B*W)/(W**3)
C
C     AND EACH INTEGRAL IS COMPARED WITH ITS RESPECTIVE QUADRATURE,
C     WHERE     A.GT.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0.
C
C     OUTPUT IS WRITTEN TO FILE I13COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFI13/ CA2,CB,KKODE
      EXTERNAL DQI13
      OPEN(UNIT=7,FILE="I13COMP.TXT")
      DO 5 KODE=1,2
        KKODE=KODE
        WRITE(7,109) KODE
 109    FORMAT(/,27X,"KODE = ",I1)
        WRITE (7,110)
 110    FORMAT('       A          B           T          QUAD       REL
     &ERR    IERR')
        DO 10 IT=1,25,2
          T=DBLE(FLOAT(IT))/5.0D0
          DO 20 IA=1,21,2
            A=DBLE(FLOAT(IA))/10.0D0
            A2=A*A
            CA2=A2
            DO 30 IB=1,21,2
              B=DBLE(FLOAT(IB-1))/10.0D0
              CB=B
              B2=B*B
              CALL INTEGI13(A,B,T,KODE,ANS13)
              INIT=0
              X1=T
              SIG=3.0D0/A
              IF(KODE.EQ.2) THEN
                SIG=3.0D0/DSQRT(A2+B2)
              ENDIF
              REL=0.5D-15
              CALL DQUAD8(DQI13,INIT,X1,SIG,REL,X2,QANS,IERR)
              ERR=DABS(QANS-ANS13)
              IF(QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE(7,100) A,B,T,QANS,ERR,IERR
 100          FORMAT(3D12.4,D13.5,D12.4,I4)
   30       CONTINUE
   20     CONTINUE
   10   CONTINUE
   5  CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQI13(T)
C
C     DQI13 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI13/ CA2,CB,KKODE
      T2=T*T
      ARG1=CA2*T2
      NZ=0
      ARG=CB*T
      ERFB=DRERF(ARG,KKODE,NZ)
      DQI13=DEXP(-ARG1)*ERFB/(T2*T)
      RETURN
      END
      SUBROUTINE INTEGI13(A,B,T,KODE,ANS13)
C
C     REF: FOLDER 13
C
C     ON KODE=1, INTEGI13 COMPUTES THE INTEGRAL
C
C          ANS13=INT ON (T,INF) OF EXP(-A^2*W^2)*ERF(B*W)/(W**3);
C
C     ON KODE=2 INTEGI13 COMPUTES THE COMPLEMENTARY INTEGRAL
C
C          ANS13=INT ON (T,INF) OF EXP(-A^2*W^2)*ERFC(B*W)/(W**3)
C
C     WHERE    A.GT.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0.
C
C     CALLS ROUTINES: DRERF, DIERFC, DEXINT, INTEGP, INTEGQ
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      DATA RTPI       /1.772453850905516D0/
      T2=T*T
      A2=A*A
      B2=B*B
      NZ=0
      REL=0.5D-14
      IF(KODE.EQ.1) THEN
C       KODE=1 EVALUATES FORMULAE FOR I13
C       IFORM=1 AND 2 EVALUATE 2 DIFFERENT FORMULAS FOR I13
C       IFORM=2 SEEMS TO GIVE BETTER OVERALL RESULTS 
        IFORM=2
        LKODE=1
        ARG=B*T
        ERFB=DRERF(ARG,LKODE,NZ)
        X=T*DSQRT(A2+B2)
        LKODE=1
        CALL DIERFC(X,LKODE,ANS,IERR)
        IF(IFORM.EQ.1) THEN
          ARG=A2*T2
          S=0.5D0*ERFB*DEXP(-ARG)/T2+B*ANS/T
          TOL=REL
          KODE=1
          CALL INTEGP(A,B,T,KODE,TOL,PANS,IERR)
          ANS13=S-A2*PANS
        ELSE
          X=A2*T2
          N=2
          LKODE=1
          M=1
          TOL=REL
          CALL DEXINT(X, N, LKODE, M, TOL, EN, NZ, IERR)
          TOL=REL
          CALL INTEGQ(B,A,T,TOL,QANS,IERR)
          ANS13=0.5D0*ERFB*EN(1)/T2+B*ANS/T-A2*B*QANS/RTPI
        ENDIF
      ELSE
C       KODE=2 EVALUATES FORMULAE FOR I13C       
C       IFORM=1 AND 2 EVALUATE 2 DIFFERENT FORMULAS FOR I13C
C       IFORM=2 SEEMS TO GIVE BETTER OVERALL RESULTS 
        IFORM=2
        LKODE=2
        ARG=B*T
        ERFB=DRERF(ARG,LKODE,NZ)
        X=T*DSQRT(A2+B2)
        LKODE=1
        CALL DIERFC(X,LKODE,ANS,IERR)
        X=A2*T2
        LKODE=1
        M=1
        TOL=REL
        CALL DEXINT(X, IFORM, LKODE, M, TOL, EN, NZ, IERR)
        IF(IFORM.EQ.1) THEN
          ARG=A2*T2
          S=0.5D0*ERFB*DEXP(-ARG)/T2-0.5D0*A2*EN(1)-B*ANS/T
          TOL=REL
          KODE=1
          CALL INTEGP(A,B,T,KODE,TOL,PANS,IERR)
          ANS13=S+A2*PANS
        ELSE
          TOL=REL
          CALL INTEGQ(B,A,T,TOL,QANS,IERR)
          ANS13=0.5D0*EN(1)*ERFB/T2-B*ANS/T+A2*B*QANS/RTPI
        ENDIF
      ENDIF
      RETURN
      END
      PROGRAM I14COMP
C
C     DONALD E. AMOS   DECEMBER, 2001; DECEMBER, 2005
C
C     REF: FOLDER 14
C
C     ON KODE=1, I14COMP COMPUTES THE INTEGRAL
C
C       I14=INT ON (T,INF) OF EXP(-C^2*W^2)*ERF(A*W)*ERF(B*W)/(W**2);
C
C     ON KODE=2 I14COMP COMPUTES  THE INTEGRAL
C
C       I14C=INT ON (T,INF) OF EXP(-C^2*W^2)*ERFC(A*W)*ERFC(B*W)/(W**2)
C
C     IN SUBROUTINE INTEGI14 AND EACH INTEGRAL IS COMPARED WITH ITS
C     RESPECTIVE QUADRATURE WHERE
C
C             A.GE.0.0D0, B.GE.0.0D0, C.GT.0.0D0 AND T.GT.0.0D0.
C
C     OUTPUT IS WRITTEN TO FILE I14COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFI14/ AA,BB,CC,C2,KKODE
      EXTERNAL DQI14
      OPEN(UNIT=7,FILE="I14COMP.TXT")
      DO 5 KODE=1,1
        KKODE=KODE
        WRITE(7,105) KODE
 105    FORMAT(/' KODE = ',I2)
        WRITE (7,110)
 110    FORMAT('      A         B          C          T         QUAD
     &  REL ERR  IERR')
        DO 10 IT=1,25,2
          T=DBLE(FLOAT(IT))/5.0D0
          DO 20 IA=1,21,2
            A=DBLE(FLOAT(IA-1))/10.0D0
            DO 30 IB=1,21,2
              B=DBLE(FLOAT(IB-1))/10.0D0
              DO 40 IC=1,21,2
                C=DBLE(FLOAT(IC))/10.0D0
                CALL INTEGI14(A,B,C,T,KODE,ANS14,I3ERR)
                IF((I3ERR.NE.0)) THEN
                  WRITE (7,405) A,B,C,T,I3ERR
 405              FORMAT( 4D11.4,1X,'IN INTEGI14, I3ERR=',I1)
                  GOTO 40
                ENDIF
                JD=50
                X2=T
                IF(KODE.EQ.1) THEN
                  XA=MIN(A,B)
                  IF(XA.EQ.0.0D0) THEN
                    QANS=0.0D0
                    IERR=1
                    GOTO 55
                  ENDIF
                  WSM=MIN(6.0D0/A,6.0D0/B)
                  WBM=MAX(6.0D0/A,6.0D0/B)
                ELSE
                  SIG=3.0D0/DSQRT(A*A+B*B+C*C)
                ENDIF
                AA=A
                BB=B
                CC=C
                C2=C*C
                SIGC=3.0D0/C
                QANS=0.0D0
                REL=0.50D-14
                DO 50 J=1,JD
                  X1=X2
                  IF(KODE.EQ.1) THEN
                    IF(X1.LE.WSM) THEN
                      SIG=MIN(SIGC,0.5D0*WSM)
                    ELSE
                      IF(X1.LE.WBM) THEN
                        SIG=MIN(SIGC,0.5D0*WBM)
                      ENDIF
                    ENDIF
                  ENDIF
                  X2=X1+SIG
                  CALL DGAUS8(DQI14,X1,X2,REL,ANS,IERR)
                  QANS=QANS+ANS
                  IF (DABS(ANS).LE.REL*DABS(QANS)) GOTO 55
  50            CONTINUE
                WRITE (7,300)
 300  FORMAT('NO CONVERGENCE FROM DGAUS8 LOOP IN I14COMP AFTER 50 STEPS'
     &)
  55            CONTINUE
                ERR=DABS(ANS14-QANS)
                IF (QANS.NE.0.0D0) THEN
                  ERR=ERR/DABS(QANS)
                ENDIF
                WRITE (7,200) A,B,C,T,QANS,ERR,IERR
 200            FORMAT(4D11.4,D12.5,D12.4,I4)
  40          CONTINUE 
  30        CONTINUE
  20      CONTINUE
  10    CONTINUE
   5  CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQI14(T)
C
C     DQI14 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I14
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI14/ AA,BB,CC,C2,KKODE
      ARG=BB*T
      NZ=0
      ERFB=DRERF(ARG,KKODE,NZ)/T
      ARG=AA*T
      ERFA=DRERF(ARG,KKODE,NZ)/T
      T2=T*T
      ARG=C2*T2
      DQI14=ERFA*ERFB*DEXP(-ARG)
      RETURN
      END
      SUBROUTINE INTEGI14(A,B,C,T,KODE,ANS14,IERR14)
C
C     REF: FOLDER 14 
C
C     ON KODE=1, I14COMP COMPUTES THE INTEGRAL
C
C       I14=INT ON (T,INF) OF EXP(-C^2*W^2)*ERF(A*W)*ERF(B*W)/(W*W);
C
C     ON KODE=2 I14COMP COMPUTES  THE INTEGRAL
C
C       I14C=INT ON (T,INF) OF EXP(-C^2*W^2)*ERFC(A*W)*ERFC(B*W)/(W*W)
C
C     WHERE    A.GE.0.0D0, B.GE.0.0D0, C.GT.0.0D0, AND T.GT.0.0D0.
C
C     CALLS ROUTINES: INTEGI1,INTEGI2,INTEGI3,INTEGP,INTEGI13 
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI/ 1.772453850905516D0/
      IERR14=0
      KFORM=0
      REL=0.5D-14
      XA=MIN(A,B)
      XB=MAX(A,B)
      AT=XA*T
      BT=XB*T
      IF(KODE.EQ.1) THEN
        IF(XA.EQ.0.0D0) THEN
          ANS14=0.0D0
          RETURN
        ENDIF
        IF(BT.GT.6.0D0) THEN
          IF(AT.GT.6.0D0) THEN
            KKODE=1
            X=C*T
            CALL DIERFC(X,KKODE,ANS,IERR)
            ANS14=RTPI*ANS/T
            RETURN
          ELSE
            CALL INTEGI1(C,XA,T,KODE,REL,ANS14,IERR)
            RETURN
          ENDIF
        ENDIF
      ELSE
        IF(AT.EQ.0.0D0) THEN
          TOL=REL
C         ON KODE=2 AND ONE OF A OR B EQUAL TO ZERO, I14C BECOMES I1C OF
C         FOLDERC 1,2, AND 10
          CALL INTEGI1(C,XB,T,KODE,TOL,ANS14,IERR)
          RETURN
        ENDIF
      ENDIF
      A2=A*A
      B2=B*B
      C2=C*C
      T2=T*T
      CALL INTEGI3(A,B,C,T,KODE,ANSI3,IERR,KFORM)
      RTA2C2=DSQRT(A2+C2)
      RTB2C2=DSQRT(B2+C2)
      RTA2B2=DSQRT(A2+B2)
      IF(KODE.EQ.1) THEN
        SS=ANSI3
        TOL=REL
        CALL INTEGP(RTA2C2,B,T,KODE,TOL,PANS,IERR)
        SS=SS+PANS/(A*RTPI)
        TOL=REL
        CALL INTEGP(RTB2C2,A,T,KODE,TOL,PANS,IERR)
        SS=SS+PANS/(B*RTPI)
        TOL=REL
        CALL INTEGP(C,RTA2B2,T,KODE,TOL,PANS,IERR)
        SS=SS-RTA2B2*PANS/(A*B*RTPI)
        CALL INTEGI2(A,B,T,KODE,ANSI2,IERR)
        STEMP=DEXP(-C2*T2)*ANSI2/T2-2.0D0*C2*SS
        CALL INTEGI13(RTA2C2,B,T,KODE,CU)
        SS=CU/(A*RTPI)
        CALL INTEGI13(RTB2C2,A,T,KODE,CU)
        SS=SS+CU/(B*RTPI)
        CALL INTEGI13(C,RTA2B2,T,KODE,CU)
        SS=SS-RTA2B2*CU/(A*B*RTPI)
        ANS14=STEMP-2.0D0*SS
      ELSE
        SS=-ANSI3
        TOL=REL
        CALL INTEGP(RTA2C2,B,T,KODE,TOL,PANS,IERR)
        SS=SS+PANS/(A*RTPI)
        TOL=REL
        CALL INTEGP(RTB2C2,A,T,KODE,TOL,PANS,IERR)
        SS=SS+PANS/(B*RTPI)
        TOL=REL
        CALL INTEGP(C,RTA2B2,T,KODE,TOL,PANS,IERR)
        SS=SS-RTA2B2*PANS/(A*B*RTPI)
        CALL INTEGI2(A,B,T,KODE,ANSI2,IERR)
        STEMP=-DEXP(-C2*T2)*ANSI2/T2+2.0D0*C2*SS
        CALL INTEGI13(RTA2C2,B,T,KODE,CU)
        SS=CU/(A*RTPI)
        CALL INTEGI13(RTB2C2,A,T,KODE,CU)
        SS=SS+CU/(B*RTPI)
        CALL INTEGI13(C,RTA2B2,T,KODE,CU)
        SS=SS-RTA2B2*CU/(A*B*RTPI)
        ANS14=STEMP+2.0D0*SS
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGI13(A,B,T,KODE,ANS13)
C
C     REF: FOLDER 13
C
C     ON KODE=1, INTEGI13 COMPUTES THE INTEGRAL
C
C          ANS13=INT ON (T,INF) OF EXP(-A^2*W^2)*ERF(B*W)/(W**3);
C
C     ON KODE=2 INTEGI13 COMPUTES THE COMPLEMENTARY INTEGRAL
C
C          ANS13=INT ON (T,INF) OF EXP(-A^2*W^2)*ERFC(B*W)/(W**3)
C
C     WHERE    A.GT.0.0D0, B.GE.0.0D0, AND T.GT.0.0D0.
C
C     CALLS ROUTINES: DRERF, DIERFC, DEXINT, INTEGP, INTEGQ
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      DATA RTPI       /1.772453850905516D0/
      T2=T*T
      A2=A*A
      B2=B*B
      NZ=0
      REL=0.5D-14
      IF(KODE.EQ.1) THEN
C       KODE=1 EVALUATES FORMULAE FOR I13
C       IFORM=1 AND 2 EVALUATE 2 DIFFERENT FORMULAS FOR I13
C       IFORM=2 SEEMS TO GIVE BETTER OVERALL RESULTS 
        IFORM=2
        LKODE=1
        ARG=B*T
        ERFB=DRERF(ARG,LKODE,NZ)
        X=T*DSQRT(A2+B2)
        LKODE=1
        CALL DIERFC(X,LKODE,ANS,IERR)
        IF(IFORM.EQ.1) THEN
          ARG=A2*T2
          S=0.5D0*ERFB*DEXP(-ARG)/T2+B*ANS/T
          TOL=REL
          KODE=1
          CALL INTEGP(A,B,T,KODE,TOL,PANS,IERR)
          ANS13=S-A2*PANS
        ELSE
          X=A2*T2
          N=2
          LKODE=1
          M=1
          TOL=REL
          CALL DEXINT(X, N, LKODE, M, TOL, EN, NZ, IERR)
          TOL=REL
          CALL INTEGQ(B,A,T,TOL,QANS,IERR)
          ANS13=0.5D0*ERFB*EN(1)/T2+B*ANS/T-A2*B*QANS/RTPI
        ENDIF
      ELSE
C       KODE=2 EVALUATES FORMULAE FOR I13C       
C       IFORM=1 AND 2 EVALUATE 2 DIFFERENT FORMULAS FOR I13C
C       IFORM=2 SEEMS TO GIVE BETTER OVERALL RESULTS 
        IFORM=2
        LKODE=2
        ARG=B*T
        ERFB=DRERF(ARG,LKODE,NZ)
        X=T*DSQRT(A2+B2)
        LKODE=1
        CALL DIERFC(X,LKODE,ANS,IERR)
        X=A2*T2
        LKODE=1
        M=1
        TOL=REL
        CALL DEXINT(X, IFORM, LKODE, M, TOL, EN, NZ, IERR)
        IF(IFORM.EQ.1) THEN
          ARG=A2*T2
          S=0.5D0*ERFB*DEXP(-ARG)/T2-0.5D0*A2*EN(1)-B*ANS/T
          TOL=REL
          KODE=1
          CALL INTEGP(A,B,T,KODE,TOL,PANS,IERR)
          ANS13=S+A2*PANS
        ELSE
          TOL=REL
          CALL INTEGQ(B,A,T,TOL,QANS,IERR)
          ANS13=0.5D0*EN(1)*ERFB/T2-B*ANS/T+A2*B*QANS/RTPI
        ENDIF
      ENDIF
      RETURN
      END
      PROGRAM I19COMP
C
C     DONALD E. AMOS    APRIL, 2002 ; JUNE, 2003
C
C     REFERENCE: FOLDER19
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     ON KODE=1, I19COMP COMPUTES THE INTEGRAL
C
C            INT ON (0,T) OF ERF(A*W)*ERF(B*W)/W,      A>0, B>0, T>0
C
C     AND ON KODE=2, I19COMP COMPUTES THE INTEGRAL
C
C            INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)/W,  A>0, B>0, T>0
C
C     BY QUADRATURE (DQUAD8) AND BY SERIES (INTEGI19) AND PRINTS THE
C     RELATIVE ERROR.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /QFUN/ CA,CB,KKODE
      EXTERNAL QFUNI19
      OPEN(UNIT=7,FILE="I19COMP.TXT")
      KODE=2
      WRITE(7,150) KODE
 150  FORMAT('KODE = ',I3)   
      WRITE(7,200)
 200  FORMAT('     A          B          T         QUAD       SERIES   
     C  REL ERR   IERR')
      DO 10 IA=1,21,2
         A=DBLE(FLOAT(IA))/10.0D0
        DO 20 IB=1,21,2
          B=DBLE(FLOAT(IB))/10.0D0
          DO 30 IT=1,41,2
            T=0.0D0+0.2D0*DBLE(FLOAT(IT))
            CALL INTEGI19(A,B,T,KODE,ANS,JERR)
            CA=A
            CB=B
            KKODE=KODE
            IF(KODE.EQ.1) THEN
              X1=0.0D0
              X2=T
              REL=0.5D-14
              CALL DGAUS8(QFUNI19,X1,X2,REL,QANS,IERR)
            ELSE
              INIT=0
              X1=T
              SIG=1.50D0
              REL=0.5D-14
              CALL DQUAD8(QFUNI19,INIT,X1,SIG,REL,X2,QANS,IERR)
C             CALL DQUAD8 AGAIN TO BE SURE THE TRUCATION IS ADEQUATE
              SIG=3.0D0
              REL=0.5D-14
              CALL DQUAD8(QFUNI19,INIT,X1,SIG,REL,X2,QANS,IERR)
            ENDIF
            ERR=DABS(ANS-QANS)
            IF(QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE(7,100) A,B,T,QANS,ANS,ERR,IERR
  100       FORMAT(3D11.4,3D12.5,I3)
  30      CONTINUE
  20    CONTINUE
  10  CONTINUE
      END
      SUBROUTINE INTEGI19(A,B,T,KODE,ANS,IERR)
C----------------------------------------------------------------------
C     REFERENCE: FOLDER19
C
C     ON KODE=1, I19COMP COMPUTES THE INTEGRAL
C
C            I19 =INT ON (0,T) OF ERF(A*W)*ERF(B*W)/W
C
C     AND ON KODE=2, I19COMP COMPUTES THE COMPLEMENTARY INTEGRAL
C
C            I19C=INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)/W
C
C     WHERE   A>0  ,  B>0  , T>0.
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EGH(75),ENH(75),C(75),RTMP(2)
      DATA PI, RTPI/3.14159265358979324D0, 1.77245385090551603D0/
      IERR=0
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        IERR=5
        ANS=0.0D0
        RETURN
      ENDIF
      REL=0.5D-15
C     CHOOSING AA TO BE MIN AND BB TO B MAX OF A AND B
      IF(A.LE.B) THEN
        AA=A
        BB=B
      ELSE
        AA=B
        BB=A
      ENDIF
      AT=AA*T
      BT=BB*T
      KKODE=1
      GERFBT=DGERFC(BT,KKODE,REL,IERRG)
      IF(IERRG.NE.0) THEN
        ANS=0.0D0
        RETURN
      ENDIF      
      ERFAT=DRERF(AT,KODE,NZ)
      IF(NZ.NE.0) THEN
        ANS=0.0D0
        RETURN
      ENDIF
      SS=ERFAT*GERFBT
C     HERE SS IS ERF(AT)*G(BT) ON KODE=1 OR ERFC(AT)*G(BT) ON KODE=2
C     COMPUTE THE REGULAR PART OF I19C 
      DO 5 I=1,3-KODE
        IF(I.EQ.1) THEN
          TD=T
        ELSE
          TD=0.0D0
        ENDIF        
        A2=AA*AA
        B2=BB*BB
        A2PB2=A2+B2
        R1=A2/A2PB2
        R2=B2/A2PB2
        X=TD*TD*A2PB2
        RTAB=DSQRT(A2PB2)
        CALL INTEGI5(AA,BB,TD,ANSI5,IERR5)      
        TRMLN=AA*DLOG(R2)*ANSI5/RTPI
        FNH=1.5D0
        KKODE=1
        MM=60
        TOL=REL
        CALL DGHEXINT(X, FNH, KKODE, MM, TOL, EGH, ENH, NZ, IERR)
        IF(NZ.NE.0) THEN
          ANS=0.0D0
          RETURN
        ENDIF
C       COMBIME THE TERMS OF THE TWO SERIES. THE POWERS OF R1 REDUCE
C       THE EFFECT OF TERMS WHERE LOSSES OF SIGNIFICANCE OCCUR IN THE
C       SUBTRACTION
        PWR=R1
        C(1)=1.0D0
        AK=0.50D0
        BK=1.0D0
        S=0.0D0
        TOL=R1*REL/10.0D0
        DO 10 K=2,60
          CK=C(K-1)*AK/BK
          T1=CK*EGH(K)
          C(K)=CK
          CSUM=0.0D0
          AM=1.0D0
          DO 20 M=1,K-1
            CSUM=CSUM+C(K-M)/AM
            AM=AM+1.0D0
 20       CONTINUE
          T2=ENH(K)*CSUM
          S=S+PWR*(T2-T1)
          IF(DABS(PWR).LE.TOL) GOTO 30
          PWR=PWR*R1
          AK=AK+1.0D0
          BK=BK+1.0D0
 10       CONTINUE
          WRITE(7,190)
 190      FORMAT('IN INTEGI19, DROP THROUGH THE SERIES LOOP')
          PAUSE
  30    CONTINUE
        S=S-EGH(1)
C       RTMP(1) CONTAINS THE EVALUATION OF R AT T
C       RTMP(2) CONTAINS THE EVALUATION OF R AT T=0.0D0
        RTMP(I)=TRMLN+0.5D0*AA*S/(PI*RTAB)
 5    CONTINUE
      IF(KODE.EQ.2) THEN
        ANS=SS+RTMP(1)
      ELSE
        FAT=DFERF(AT,REL,KERR)
        ANS=FAT+RTMP(2)+SS-RTMP(1)
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION QFUNI19(W)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /QFUN/ CA,CB,KKODE
      ARG=CA*W
      ERFAT=DRERF(ARG,KKODE,NZ)
      ARG=CB*W
      ERFBT=DRERF(ARG,KKODE,NZ)
      QFUNI19=ERFAT*ERFBT/W
      RETURN
      END
      PROGRAM I20COMP
C
C     DONALD E. AMOS    APRIL, 2002: JUNE, 2003 
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REFERENCE: FOLDER 20 
C
C     ON KODE=1, I20COMP COMPUTES THE INTEGRAL
C
C        I20 =  INT ON (T,INF) OF EXP(-A^2*X^2)*ERF(B*X)*LN(X)
C
C     ON KODE=2, I20COMP COMPUTES THE COMPLEMENTARY INTEGRAL
C
C        I20C=  INT ON (T,INF) OF EXP(-A^2*X^2)*ERFC(B*X)*LN(X)
C
C                     A > 0, B > 0, T > 0
C
C     BY QUADRATURE (DQUAD8) AND BY SERIES (INTEGI20) AND PRINTS THE
C     RELATIVE ERROR IN FILE I20COMP.TXT. SUBROUTINE INTEGI20 USES
C     SUBROUTINE INTEGI19.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /QFUN/ CA,CB,KKODE
      EXTERNAL QFUNI20
      OPEN(UNIT=7,FILE="I20COMP.TXT")
      KODE=2
      WRITE(7,150) KODE
 150  FORMAT(' KODE = ',I3)
      WRITE(7,200)
 200  FORMAT('     A          B          T         QUAD       SERIES   
     C  REL ERR   IERR')
      DO 10 IA=1,21,2
        A=DBLE(FLOAT(IA))/10.0D0
        DO 20 IB=1,21,2
          B=DBLE(FLOAT(IB))/10.0D0
          DO 30 IT=1,41,2
            T=0.0D0+0.2D0*DBLE(FLOAT(IT))
            CALL INTEGI20(A,B,T,KODE,ANS,JERR)
            CA=A
            CB=B
            KKODE=KODE
            INIT=0
            X1=T
            SIG=1.50D0
            REL=0.5D-14
            CALL DQUAD8(QFUNI20,INIT,X1,SIG,REL,X2,QANS,IERR)
C           CALL DQUAD8 AGAIN TO BE SURE THE TRUCATION IS ADEQUATE
            SIG=3.0D0
            REL=0.5D-14
            CALL DQUAD8(QFUNI20,INIT,X1,SIG,REL,X2,QANS,IERR)
            ERR=DABS(ANS-QANS)
            IF(QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE(7,100) A,B,T,QANS,ANS,ERR,IERR
  100       FORMAT(3D11.4,3D12.5,I3)
  30      CONTINUE
  20    CONTINUE
  10  CONTINUE
      END
      SUBROUTINE INTEGI20(A,B,T,KODE,ANS,IERR)
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REFERENCE: FOLDER20
C
C     ON KODE=1, INTEGI20 COMPUTES THE INTEGRAL
C
C        ANS = INT ON (T,INF) OF EXP(-A^2*X^2)*ERF(B*X)*LN(X)
C
C     ON KODE=2, INTEGI20 COMPUTES THE COMPLEMENTARY INTEGRAL
C
C        ANS= INT ON (T,INF) OF EXP(-A^2*X^2)*ERFC(B*X)*LN(X)
C
C                       A > 0, B > 0, T > 0
C
C     INTEGI20 USES SUBROUTINE INTEGI19
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(60)
      DATA RTPI/1.77245385090551603D0/
      IERR=0
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        ANS=0.0D0
        IERR=5
        RETURN
      ENDIF
      REL=0.5D-15
C     AA= MIN(A,B)= FIRST ARGUMENT OF I20C,
C     BB= MAX(A,B)= SECOND ARGUMENT OF I20C
      IF(A.LE.B) THEN
        AA=A
        BB=B
        IFLAG=0
      ELSE
        AA=B
        BB=A
        IFLAG=1
      ENDIF
C     COMPUTE I20C FOR I20C(AA,BB,T)
      CALL INTEGI5(AA,BB,T,ANSI5,IERR5)      
      SS=DLOG(T)*ANSI5
      A2=AA*AA
      B2=BB*BB
      A2PB2=A2+B2
      R1=A2/A2PB2
C      R2=B2/A2PB2
      X=T*T*A2PB2
      N=1
      KKODE=1
      MM=1
      TOL=REL
      CALL DEXINT(X, N, KKODE, MM, TOL, EN, NZ, IERR)
      SS=SS+0.5D0*EN(1)*DATAN(AA/BB)/(AA*RTPI)
C     COMPUTE S SERIES
      FNH=0.5D0
      KKODE=1
      MM=60
      TOL=0.5D-15
      NZ=0
      KERR=0
      CALL DHEXINT(X, FNH, KKODE, MM, TOL, EN, NZ, KERR)
      IF(NZ.NE.0) THEN
        ANS=0.0D0
        IERR=1
        RETURN
      ENDIF
      AK=1.0D0
      S=2.0D0*EN(2)
      DH=0.5D0
      FAC=1.0D0
      M=59
      DO 10 K=2,M
        AK=AK*DH/FAC
        AK=AK*R1
        TRM=AK*EN(K+1)/(DH+1.0D0)
        S=S+TRM
        IF (DABS(TRM).LE.TOL*DABS(S)) GOTO 15        
        DH=DH+1.0D0
        FAC=FAC+1.0D0
  10  CONTINUE
       IERR=2
C      WRITE (7,300)
C 300  FORMAT('S-SERIES NOT CONVERGED IN 55 TERMS')
  15  CONTINUE
      SS=SS-0.25D0*S/(RTPI*DSQRT(A2PB2))
      IF(IFLAG.EQ.0) THEN
        ANSI20C=SS
      ELSE
        KKODE=2
        CALL INTEGI19(BB,AA,T,KKODE,ANS19,IERR)
        ARG=AA*T
        ERFAT=DRERF(ARG,KKODE,NZ)
        ARG=BB*T
        ERFBT=DRERF(ARG,KKODE,NZ)
        S=0.5D0*(ANS19+DLOG(T)*ERFAT*ERFBT)*RTPI/BB
        ANSI20C=S-(AA/BB)*SS
      ENDIF
C     I20C = ANSI20C HERE
      IF(KODE.EQ.1) THEN
        ARG=A*T
        KKODE=2
        ERFAT=DRERF(ARG,KKODE,NZ)
        KKODE=1
        REL=0.5D-15
        TRM=RTPI*(ERFAT*DLOG(T)+DGERFC(ARG,KKODE,REL,IERR))/(A+A)
        ANS=TRM-ANSI20C
      ELSE
        ANS=ANSI20C
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGI19(A,B,T,KODE,ANS,IERR)
C----------------------------------------------------------------------
C     REFERENCE: FOLDER19
C
C     ON KODE=1, I19COMP COMPUTES THE INTEGRAL
C
C            I19 =INT ON (0,T) OF ERF(A*W)*ERF(B*W)/W
C
C     AND ON KODE=2, I19COMP COMPUTES THE COMPLEMENTARY INTEGRAL
C
C            I19C=INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)/W
C
C     WHERE   A>0  ,  B>0  , T>0.
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EGH(75),ENH(75),C(75),RTMP(2)
      DATA PI, RTPI/3.14159265358979324D0, 1.77245385090551603D0/
      IERR=0
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        IERR=5
        ANS=0.0D0
        RETURN
      ENDIF
      REL=0.5D-15
C     CHOOSING AA TO BE MIN AND BB TO B MAX OF A AND B
      IF(A.LE.B) THEN
        AA=A
        BB=B
      ELSE
        AA=B
        BB=A
      ENDIF
      AT=AA*T
      BT=BB*T
      KKODE=1
      GERFBT=DGERFC(BT,KKODE,REL,IERRG)
      IF(IERRG.NE.0) THEN
        ANS=0.0D0
        RETURN
      ENDIF      
      ERFAT=DRERF(AT,KODE,NZ)
      IF(NZ.NE.0) THEN
        ANS=0.0D0
        RETURN
      ENDIF
      SS=ERFAT*GERFBT
C     HERE SS IS ERF(AT)*G(BT) ON KODE=1 OR ERFC(AT)*G(BT) ON KODE=2
C     COMPUTE THE REGULAR PART OF I19C 
      DO 5 I=1,3-KODE
        IF(I.EQ.1) THEN
          TD=T
        ELSE
          TD=0.0D0
        ENDIF        
        A2=AA*AA
        B2=BB*BB
        A2PB2=A2+B2
        R1=A2/A2PB2
        R2=B2/A2PB2
        X=TD*TD*A2PB2
        RTAB=DSQRT(A2PB2)
        CALL INTEGI5(AA,BB,TD,ANSI5,IERR5)      
        TRMLN=AA*DLOG(R2)*ANSI5/RTPI
        FNH=1.5D0
        KKODE=1
        MM=60
        TOL=REL
        CALL DGHEXINT(X, FNH, KKODE, MM, TOL, EGH, ENH, NZ, IERR)
        IF(NZ.NE.0) THEN
          ANS=0.0D0
          RETURN
        ENDIF
C       COMBIME THE TERMS OF THE TWO SERIES. THE POWERS OF R1 REDUCE
C       THE EFFECT OF TERMS WHERE LOSSES OF SIGNIFICANCE OCCUR IN THE
C       SUBTRACTION
        PWR=R1
        C(1)=1.0D0
        AK=0.50D0
        BK=1.0D0
        S=0.0D0
        TOL=R1*REL/10.0D0
        DO 10 K=2,60
          CK=C(K-1)*AK/BK
          T1=CK*EGH(K)
          C(K)=CK
          CSUM=0.0D0
          AM=1.0D0
          DO 20 M=1,K-1
            CSUM=CSUM+C(K-M)/AM
            AM=AM+1.0D0
 20       CONTINUE
          T2=ENH(K)*CSUM
          S=S+PWR*(T2-T1)
          IF(DABS(PWR).LE.TOL) GOTO 30
          PWR=PWR*R1
          AK=AK+1.0D0
          BK=BK+1.0D0
 10       CONTINUE
          WRITE(7,190)
 190      FORMAT('IN INTEGI19, DROP THROUGH THE SERIES LOOP')
          PAUSE
  30    CONTINUE
        S=S-EGH(1)
C       RTMP(1) CONTAINS THE EVALUATION OF R AT T
C       RTMP(2) CONTAINS THE EVALUATION OF R AT T=0.0D0
        RTMP(I)=TRMLN+0.5D0*AA*S/(PI*RTAB)
 5    CONTINUE
      IF(KODE.EQ.2) THEN
        ANS=SS+RTMP(1)
      ELSE
        FAT=DFERF(AT,REL,KERR)
        ANS=FAT+RTMP(2)+SS-RTMP(1)
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION QFUNI20(W)
C-----------------------------------------------------------------------
C     QFUNI20 COMPUTES THE INTEGRAND FOR I20 OR I20C FOR THE DQUAD8
C     EVALUATION OF THE INTEGRALS I20 OR I20C IN I20COMP.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /QFUN/ CA,CB,KKODE
      ARG=CB*W
      ERFBT=DRERF(ARG,KKODE,NZ)
      ARG=CA*CA*W*W
      QFUNI20=ERFBT*DLOG(W)*DEXP(-ARG)
      RETURN
      END
      PROGRAM I24COMP
C
C     DONALD E. AMOS   DECEMBER, 2002; MAY, 2006
C
C     REF: FOLDER 24
C
C     ON KODE=1, I24COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGI24
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          I24=INT ON (0,T) OF W*EXP(A^2*W+2*A*B)*ERFC(X)*ERF(C/RW)
C
C     ON KODE=2, I24COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGI24
C     AND DGAUS8 QUADRATURE EVALUATIONS OF THE COMPLEMENTARY FUNCTION
C
C          I24=INT ON (0,T) OF W*EXP(A^2*W+2*A*B)*ERFC(X)*ERFC(C/RW)
C              WITH X=A*RW+B/RW AND RW=SQRT(W)
C
C     AND A, B AND C ARE POSITIVE.
C
C     OUTPUT IS WRITTEN TO FILE I24COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I24COMP.TXT")
      WRITE (7,109)
 109  FORMAT('----------------------------------------------------------
     &-----------------'/
     &'LEGENDS FOR KFORM:'//
     &'KFORM DIGIT=0 MEANS ONE OR MORE PARMETERS ARE ZERO--A SPECIAL FOR
     &MULA IS USED'/
     &'KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0'/
     &'KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT'/
     &'KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3'/
     &'KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I24'/
     &'KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3'//
     &'EXAMPLE:'/
     &'KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED'/
     &'-----------------------------------------------------------------
     &----------')
C
      DO 5 KODE=1,2
C
      WRITE (7,105) KODE
 105  FORMAT(/'THE FOLLOWING TABLE OF COMPARISONS IS FOR KODE = ',I3/)
      WRITE (7,110)
 110  FORMAT('      A         B         C          T         QUAD
     &REL ERR  IERR KFORM')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/10.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            DO 40 IC=1,11,2
              C=DBLE(FLOAT(IC-1))/5.0D0
              CALL INTEGI24(A,B,C,T,KODE,ANS2,I2ERR,KFORM)
              IF((I2ERR.NE.0)) THEN
                WRITE (7,405) A,B,C,T,I2ERR,KFORM
 405            FORMAT( 4D11.4,1X,'IN INTEGI24, I2ERR=',I2,' KFORM=',I2)
                GOTO 40
              ENDIF
C-----------------------------------------------------------------------
C     LEGENDS FOR KFORM:
C
C     KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0
C     KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR VOFT
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3
C     KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I24
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3
C
C     EXAMPLE:
C     KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED
C-----------------------------------------------------------------------
              CALL I24QUAD(A,B,C,T,KODE,QANS,IERR)              
              ERR=DABS(ANS2-QANS)
              IF (QANS.NE.0.0D0) THEN
                ERR=ERR/DABS(QANS)
              ENDIF
              WRITE (7,200) A,B,C,T,QANS,ERR,IERR,KFORM
 200          FORMAT(4D11.4,D12.5,D12.4,2I4)
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
   5  CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQI24(X)
C
C     DQI24 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I24 WITH INTEGI24
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI24/ CA,CB,CC,IKODE
C     A CHANGE OF VARIABLE T=X*X REMOVES THE SQRT FROM THE INTEGRAND
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T)
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFX=ERFX*DEXP(-ARG)
      ARG=CC/X
      NZ=0
      ERFA=DRERF(ARG,IKODE,NZ)
      DQI24=2.0D0*X*X2*ERFX*ERFA
      RETURN
      END
      SUBROUTINE I24QUAD(A,B,C,T,KODE,QANS24,IERR)
C     QUADRATURE FOR THE I24 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI24/ CA,CB,CC,IKODE
      DATA TOL /0.5D-14/
      EXTERNAL DQI24
      IERR=0
      IF((C.EQ.0.0D0).AND.(KODE.EQ.1)) THEN
        QANS24=0.0D0
        IERR=1
        RETURN
      ENDIF
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
C       KODE=2 HERE BECAUSE C=0 AND THE PREVIOUS TEST FAILED
        QANS24=0.5D0*T*T
        IERR=1
        RETURN
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANS24=0.0D0
        IERR=1
        RETURN
      ENDIF
      IF(A.EQ.0.0D0) THEN
        IF(KODE.EQ.1) THEN
          XLIM=MIN(B,C)/6.0D0
        ELSE
          XLIM=DSQRT(B*B+C*C)/6.0D0
        ENDIF
      ELSE
        IF(B.EQ.0.0D0) THEN
          XLIM=C/6.0D0
        ELSE
          IF(KODE.EQ.1) THEN
            XLIM=MIN(B,C)/6.0D0
          ELSE
            XLIM=DSQRT(B*B+C*C)/6.0D0
          ENDIF
        ENDIF
      ENDIF
      CA=A
      CB=B
      CC=C
      IKODE=KODE
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI24,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI24,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS        
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQI24(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQI24,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS24=SS
      RETURN
      END
      SUBROUTINE INTEGI24(A,B,C,T,KODE,ANS24,I2ERR)
C----------------------------------------------------------------------
C     REFERENCE: FOLDER24
C
C     ON KODE=1, INTEGI24 COMPUTES THE INTEGRAL
C
C          I24=INT ON (0,T) OF W*U(A,B,W)*ERF(C/RW)
C
C      AND ON KODE=2 INTEGI24 COMPUTES THE INTEGRAL
C
C          I24=INT ON (0,T) OF W*U(A,B,W)*ERFC(C/RW)
C
C     WHERE U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     AND A, B ARE NON-NGATIVE.
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1),Y(60),YN(60)
      DATA RTPI /1.772453850905516D0/
      I2ERR=0
      RT=DSQRT(T)
      AS=A*A
      BS=B*B
      CS=C*C
      CAPT=1.0D0/RT
      REL=0.5D-14
      ART=A*RT
      IF(ART.LE.1.0D0) THEN
        N0=0
        M=50
        KKODE=1
        X=B*CAPT
        CALL DINERFC(X,N0,KKODE,M,REL,Y,NZ)
        CALL GNSEQ(C,B,CAPT,M,REL,YN)
        S1=Y(3)
        S2=Y(5)
        S3=YN(4)
        S4=YN(5)+YN(5)
        X=ART+ART
        PW=-X
        AK=3.0D0
        J1=0
        J2=0
        J3=0
        J4=0
        JSTOP=0
        KL=M-4
        DO 10 K=2,KL
          IF(J1.EQ.0) THEN
            TRM1=PW*Y(K+2)
            S1=S1+TRM1
            IF(DABS(TRM1).LT.REL*DABS(S1)) THEN
              JSTOP=JSTOP+1
              J1=1
            ENDIF
          ENDIF
          IF(J2.EQ.0) THEN
            TRM2=PW*Y(K+4)
            S2=S2+TRM2
            IF(DABS(TRM2).LT.REL*DABS(S2)) THEN
              JSTOP=JSTOP+1
              J2=1
            ENDIF
          ENDIF
          IF(J3.EQ.0) THEN
            TRM3=PW*YN(K+3)
            S3=S3+TRM3
            IF(DABS(TRM3).LT.REL*DABS(S3)) THEN
              JSTOP=JSTOP+1
              J3=1
            ENDIF
          ENDIF
          IF(J4.EQ.0) THEN
            TRM4=AK*PW*YN(K+4)
            S4=S4+TRM4
            IF(DABS(TRM4).LT.REL*DABS(S4)) THEN
              JSTOP=JSTOP+1
              J4=1
            ENDIF
          ENDIF
          IF((K.EQ.KL).OR.(JSTOP.EQ.4)) GOTO 20
          PW=-X*PW
          AK=AK+1.0D0
 10     CONTINUE
 20     CONTINUE
        ARG=C*CAPT
        ERFX=DRERF(ARG,KODE,NZ)
        SUM=(4.0D0*S1-16.0D0*S2)*T*T*ERFX
        X=(BS+CS)/T
        COEF=8.0D0*C*T*DEXP(-X)/RTPI
        IF(KODE.EQ.1) THEN
          ANS24=SUM+COEF*(B*S3+RT*S4)
        ELSE
          ANS24=SUM-COEF*(B*S3+RT*S4)
        ENDIF
        RETURN
      ENDIF
      VT=DVOFT(A,B,T,REL,IERR,KFORM)
      ARG=C*CAPT
      ERFX=DRERF(ARG,KODE,NZ)
      SUM=T*VT*ERFX
C     CALC T0=T0C
      X=(BS+CS)/T
      N=2
      KKODE=1
      M=1
      TOL=REL
      CALL DEXINT(X, N, KKODE, M, TOL, EN, NZ, IERR) 
      T0=(T+T)*EN(1)/(A*RTPI)
      COEF=2.0D0*(1.0D0/AS+(B+B)/A)
      KKODE=2
      CALL INTEGI1(C,B,CAPT,KKODE,REL,ANSI1,IERR)
      T0=T0-COEF*ANSI1
      CALL INTEGI22(A,B,C,T,ANSI22,I22ERR,KFORM)
      T0=T0+ANSI22/AS
      TRM=T0*C/RTPI
      IF(KODE.EQ.1) THEN
        SUM=SUM+TRM
      ELSE
        SUM=SUM-TRM
      ENDIF
C     CALC T1 OR T1C     
      ARG=C*CAPT      
      ERFX=DRERF(ARG,KODE,NZ)
      ARG=BS/T
      TRM1=T*RT*DEXP(-ARG)*ERFX
      TRM2=C*T*EN(1)/RTPI
      CALL INTEGI1(B,C,CAPT,KODE,REL,ANSI1,IERR)
      TRM3=2.0D0*BS*ANSI1
      COE1=4.0D0/(3.0D0*A*RTPI)
      IF(KODE.EQ.1) THEN
        T1=COE1*(TRM1+TRM2-TRM3)
      ELSE
        T1=COE1*(TRM1-TRM2-TRM3)
      ENDIF
      SUM=SUM-T1
C     CALC T2 OR T2C
      IF(KODE.EQ.1) THEN
        IFORM=2
        IF(IFORM.EQ.1) THEN 
C       IF(C*CAPT.LT.1.0D0) THEN
C         T2 USING J3 AND W3
          ARG=C*CAPT
          JKODE=1
          COE=C
        ELSE
C         T2 USING J3C AND W3C
          ARG=B*CAPT
          JKODE=2
          COE=-B
        ENDIF
        IF(IFORM.EQ.1) THEN
          ERFX=DRERF(ARG,JKODE,NZ)
          IKODE=1
          CALL DIERFC(ARG,IKODE,ANS,IERR)
          FJ3=0.5D0*ERFX*T+COE*ANS*RT
        ELSE
          N0=2
          M=1
          IKODE=1
          REL=0.5D-14
          CALL DINERFC(ARG,N0,IKODE,M,REL,Y,NZ)
          FJ3=2.0D0*T*Y(1)
        ENDIF
        CALL INTEGW3(B,C,CAPT,IFORM,REL,ANSW3,IERR)
        T2=COEF*(FJ3-ANSW3)
      ELSE
        CALL INTEGW3(B,C,CAPT,KODE,REL,ANSW3,IERR)
        T2=COEF*ANSW3
      ENDIF
      SUM=SUM+T2
C     CALC T3 OR T3C
      CALL INTEGI21(A,B,C,T,KODE,ANS21,I21ERR,KFORM)
      T3=ANS21/AS
      ANS24=SUM-T3
      RETURN
      END
      PROGRAM J24COMP
C
C     DONALD E. AMOS   DECEMBER, 2002; MAY, 2006
C
C     REF: FOLDER 24
C
C     J24COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGJ24
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          J24=INT ON (0,T) OF W*EXP(A^2*W+2*A*B)*ERFC(X)
C                      X=A*RT+B/RT, RT=SQRT(W)
C
C     AND A AND B ARE NON-NGATIVE.
C
C     OUTPUT IS WRITTEN TO FILE J24COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="J24COMP.TXT")
C
      WRITE (7,110)
 110  FORMAT('      A         B          T         QUAD        REL ERR
     &IERR')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/5.0D0
        DO 20 IA=1,11,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,11,2
            B=DBLE(FLOAT(IB-1))/2.0D0
            CALL INTEGJ24(A,B,T,ANS2,J2ERR)
            IF((J2ERR.NE.0)) THEN
              WRITE (7,405) A,B,T,J2ERR
 405          FORMAT( 3D11.4,1X,'IN INTEGJ24, J2ERR=',I2)
              GOTO 30
            ENDIF
            CALL J24QUAD(A,B,T,QANS,IERR)              
            ERR=DABS(ANS2-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,T,QANS,ERR,IERR
 200        FORMAT(3D11.4,D12.5,D12.4,I4)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQJ24(X)
C
C     DQJ24 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF J24 WITH INTEGJ24
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ24/ CA,CB
C     A CHANGE OF VARIABLE TAU=X*X REMOVES THE SQRT FROM THE INTEGRAND
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T)
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFX=ERFX*DEXP(-ARG)
      DQJ24=2.0D0*ERFX*X2*X
      RETURN
      END
      SUBROUTINE J24QUAD(A,B,T,QANS24,IERR)
C     QUADRATURE FOR THE J24 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFJ24/ CA,CB
      DATA TOL /0.5D-14/
      EXTERNAL DQJ24
      IERR=0
      IF(B.EQ.0.0D0) THEN
        XLIM=4.0D0
      ELSE
        XLIM=B/6.0D0
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANS24=0.0D0
        IERR=1
        RETURN
      ENDIF
      CA=A
      CB=B
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQJ24,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQJ24,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQJ24(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQJ24,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS24=SS
      RETURN
      END
      SUBROUTINE INTEGJ24(A,B,T,ANS24,I2ERR)
C----------------------------------------------------------------------
C     REFERENCE: FOLDER24
C
C     INTEGJ24 COMPUTES THE INTEGRAL
C
C          J24=INT ON (0,T) OF W*U(A,B,W)
C
C     WITH U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     AND A, B ARE NON-NGATIVE.
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Y(50),EN(1)
      DATA REL / 0.5D-14/
      DATA RTPI /1.772453850905516D0/
      I2ERR=0
      RT=DSQRT(T)
      AS=A*A
      BS=B*B
      CAPT=1.0D0/RT
      ART=A*RT
      IF(ART.LE.1.0D0) THEN
        N0=4
        KODE=1
        N=50
        X=B*CAPT
        CALL DINERFC(X,N0,KODE,N,REL,Y,NZ)
        SUM=Y(1)
        X=ART+ART
        PW=-X
        DO 10 K=2,N
          TRM=PW*Y(K)
          SUM=SUM+TRM
          IF(DABS(TRM).LT.REL*DABS(SUM)) GOTO 20
          PW=-X*PW
 10     CONTINUE
 20     CONTINUE
        TOL=REL
        VT=DVOFT(A,B,T,TOL,IERR,KFORM)
        ANS24=T*(VT-16.0D0*T*SUM)
        RETURN
      ENDIF
      TOL=REL
      VT=DVOFT(A,B,T,TOL,IERR,KFORM)
      SUM=(T-1.0D0/AS)*VT
      IFORM=3
      ARG=B*CAPT
      N0=1
      KKODE=1
      N=3
      TOL=REL
      CALL DINERFC(ARG,N0,KKODE,N,TOL,Y,NZ)
      IF(IFORM.EQ.3) THEN
        ANS24=SUM-8.0D0*T*RT*Y(3)/A+4.0D0*T*Y(2)/AS
        RETURN
      ENDIF
      IF (IFORM.EQ.1) THEN
        COEF=4.0D0/(3.0D0*A*RTPI)
        TRM=(T*DEXP(-ARG*ARG)-2.0D0*BS*RTPI*Y(1))*RT
        SUM=SUM-COEF*TRM
        COEF=4.0D0*(1.0D0/AS+(B+B)/A)*T
        ANS24=SUM+COEF*Y(2)
        RETURN
      ELSE
        COEF=2.0D0/(A*RTPI)
        FNH=2.5D0
        KKODE=1
        M=1
        ARG2=ARG*ARG
        CALL DHEXINT(ARG2, FNH, KKODE, M, TOL, EN, NZ, IERR)
        TRM=T*RT*EN(1)
        SUM=SUM-COEF*TRM
        COEF=4.0D0*(1.0D0/AS+(B+B)/A)*T
        ANS24=SUM+COEF*Y(2)
        RETURN
      ENDIF
      END
      PROGRAM V24COMP
C
C     DONALD E. AMOS   DECEMBER, 2002; MAY, 2006
C
C     REF: FOLDER 24
C
C     V24COMP COMPARES THE PROCEDURE FROM SUBROUTINE INTEGV24
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          V24=INT ON (0,T) OF V(A,B,W)
C
C     WHERE V(A,B,W)=DVOFT=INT ON (0,T) OF U(A,B,W)    WITH
C           U(A,B,T)=EXP(A*A*T+2*A*B)*ERFC(A*RT+B/RT), RT=SQRT(T)
C
C     AND A, B ARE NON-NGATIVE.
C
C     OUTPUT IS WRITTEN TO FILE V24COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="V24COMP.TXT")
C
      WRITE (7,110)
 110  FORMAT('      A         B          T         QUAD        REL ERR
     &IERR')
      DO 10 IT=1,21,2
        T=DBLE(FLOAT(IT))/5.0D0
        DO 20 IA=1,21,2
          A=DBLE(FLOAT(IA-1))/5.0D0
          DO 30 IB=1,21,2
            B=DBLE(FLOAT(IB-1))/5.0D0
            CALL INTEGV24(A,B,T,ANS2,J2ERR)
            IF((J2ERR.NE.0)) THEN
              WRITE (7,405) A,B,T,J2ERR
 405          FORMAT( 3D11.4,1X,'IN INTEGV24, J2ERR=',I2)
              GOTO 30
            ENDIF
            CALL V24QUAD(A,B,T,QANS,IERR)              
            ERR=DABS(ANS2-QANS)
            IF (QANS.NE.0.0D0) THEN
              ERR=ERR/DABS(QANS)
            ENDIF
            WRITE (7,200) A,B,T,QANS,ERR,IERR
 200        FORMAT(3D11.4,D12.5,D12.4,I4)
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE
      END
      DOUBLE PRECISION FUNCTION DQV24(X)
C
C     DQV24 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF V24 WITH INTEGV24
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFV24/ CA,CB
C     A CHANGE OF VARIABLE TAU=X*X REMOVES THE SQRT FROM THE INTEGRAND
C     THE UPPER LIMIT ON THE INTEGRAL IS THEN DSQRT(T)
      DATA REL /0.5D-14/
      CCA=CA
      CCB=CB
      X2=X*X
      DQV24=2.0D0*X*DVOFT(CCA,CCB,X2,REL,IERR,KFORM)
      RETURN
      END
      SUBROUTINE V24QUAD(A,B,T,QANS24,IERR)
C     QUADRATURE FOR THE V24 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFV24/ CA,CB
      DATA TOL /0.50D-14/
      EXTERNAL DQV24
      IERR=0
      IF(B.EQ.0.0D0) THEN
        XLIM=4.0D0
      ELSE
        XLIM=B/6.0D0
      ENDIF
      IF(T.EQ.0.0D0) THEN
        QANS24=0.0D0
        IERR=1
        RETURN
      ENDIF
      CA=A
      CB=B
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQV24,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQV24,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQV24(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQV24,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS24=SS
      RETURN
      END
      SUBROUTINE INTEGV24(A,B,T,ANS24,I2ERR)
C----------------------------------------------------------------------
C     REFERENCE: FOLDER24
C     
C     INTEGV24 COMPUTES THE INTEGRAL
C
C          V24=INT ON (0,T) OF V(A,B,W)
C
C     WHERE V(A,B,W)=DVOFT=INT ON (0,T) OF U(A,B,W)    WITH
C           U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     AND A, B ARE NON-NGATIVE.
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Y(50),EN(1)
      DATA RTPI /1.772453850905516D0/
      I2ERR=0
      RT=DSQRT(T)
      AS=A*A
      BS=B*B
      CAPT=1.0D0/RT
      REL=0.5D-14
      ART=A*RT
      IF(ART.LE.1.0D0) THEN
        N0=4
        KODE=1
        N=50
        X=B*CAPT
        CALL DINERFC(X,N0,KODE,N,REL,Y,NZ)
        SUM=Y(1)
        X=ART+ART
        PW=-X
        DO 10 K=2,N
          TRM=PW*Y(K)
          SUM=SUM+TRM
          IF(DABS(TRM).LT.REL*DABS(SUM)) GOTO 20
          PW=-X*PW
 10     CONTINUE
 20     CONTINUE
        ANS24=16.0D0*T*T*SUM
        RETURN
      ENDIF
      VT=DVOFT(A,B,T,REL,IERR,KFORM)
      SUM=VT/AS
      IFORM=3
      ARG=B*CAPT
      N0=1
      KKODE=1
      N=3
      REL=0.5D-14
      CALL DINERFC(ARG,N0,KKODE,N,REL,Y,NZ)
      IF (IFORM.EQ.3) THEN
        ANS24=SUM+8.0D0*T*RT*Y(3)/A-4.0D0*T*Y(2)/AS
        RETURN
      ENDIF
      IF (IFORM.EQ.1) THEN
        COEF=4.0D0/(3.0D0*A*RTPI)
        TRM=(T*DEXP(-ARG*ARG)-2.0D0*BS*RTPI*Y(1))*RT
        SUM=SUM+COEF*TRM
        COEF=4.0D0*(1.0D0/AS+(B+B)/A)*T
        ANS24=SUM-COEF*Y(2)
        RETURN
      ENDIF
      IF (IFORM.EQ.2) THEN
        COEF=2.0D0/(A*RTPI)
        FNH=2.5D0
        KKODE=1
        M=1
        ARG2=ARG*ARG
        CALL DHEXINT(ARG2, FNH, KKODE, M, REL, EN, NZ, IERR)
        TRM=T*RT*EN(1)
        SUM=SUM+COEF*TRM
        COEF=4.0D0*(1.0D0/AS+(B+B)/A)*T
        ANS24=SUM-COEF*Y(2)
        RETURN
      ENDIF
      END
      PROGRAM I25COMP
C
C     DONALD E. AMOS   JANUARY, 2003;  MAY, 2006
C
C     REF: FOLDER 25
C
C     I25COMP COMPARES THE NON-SYMMETRIC FORMULA CODED IN INTEGI25
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          I25=INT ON (0,T) OF U(A,B,W)*U(C,D,W)
C
C     WITH U(A,B,T)=EXP(A*A*T+2*A*B)*ERFC(A*RT+B/RT), RT=SQRT(T)
C
C     AND A, B ARE NON-NGATIVE.
C
C     OUTPUT IS WRITTEN TO FILE I25COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I25COMP.TXT")
      WRITE (7,110)
 110  FORMAT('      A         B         C         D         T       QUAD
     &    REL ERR  IERR')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/5.0D0
        PRINT *,'IT = ',IT
        DO 20 IA=1,5
          A=DBLE(FLOAT(IA))/2.0D0
C        PRINT *,' IA = ',IA

          DO 30 IB=1,5
            B=DBLE(FLOAT(IB))/2.0D0
C        PRINT *,'   IB = ',IB

            DO 40 IC=1,5
              C=DBLE(FLOAT(IC))/2.0D0
C        PRINT *,'     IC = ',IC

              DO 50 ID=1,5
                D=DBLE(FLOAT(ID))/2.0D0
                CALL INTEGI25(A,B,C,D,T,ANS25,I2ERR)
C        PRINT *,'       ID = ',ID
                IF((I2ERR.NE.0)) THEN
                  WRITE (7,405) A,B,C,D,T,I2ERR
 405              FORMAT( 5D11.4,1X,'IN INTEGI25, I2ERR=',I2)
                  GOTO 50
                ENDIF
                CALL I25QUAD(A,B,C,D,T,QANS,IERR)
                ERR=DABS(ANS25-QANS)
                IF (QANS.NE.0.0D0) THEN
                  ERR=ERR/DABS(QANS)
                ENDIF
                WRITE (7,200) A,B,C,D,T,QANS,ERR,IERR
 200            FORMAT(5D10.3,D10.3,D10.3,I4)
   50         CONTINUE
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQI25(X)
C
C     DQI25 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I25 WITH INTEGI25
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI25/ CA,CB,CC,CD
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFAB=ERFX*DEXP(-ARG)
      ARG=CC*X+CD/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      ARG=CD*CD/X2
      ERFCD=ERFX*DEXP(-ARG)
      DQI25=2.0D0*ERFAB*ERFCD*X
      RETURN
      END
      SUBROUTINE I25QUAD(A,B,C,D,T,QANS25,IERR)
C     QUADRATURE FOR THE I25 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFI25/ CA,CB,CC,CD
      DATA TOL /0.5D-14/
      EXTERNAL DQI25
      IERR=0
      IF(T.EQ.0.0D0) THEN
        QANS25=0.0D0
        IERR=1
        RETURN
      ENDIF
      XLIM=DSQRT(B*B+D*D)/6.0D0
      CA=A
      CB=B
      CC=C
      CD=D
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI25,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI25,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQI25(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQI25,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS25=SS
      RETURN
      END
      SUBROUTINE INTEGI25(A,B,C,D,T,ANS25,I2ERR)
C
C     DONALD E. AMOS, JANUARY, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 25
C
C     INTEGI25 COMPUTES
C
C        ANS25=INT ON (0,T) OF U(A,B,W)*U(C,D,W)
C
C        U(A,B,W)= EXP(A*A*W+2*A*B)*ERFC(A*RT+B/RT), RT=SQRT(W)
C
C     WHERE A, B, C, D  ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C           I2ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW, (B^2+D^2)/T TOO LARGE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(2)
      DATA RTPI /1.772453850905516D0/
      I2ERR=0
      IF(A.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(D.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANS25=0.0D0
         RETURN
      ENDIF
      OPT=1
C     SINCE THE INTEGRAL IS SYMMETRIC IN THE PAIRS (A,B), (C,D) WE CAN
C     EXCHANGE THE PAIRS IF WE WANT THE A PARAMETER TO BE GREATER THAN
C     THE C PARAMETER. THIS IS WHAT OPT=1 DOES.
C     OPT=1, RUNCODE WITH AA=MAX(A,C), CC=MIN(A,C)
C     OPT=2, RUNCODE WITH NO CHANGE IN PARAMETERS
      IF(OPT.EQ.1) THEN
        IF(C.GT.A)THEN
          AA=C
          BB=D
          CC=A
          DD=B
        ELSE
          AA=A
          BB=B
          CC=C
          DD=D
        ENDIF
      ELSE
        AA=A
        BB=B
        CC=C
        DD=D
      ENDIF
      REL=0.5D-14
      RT=DSQRT(T)
      CAPT=1.0D0/RT
      AS=AA*AA
      BS=BB*BB
      CS=CC*CC
      DS=DD*DD
      CAPX=(BS+DS)/T
      IF(CAPX.GT.667.0D0) THEN
        ANS25=0.0D0
        I2ERR=5
        RETURN
      ENDIF
      VABT=DVOFT(AA,BB,T,REL,IVERR,IFORM)
      ARG=CC*RT+DD/RT      
      KODE=3
      NZ=0
      ERFU=DRERF(ARG,KODE,NZ)
      ARG=DS/T
      UCDT=ERFU*DEXP(-ARG)
      SS=AS*VABT*UCDT
      N=1
      KKODE=1
      M=2
      TOL=REL
      NZ=0
      IERR=0
      CALL DEXINT(CAPX, N, KKODE, M, TOL, EN, NZ, IERR)
      CAB=1.0D0+(BB+BB)*AA
      KODE=2
      CALL INTEGI1(DD,BB,CAPT,KODE,REL,ANSI1C,IERR)
      CALL INTEGI22(AA,BB,DD,T,ANSI22,I2ERR,KFORM)
      R1ABDT=AA*T*EN(2)/RTPI-CAB*ANSI1C
      R1ABDT=R1ABDT+R1ABDT+ANSI22
      CALL INTEGI5(DD,BB,CAPT,ANSI5,IERR)
      CALL INTEGJ21(AA,BB,DD,T,ANSJ21,I2ERR,KFORM)
      R2ABDT=AA*EN(1)/RTPI-CAB*ANSI5
      R2ABDT=R2ABDT+R2ABDT+ANSJ21
      SS=SS+(CC*R1ABDT-DD*R2ABDT)/RTPI
      CALL INTEGJ22(CC,DD,BB,T,ANSJ22,I2ERR,KFORM)
      KODE=2
      CALL INTEGI21(CC,DD,BB,T,KODE,ANSI21C,I2ERR,KFORM)
      SS=SS-(AA+AA)*CS*ANSJ22/RTPI+CS*CAB*ANSI21C
      ANS25=SS/(AS+CS)
      RETURN
      END
      PROGRAM I26COMP
C
C     DONALD E. AMOS   APRIL, 2003;  MAY, 2006
C
C     REF: FOLDER 26
C
C     I26COMP COMPARES THE SYMMETRIC FORMULA CODED IN INTEGI26
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          I26=INT ON (0,T) OF W*U(A,B,W)*U(C,D,W)
C
C     WITH U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RT=SQRT(W)
C
C     AND A, B ARE NON-NGATIVE.
C
C     OUTPUT IS WRITTEN TO FILE I26COMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I26COMP.TXT")
      WRITE (7,110)
 110  FORMAT('      A         B         C         D         T       QUAD
     &    REL ERR  IERR')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/5.0D0
        PRINT *,'IT = ',IT
        DO 20 IA=1,5
          A=DBLE(FLOAT(IA))/2.0D0
C        PRINT *,' IA = ',IA

          DO 30 IB=1,5
            B=DBLE(FLOAT(IB))/2.0D0
C        PRINT *,'   IB = ',IB

            DO 40 IC=1,5
              C=DBLE(FLOAT(IC))/2.0D0
C        PRINT *,'     IC = ',IC

              DO 50 ID=1,5
                D=DBLE(FLOAT(ID))/2.0D0
                CALL INTEGI26(A,B,C,D,T,ANS26,I2ERR)
C        PRINT *,'       ID = ',ID
                IF((I2ERR.NE.0)) THEN
                  WRITE (7,405) A,B,C,D,T,I2ERR
 405            FORMAT( 5D11.4,1X,'IN INTEGI26, I2ERR=',I2)
                  GOTO 50
                ENDIF
                CALL I26QUAD(A,B,C,D,T,QANS,IERR)
                ERR=DABS(ANS26-QANS)
                IF (QANS.NE.0.0D0) THEN
                  ERR=ERR/DABS(QANS)
                ENDIF
                WRITE (7,200) A,B,C,D,T,QANS,ERR,IERR
 200            FORMAT(5D10.3,D10.3,D10.3,I4)
   50         CONTINUE
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQI26(X)
C
C     DQI26 COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I26 WITH INTEGI26
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI26/ CA,CB,CC,CD
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFAB=ERFX*DEXP(-ARG)
      ARG=CC*X+CD/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      ARG=CD*CD/X2
      ERFCD=ERFX*DEXP(-ARG)
      DQI26=2.0D0*ERFAB*ERFCD*X*X2
      RETURN
      END
      SUBROUTINE I26QUAD(A,B,C,D,T,QANS26,IERR)
C     QUADRATURE FOR THE I26 INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI26/ CA,CB,CC,CD
      DATA TOL /0.5D-14/
      EXTERNAL DQI26
      IERR=0
      IF(T.EQ.0.0D0) THEN
        QANS26=0.0D0
        IERR=1
        RETURN
      ENDIF
      XLIM=DSQRT(B*B+D*D)/6.0D0
      CA=A
      CB=B
      CC=C
      CD=D
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI26,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI26,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQI26(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQI26,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS26=SS
      RETURN
      END
      SUBROUTINE INTEGI26(A,B,C,D,T,ANS26,I26ERR)
C
C     DONALD E. AMOS, APRIL, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 26
C
C     INTEGI26 COMPUTES ANS26 BY THE SYMMETRIC FORMULA FOR
C
C        I26=INT ON (0,T) OF W*U(A,B,W)*U(C,D,W)
C
C        U(A,B,W)= EXP(A*A*W+2*A*B)*ERFC(A*RT+B/RT), RT=SQRT(W)
C
C     WHERE A, B, C, D  ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C          I26ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW IN SOME SUBROUTINE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI /1.772453850905516D0/
      I26ERR=0
      IF(A.LT.0.0D0)THEN
         I26ERR=4
         ANS26=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         I26ERR=4
         ANS26=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         I26ERR=4
         ANS26=0.0D0
         RETURN
      ENDIF
      IF(D.LT.0.0D0)THEN
         I26ERR=4
         ANS26=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         I26ERR=4
         ANS26=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANS26=0.0D0
         RETURN
      ENDIF
      RT=DSQRT(T)
      AS=A*A
      BS=B*B
      CS=C*C
      DS=D*D
      ARG=A*RT+B/RT      
      KODE=3
      NZ=0
      ERFU=DRERF(ARG,KODE,NZ)
      ARG=BS/T
      UABT=ERFU*DEXP(-ARG)
      ARG=C*RT+D/RT      
      KODE=3
      NZ=0
      ERFU=DRERF(ARG,KODE,NZ)
      ARG=DS/T
      UCDT=ERFU*DEXP(-ARG)
      SS=T*UABT*UCDT
      CALL INTEGI25(A,B,C,D,T,ANSI25,I2ERR)
      I26ERR=I26ERR+I2ERR
      IF(I26ERR.NE.0) THEN
        ANS26=0.0D0
        RETURN
      ENDIF
      CALL INTEGI22(A,B,D,T,AI22AB,I2ERR,KFORM)
      I26ERR=I26ERR+I2ERR
      IF(I26ERR.NE.0) THEN
        ANS26=0.0D0
        RETURN
      ENDIF
      CALL INTEGJ22(A,B,D,T,AJ22AB,I2ERR,KFORM)
      I26ERR=I26ERR+I2ERR
      IF(I26ERR.NE.0) THEN
        ANS26=0.0D0
        RETURN
      ENDIF
      CALL INTEGI22(C,D,B,T,AI22CD,I2ERR,KFORM)
      I26ERR=I26ERR+I2ERR
      IF(I26ERR.NE.0) THEN
        ANS26=0.0D0
        RETURN
      ENDIF
      CALL INTEGJ22(C,D,B,T,AJ22CD,I2ERR,KFORM)
      I26ERR=I26ERR+I2ERR
      IF(I26ERR.NE.0) THEN
        ANS26=0.0D0
        RETURN
      ENDIF
      SS=SS-ANSI25+(C*AJ22AB-D*AI22AB+A*AJ22CD-B*AI22CD)/RTPI
      ANS26=SS/(AS+CS)
      RETURN
      END
      SUBROUTINE INTEGI25(A,B,C,D,T,ANS25,I2ERR)
C
C     DONALD E. AMOS, JANUARY, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 25
C
C     INTEGI25 COMPUTES
C
C        ANS25=INT ON (0,T) OF U(A,B,W)*U(C,D,W)
C
C        U(A,B,W)= EXP(A*A*W+2*A*B)*ERFC(A*RT+B/RT), RT=SQRT(W)
C
C     WHERE A, B, C, D  ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C           I2ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW, (B^2+D^2)/T TOO LARGE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(2)
      DATA RTPI /1.772453850905516D0/
      I2ERR=0
      IF(A.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(D.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANS25=0.0D0
         RETURN
      ENDIF
      OPT=1
C     SINCE THE INTEGRAL IS SYMMETRIC IN THE PAIRS (A,B), (C,D) WE CAN
C     EXCHANGE THE PAIRS IF WE WANT THE A PARAMETER TO BE GREATER THAN
C     THE C PARAMETER. THIS IS WHAT OPT=1 DOES.
C     OPT=1, RUNCODE WITH AA=MAX(A,C), CC=MIN(A,C)
C     OPT=2, RUNCODE WITH NO CHANGE IN PARAMETERS
      IF(OPT.EQ.1) THEN
        IF(C.GT.A)THEN
          AA=C
          BB=D
          CC=A
          DD=B
        ELSE
          AA=A
          BB=B
          CC=C
          DD=D
        ENDIF
      ELSE
        AA=A
        BB=B
        CC=C
        DD=D
      ENDIF
      REL=0.5D-14
      RT=DSQRT(T)
      CAPT=1.0D0/RT
      AS=AA*AA
      BS=BB*BB
      CS=CC*CC
      DS=DD*DD
      CAPX=(BS+DS)/T
      IF(CAPX.GT.667.0D0) THEN
        ANS25=0.0D0
        I2ERR=5
        RETURN
      ENDIF
      VABT=DVOFT(AA,BB,T,REL,IVERR,IFORM)
      ARG=CC*RT+DD/RT      
      KODE=3
      NZ=0
      ERFU=DRERF(ARG,KODE,NZ)
      ARG=DS/T
      UCDT=ERFU*DEXP(-ARG)
      SS=AS*VABT*UCDT
      N=1
      KKODE=1
      M=2
      TOL=REL
      NZ=0
      IERR=0
      CALL DEXINT(CAPX, N, KKODE, M, TOL, EN, NZ, IERR)
      CAB=1.0D0+(BB+BB)*AA
      KODE=2
      CALL INTEGI1(DD,BB,CAPT,KODE,REL,ANSI1C,IERR)
      CALL INTEGI22(AA,BB,DD,T,ANSI22,I2ERR,KFORM)
      R1ABDT=AA*T*EN(2)/RTPI-CAB*ANSI1C
      R1ABDT=R1ABDT+R1ABDT+ANSI22
      CALL INTEGI5(DD,BB,CAPT,ANSI5,IERR)
      CALL INTEGJ21(AA,BB,DD,T,ANSJ21,I2ERR,KFORM)
      R2ABDT=AA*EN(1)/RTPI-CAB*ANSI5
      R2ABDT=R2ABDT+R2ABDT+ANSJ21
      SS=SS+(CC*R1ABDT-DD*R2ABDT)/RTPI
      CALL INTEGJ22(CC,DD,BB,T,ANSJ22,I2ERR,KFORM)
      KODE=2
      CALL INTEGI21(CC,DD,BB,T,KODE,ANSI21C,I2ERR,KFORM)
      SS=SS-(AA+AA)*CS*ANSJ22/RTPI+CS*CAB*ANSI21C
      ANS25=SS/(AS+CS)
      RETURN
      END
      PROGRAM I26ACOMP
C
C     DONALD E. AMOS   APRIL, 2003;  MAY, 2006
C
C     REF: FOLDER 26
C
C     I26ACOMP COMPARES THE SYMMETRIC FORMULA CODED IN INTEGI26A
C     AND DGAUS8 QUADRATURE EVALUATIONS OF
C
C          I26A=INT ON (0,T) OF W*U(A,B,W)*U(C,D,W)
C
C     WITH U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RT=SQRT(W)
C
C     AND A, B ARE NON-NGATIVE.
C
C     OUTPUT IS WRITTEN TO FILE I26ACOMP.TXT
C----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      OPEN(UNIT=7,FILE="I26ACOMP.TXT")
      WRITE (7,110)
 110  FORMAT('      A         B         C         D         T       QUAD
     &    REL ERR  IERR')
      DO 10 IT=1,11,2
        T=DBLE(FLOAT(IT))/5.0D0
        PRINT *,'IT = ',IT
        DO 20 IA=1,5
          A=DBLE(FLOAT(IA))/2.0D0
C        PRINT *,' IA = ',IA

          DO 30 IB=1,5
            B=DBLE(FLOAT(IB))/2.0D0
C        PRINT *,'   IB = ',IB

            DO 40 IC=1,5
              C=DBLE(FLOAT(IC))/2.0D0
C        PRINT *,'     IC = ',IC

              DO 50 ID=1,5
                D=DBLE(FLOAT(ID))/2.0D0
                CALL INTEG26A(A,B,C,D,T,ANS26,I2ERR)
C        PRINT *,'       ID = ',ID
                IF((I2ERR.NE.0)) THEN
                  WRITE (7,405) A,B,C,D,T,I2ERR
 405            FORMAT( 5D11.4,1X,'IN INTEGI26A, I2ERR=',I2)
                  GOTO 50
                ENDIF
                CALL I26AQUAD(A,B,C,D,T,QANS,IERR)
                ERR=DABS(ANS26-QANS)
                IF (QANS.NE.0.0D0) THEN
                  ERR=ERR/DABS(QANS)
                ENDIF
                WRITE (7,200) A,B,C,D,T,QANS,ERR,IERR
 200            FORMAT(5D10.3,D10.3,D10.3,I4)
   50         CONTINUE
   40       CONTINUE 
   30     CONTINUE
   20   CONTINUE
   10 CONTINUE 
      END
      DOUBLE PRECISION FUNCTION DQI26A(X)
C
C     DQI26A COMPUTES THE INTEGRAND FOR THE QUADRATURE COMPARISON
C     OF I26A WITH INTEG26A
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI26A/ CA,CB,CC,CD
      ARG=CA*X+CB/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      X2=X*X
      ARG=CB*CB/X2
      ERFAB=ERFX*DEXP(-ARG)
      ARG=CC*X+CD/X
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      ARG=CD*CD/X2
      ERFCD=ERFX*DEXP(-ARG)
      DQI26A=2.0D0*ERFAB*ERFCD*X*X2
      RETURN
      END
      SUBROUTINE I26AQUAD(A,B,C,D,T,QANS26,IERR)
C     QUADRATURE FOR THE I26A INTEGRAL. A CHANGE OF VARIABLES TAU=X*X
C     IS USED. THE UPPER LIMIT ON THE X INTEGRAL IS SQRT(T)=RT.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/ CFI26A/ CA,CB,CC,CD
      DATA TOL /0.5D-14/
      EXTERNAL DQI26A
      IERR=0
      IF(T.EQ.0.0D0) THEN
        QANS26=0.0D0
        IERR=1
        RETURN
      ENDIF
      XLIM=DSQRT(B*B+D*D)/6.0D0
      CA=A
      CB=B
      CC=C
      CD=D
      RT=DSQRT(T)
      JD=10
      INIT=-JD
      X1=0.0D0
      REL=TOL
      SS=0.0D0
      IF (RT.LE.XLIM) THEN
        SIG=RT/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI26A,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
      ELSE
        SIG=XLIM/DBLE(FLOAT(JD))
        CALL DQUAD8(DQI26A,INIT,X1,SIG,REL,X2,QANS,IERR)
        SS=SS+QANS
        INIT=-JD
        X1=X2
        SIG=(RT-X2)/DBLE(FLOAT(JD))
C       TEST FOR RT TOO CLOSE TO X2 FOR AN ACCURATE QUADRATURE
C       (SEE THE ERROR MESSAGE AND TEST IN DGAUS8)
        XMID=X1+0.5D0*SIG
        FMID=DQI26A(XMID)
        IF(DABS(FMID*SIG).LT.2.0D0*TOL*DABS(SS)) GOTO 20
        REL=TOL
        CALL DQUAD8(DQI26A,INIT,X1,SIG,REL,X2,QANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        SS=SS+QANS
  20    CONTINUE
      ENDIF
      QANS26=SS
      RETURN
      END
      SUBROUTINE INTEG26A(A,B,C,D,T,ANS26A,I26AERR)
C
C     DONALD E. AMOS, APRIL, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 26
C
C     INTEG26A COMPUTES THE NON-SYMMETRIC FORM OF I26 IN FOLDER 26 FOR
C
C        I26=INT ON (0,T) OF W*U(A,B,W)*U(C,D,W)
C
C        U(A,B,W)= EXP(A*A*W+2*A*B)*ERFC(A*RT+B/RT), RT=SQRT(W)
C
C     WHERE A, B, C, D  ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C          I26AERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW IN SOME SUBROUTINE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(2)
      DATA RTPI /1.772453850905516D0/
      I26AERR=0
      IF(A.LT.0.0D0)THEN
         I26AERR=4
         ANS26A=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         I26AERR=4
         ANS26A=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         I26AERR=4
         ANS26A=0.0D0
         RETURN
      ENDIF
      IF(D.LT.0.0D0)THEN
         I26AERR=4
         ANS26A=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         I26AERR=4
         ANS26A=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANS26A=0.0D0
         RETURN
      ENDIF
      OPT=2
C     SINCE THE INTEGRAL IS SYMMETRIC IN THE PAIRS (A,B), (C,D) WE CAN
C     EXCHANGE THE PAIRS IF WE WANT THE A PARAMETER TO BE GREATER THAN
C     THE C PARAMETER. THIS IS WHAT OPT=1 DOES.
C     OPT=1, RUNCODE WITH AA=MAX(A,C), CC=MIN(A,C)
C     OPT=2, RUNCODE WITH NO CHANGE IN PARAMETERS
      IF(OPT.EQ.1) THEN
        IF(C.GT.A)THEN
          AA=C
          BB=D
          CC=A
          DD=B
        ELSE
          AA=A
          BB=B
          CC=C
          DD=D
        ENDIF
      ELSE
        AA=A
        BB=B
        CC=C
        DD=D
      ENDIF
      REL=0.5D-14
      RT=DSQRT(T)
      CAPT=1.0D0/RT
      AS=AA*AA
      BS=BB*BB
      CS=CC*CC
      DS=DD*DD
      CAPX=(BS+DS)/T
      IF(CAPX.GT.667.0D0) THEN
        ANS26A=0.0D0
        I2ERR=5
        RETURN
      ENDIF
      VABT=DVOFT(AA,BB,T,REL,IVERR,IFORM)
      ARG=CC*RT+DD/RT      
      KODE=3
      NZ=0
      ERFU=DRERF(ARG,KODE,NZ)
      ARG=DS/T
      UCDT=ERFU*DEXP(-ARG)
      SS=AS*T*VABT*UCDT
      N=2
      KKODE=1
      M=2
      TOL=REL
      NZ=0
      IERR=0
      CALL DEXINT(CAPX, N, KKODE, M, TOL, EN, NZ, IERR)
      CAB=1.0D0+2.0D0*AA*BB
      CABP=CAB+CAB
      T2=T*T
      TRT=T*RT
      EX=DEXP(-CAPX)
      ARG=BB/RT
      KODE=3
      ERFB=DRERF(ARG,KODE,NZ)
      KODE=2
      CALL INTEGI1(DD,BB,CAPT,KODE,REL,ANSI1DB,IERR)
      R12DBT=(EX*ERFB*TRT-BB*T*EN(1)/RTPI-(DS+DS)*ANSI1DB)/3.0D0
      CALL INTEGJ22(AA,BB,DD,T,ANSJ22,J22ERR,KFORM)
      R1ABDT=(AA+AA)*T2*EN(2)/RTPI-CABP*R12DBT+ANSJ22
      CALL INTEGI22(AA,BB,DD,T,ANSI22,I22ERR,KFORM)
      R2ABDT=(AA+AA)*T*EN(1)/RTPI-CABP*ANSI1DB+ANSI22
      SS=SS+(CC*R1ABDT-DD*R2ABDT)/RTPI
      CALL INTEGI25(AA,BB,CC,DD,T,ANSI25,I2ERR)
      CALL INTEGJ22(CC,DD,BB,T,ANSJ22,J22ERR,KFORM)
      KODE=2
      CALL INTEGI21(CC,DD,BB,T,KODE,ANSI21,I21ERR,KFORM)
      R3ABCDT=(AA+AA)*ANSJ22/RTPI-CAB*ANSI21+ANSI25
      SS=SS-R3ABCDT
      CCD=1.0D0+2.0D0*CC*DD
      CCDP=CCD+CCD
      KODE=2
      CALL INTEGI1(BB,DD,CAPT,KODE,REL,ANSI1BD,IERR)
      CALL INTEGI22(CC,DD,BB,T,ANSI22,I22ERR,KFORM)      
      R2CDBT=(T+T)*CC*EN(1)/RTPI-CCDP*ANSI1BD+ANSI22
      ARG=DD/RT
      KODE=3
      ERFD=DRERF(ARG,KODE,NZ)
      R12BDT=(EX*ERFD*TRT-DD*T*EN(1)/RTPI-(BS+BS)*ANSI1BD)/3.0D0
      R413CDBT=(T2+T2)*CC*EN(2)/RTPI-CCDP*R12BDT+ANSJ22
      ARG=BS/T
      EX=DEXP(-ARG)
      VCDT=DVOFT(CC,DD,T,REL,IVERR,IFORM)
      R41CDBT=CS*TRT*EX*VCDT-BS*R2CDBT-1.5D0*R413CDBT
      KODE=2
      CALL INTEGW3(BB,DD,CAPT,KODE,REL,ANSIW3,IERR)
      R423CDBT=4.0D0*CC*R12DBT/RTPI-CCDP*ANSIW3+ANSI21
      R42CDBT=CS*T*ERFB*EX*VCDT-BB*R2CDBT/RTPI-R423CDBT
      R4ABCDT=(AA+AA)*R41CDBT/RTPI-CAB*R42CDBT
      ANS26A=(SS-R4ABCDT)/(AS+CS)
      RETURN
      END
      SUBROUTINE INTEGI25(A,B,C,D,T,ANS25,I2ERR)
C
C     DONALD E. AMOS, JANUARY, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 25
C
C     INTEGI25 COMPUTES
C
C        ANS25=INT ON (0,T) OF U(A,B,W)*U(C,D,W)
C
C        U(A,B,W)= EXP(A*A*W+2*A*B)*ERFC(A*RT+B/RT), RT=SQRT(W)
C
C     WHERE A, B, C, D  ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C           I2ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW, (B^2+D^2)/T TOO LARGE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(2)
      DATA RTPI /1.772453850905516D0/
      I2ERR=0
      IF(A.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(D.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         I2ERR=4
         ANS25=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANS25=0.0D0
         RETURN
      ENDIF
      OPT=1
C     SINCE THE INTEGRAL IS SYMMETRIC IN THE PAIRS (A,B), (C,D) WE CAN
C     EXCHANGE THE PAIRS IF WE WANT THE A PARAMETER TO BE GREATER THAN
C     THE C PARAMETER. THIS IS WHAT OPT=1 DOES.
C     OPT=1, RUNCODE WITH AA=MAX(A,C), CC=MIN(A,C)
C     OPT=2, RUNCODE WITH NO CHANGE IN PARAMETERS
      IF(OPT.EQ.1) THEN
        IF(C.GT.A)THEN
          AA=C
          BB=D
          CC=A
          DD=B
        ELSE
          AA=A
          BB=B
          CC=C
          DD=D
        ENDIF
      ELSE
        AA=A
        BB=B
        CC=C
        DD=D
      ENDIF
      REL=0.5D-14
      RT=DSQRT(T)
      CAPT=1.0D0/RT
      AS=AA*AA
      BS=BB*BB
      CS=CC*CC
      DS=DD*DD
      CAPX=(BS+DS)/T
      IF(CAPX.GT.667.0D0) THEN
        ANS25=0.0D0
        I2ERR=5
        RETURN
      ENDIF
      VABT=DVOFT(AA,BB,T,REL,IVERR,IFORM)
      ARG=CC*RT+DD/RT      
      KODE=3
      NZ=0
      ERFU=DRERF(ARG,KODE,NZ)
      ARG=DS/T
      UCDT=ERFU*DEXP(-ARG)
      SS=AS*VABT*UCDT
      N=1
      KKODE=1
      M=2
      TOL=REL
      NZ=0
      IERR=0
      CALL DEXINT(CAPX, N, KKODE, M, TOL, EN, NZ, IERR)
      CAB=1.0D0+(BB+BB)*AA
      KODE=2
      CALL INTEGI1(DD,BB,CAPT,KODE,REL,ANSI1C,IERR)
      CALL INTEGI22(AA,BB,DD,T,ANSI22,I2ERR,KFORM)
      R1ABDT=AA*T*EN(2)/RTPI-CAB*ANSI1C
      R1ABDT=R1ABDT+R1ABDT+ANSI22
      CALL INTEGI5(DD,BB,CAPT,ANSI5,IERR)
      CALL INTEGJ21(AA,BB,DD,T,ANSJ21,I2ERR,KFORM)
      R2ABDT=AA*EN(1)/RTPI-CAB*ANSI5
      R2ABDT=R2ABDT+R2ABDT+ANSJ21
      SS=SS+(CC*R1ABDT-DD*R2ABDT)/RTPI
      CALL INTEGJ22(CC,DD,BB,T,ANSJ22,I2ERR,KFORM)
      KODE=2
      CALL INTEGI21(CC,DD,BB,T,KODE,ANSI21C,I2ERR,KFORM)
      SS=SS-(AA+AA)*CS*ANSJ22/RTPI+CS*CAB*ANSI21C
      ANS25=SS/(AS+CS)
      RETURN
      END
      PROGRAM DGSCOMP
C
C     WRITTEN BY DONALD E. AMOS, SEPTEMBER, 2005.
C
C     REFERENCE: FOLDERS 6 AND 16
C
C     PROGRAM DGSCOMP IMPLEMENTS THE ACCELERATION OF THE SERIES FOR
C
C              G(X)= INT ON (X, INF) OF ERFC(U)/U
C
C     IN TERMS OF THE EXPONENTIAL INTEGRAL AND COMPARES THE RESULTS WITH
C     THE IMPLEMENTATION OF THE CHEBYSHEV EXPANSION IN FUNCTION
C     DGERFC(X,KODE,REL IERR)), BOTH OF WHICH ARE DOCUMENTED IN
C     FOLDER 16. DGERFC RETURNS RELATIVE ERRORS ON THE ORDER OF
C     O(10**(-13)) OR BETTER OVER THE FULL EXPONENT RANGE.
C
C     AS NOTED IN THE NUMERICAL COMMENTS ON THE ACCELERATED SERIES, ONE
C     CAN EXPECT GOOD RELATIVE ERRORS FOR X < XCRITICAL=DSQRT(N/e) WITH
C     DEGENERATION OF RELATIVE ACCURACY (SIGNIFICANT DIGITS) FOR VALUES
C     OF X LARGER THAN XCRITICAL. HERE N IS THE NUMBER OF APPLICATIONS
C     OF THE RECURRENCE TO OBTAIN THE ACCELERATED SERIES. THE S AND W
C     SEQUENCES ARE STORED IN DATA STATEMENTS AS SN(*) AND WN(*).
C
C     THE RESULTS OF THE COMPUTATION ARE STORED IN FILE DGSCOMP.TXT.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION WN(10),EN(301),SN(10),S0(10),W(10)
      DATA (SN(I), I=1,10)/                                             
     & 0.43551721806072D+01, 0.50323516580300D+00, 0.89336136051503D-01,
     & 0.14947622042659D-01, 0.22306483497165D-02, 0.29554277395613D-03,
     & 0.34977497160467D-04, 0.37284500519472D-05, 0.36090015081122D-06,
     & 0.31959542319209D-07/
      DATA (WN(I), I=1,10)/                                             
     & 0.35517218060720D+00, 0.58790721358557D-01, 0.93361360515026D-02,
     & 0.13421798657880D-02, 0.17303518099223D-03, 0.20060680292218D-04,
     & 0.21043873642805D-05, 0.20111319127703D-06, 0.17625586083365D-07,
     & 0.14251165862602D-08/
      DATA RTPI/1.772453850905516D0/
      DATA TPI/6.28318530717958648D0/
      OPEN(UNIT=7,FILE="DGSCOMP.TXT")
C     N = NUMBER OF TERMS IN THE COEFFICIENTS OF EXP(-X**2) AND IERFC(X)
C       = NUMBER OF APPLICATIONS OF THE RECURRENCE TO ACCELERATE THE
C         CONVERGENCE OF THE SERIES.
      N=6
      P=1.0D0
      AJ=1.0D0
      DO 10 J=1,N
        P=P*AJ
        AJ=AJ+1.0D0
 10   CONTINUE
C     P= N FACTORIAL, AJ=N+1
      FACN=P
      FN=AJ-1.0D0
      FACJ=1.0D0
      FJ=1.0D0
      AJ=1.5D0
      S0(1)=4.0D0
      DO 11 J=2,N
        S0(J)=1.0D0/(FACJ*AJ*AJ)
        AJ=AJ+1.0D0
        FJ=FJ+1.0D0
        FACJ=FACJ*FJ
 11   CONTINUE
      WRITE(7,103)
 103  FORMAT(10X,"VERIFYING COMPUTATION OF WN(*)"/
     &'    J',12X,'WN(J)',11X,'REL ERR' )
      DO 12 J=1,N
        W(J)=SN(J)-S0(J)
        ERR=(W(J)-WN(J))/WN(J)
        WRITE(7,102) J,WN(J),ERR
 102    FORMAT(I5,D25.15,D12.4)
 12   CONTINUE
      XC=FN/DEXP(1.0D0)
      XC=DSQRT(XC)
      NXCRIT=INT(SNGL(XC*100.0D0))
      XCRIT=DBLE(FLOAT(NXCRIT))/100.0D0
      WRITE(7,101)XCRIT
 101  FORMAT(/"      XCRITICAL IS APPROX = ",D15.2//
     &"  N       X         G(X)      GSER(X)     REL ERR     ABS ERR   K
     &TERMS")
      DO 15 IX=1,25
C       GENERATE THE X VALUES TO BE USED IN THE COMPARISON
        X=DBLE(FLOAT(IX)/5.0D0)
C       GENERATE THE COEFFICIENT SUMS OF EXP(-X**2) AND IERFC(X)
        X2=X*X
        SS=0.0D0
        SW=0.0D0
        P=1.0D0
        DO 16 J=1,N
          SS=SS+P*S0(J)
          SW=SW+P*WN(J)
          P=-P*X2
 16     CONTINUE
C       P=(-X2)**N
        KODE=1
        CALL DIERFC(X,KODE,ANS,IERR)
C       SW=COEFFICIENT OF DEXP(-X**2)
C       SS*RTPI=COEFFICIENT OF IERFC(X)
        TRM1=DEXP(-X2)*SW
        TRM2=SS*RTPI*ANS
        SUM=TRM1+TRM2
        KODE=2
C       COMPUTE EXPONENTIAL INTEGRAL SUM
        FNH=1.5D0
        KODE=1
        M=200
        TOL=1.0D-13
C       M EXPONENTIAL INTEGRALS OF HALF ODD INTEGER ORDER IN EN(*)
        CALL DHEXINT(X2, FNH, KODE, M, TOL, EN, NZ, IERR)
        SS=0.0D0
        BK=0.5D0
        FHK=BK/FACN
        TOL=1.0D-12
        AK=FN+0.50D0
        FNK=FN
        DO 30 K=1,M
          SNK=FHK/(AK*AK)
          TRM=SNK*EN(K)
          SS=SS+TRM
          IF(DABS(TRM).LT.TOL*DABS(SS)) GOTO 31
          FNK=FNK+1.0D0
          BK=BK+1.0D0
          FHK=FHK*BK/FNK
          AK=AK+1.0D0
 30     CONTINUE
        WRITE(7,200) 
 200    FORMAT(2X,"FELL THRU E(K+0.5,X**2) LOOP")
 31     CONTINUE
C       COMPUTE H(X) 
        HX=SUM+P*SS
C       COMPUTE 0.5*E(1,X^2)
        NN=1
        KODE=1
        M=1
        TOL=1.0D-13
C       EXPONENTIAL INTEGRAL OF ORDER ONE IN EN(1)
        CALL DEXINT(X2, NN, KODE, M, TOL, EN, NZ, IERR)
        GSER=0.5D0*EN(1)-HX/TPI
        KODE=1
        REL=TOL
C       Y=G(X) FROM THE CHEBYSHEV EXPANSION
        Y=DGERFC(X,KODE,REL,IERR)
        ER=DABS(Y-GSER)
        IF(DABS(Y).GT.0.0D0) THEN
          ERR=ER/DABS(Y)
        ENDIF
        WRITE(7,300) N,X,Y,GSER,ERR,ER,K
 300    FORMAT(I3,5D12.4,I5)
 15   CONTINUE
      END
