      PROGRAM SERSTEST
C     INITSX    INITIALIZES A SERIES TO A CONSTANT VALUE
C     PXSX      MULTIPLIES  A SERIES BY X**K
C     QXSX      DIVIDES     A SERIES BY X**K
C     DKSX      TAKES K DERIVATIVES (K<10 IF ZERO POWER HAS INDEX 10)
C     INTSX     INTEGRATES  A SERIES K TIMES ON (0,X) (FOR NON-NEGATIVE
C               EXPONENTS ONLY WITH THE ZERO POWER AT INDEX 10)
C     ADDSX     ADDS        TWO SERIES
C     SUBSX     SUBTRACTS   TWO SERIES
C     CMSX      MULTIPLIES  A SERIES BY A CONSTANT
C     CDSX      DIVIDES     A SERIES BY A CONSTANT
C     COPYSX    COPIES      A SERIES ARRAY TO ANOTHER ARRAY
C     WRTSX     WRITES OUT  AN ARRAY IN THE FORM OF A DATA STATEMENT
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION F(60),G(60),H(60)
      OPEN(UNIT=7,FILE="SERSTEST.TXT")
      CON=1.0D0
      CALL INITSX(F,CON)
      WRITE (7,100)
 100  FORMAT('INIT')
      CALL WRTSX(F,7)
C
      CALL PXSX(F,H,3)
      WRITE (7,200)
 200  FORMAT('PXSX K=3')
      CALL WRTSX(H,7)
C
      CALL QXSX(F,G,4)
      WRITE (7,300)
 300  FORMAT('QXSX K=4')
      CALL WRTSX(G,7)
C
      CALL DKSX(F,H,2)
      WRITE (7,400)
 400  FORMAT('DKSX K=2')
      CALL WRTSX(H,7)
C
      CALL INTSX(F,H,2)
      WRITE (7,500)
 500  FORMAT('INTSX K=2')
      CALL WRTSX(H,7)
C
      CALL INITSX(G,3.0D0)
      CALL ADDSX(F,G,H)
      WRITE (7,600)
 600  FORMAT('ADDSX ANS=4')
      CALL WRTSX(H,7)
C
      CALL SUBSX(F,G,H)
      WRITE (7,700)
 700  FORMAT('SUBSX ANS=-2')
      CALL WRTSX(H,7)
C
      CALL CMSX(F,H,5.0D0)
      WRITE (7,800)
 800  FORMAT('CMSX ANS=5')
      CALL WRTSX(H,7)
C
      CALL CDSX(F,H,4.0D0)
      WRITE (7,900)
 900  FORMAT('CDSX ANS=1/4')
      CALL WRTSX(H,7)
      END
      SUBROUTINE INFO
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     THE SUBROUTINES IN THIS FILE MANIPULATE DOUBLE PRECISION VECTORS
C     (DIMENSIONED BY 60) WHOSE VALUES ARE INTERPRETED AS COEFFIENTS OF
C     POWER SERIES. IF THE CONSTANT TERM IS SET AT SOME INDEX GREATER
C     THAN 1 (SAY 10), THE USUAL OPERATIONS (MULTIPLICATION BY X**K,
C     MULTIPLICATION CONSTANTS, DIVISION BY X**K, ETC.) WILL WORK FOR
C     NEGATIVE INDICES AS WELL UP TO POWERS OF -9. IF THIS INDEX IS 10,
C     THEN (INDEX-10) IS THE POWER ASSOCIATED WITH THAT COEFFICIENT. THE
C     LIST IS:
C
C               ALL ARRAYS (VECTORS) ARE DIMENSIONED BY 60
C     INITSX    INITIALIZES A SERIES TO A CONSTANT VALUE
C     PXSX      MULTIPLIES  A SERIES BY X**K
C     QXSX      DIVIDES     A SERIES BY X**K
C     DKSX      TAKES K DERIVATIVES (K<10 IF ZERO POWER HAS INDEX 10)
C     INTSX     INTEGRATES  A SERIES K TIMES ON (0,X) (FOR NON-NEGATIVE
C               EXPONENTS ONLY WITH THE ZERO POWER AT INDEX 10)
C     ADDSX     ADDS        TWO SERIES
C     SUBSX     SUBTRACTS   TWO SERIES
C     CMSX      MULTIPLIES  A SERIES BY A CONSTANT
C     CDSX      DIVIDES     A SERIES BY A CONSTANT
C     COPYSX    COPIES      A SERIES ARRAY TO ANOTHER ARRAY
C     WRTSX     WRITES OUT  AN ARRAY IN THE FORM OF A DATA STATEMENT
C
      RETURN
      END
      SUBROUTINE INITSX(TEMP,CON)
C     INTITIALIZE TEMP VECTOR TO A COMMON VALUE, TEMP=CON
      DIMENSION TEMP(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 10 J=1,60
        TEMP(J)=CON
  10  CONTINUE
      RETURN
      END
      SUBROUTINE PXSX(TEMP,ANS,K)
C     MULTIPLY SERIES BY X**K
      DIMENSION TEMP(60),ANS1(60),ANS(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 5 J=1,60
        ANS1(J)=0.0D0
  5   CONTINUE
      JL=60-K
      DO 10 J=1,JL
        ANS1(J+K)=TEMP(J)
  10  CONTINUE
      CALL COPYSX(ANS1,ANS)
      RETURN
      END
      SUBROUTINE QXSX(TEMP,ANS,K)
C     DIVIDE SERIES BY X**K
      DIMENSION TEMP(60),ANS1(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 5 J=1,60
        ANS1(J)=0.0D0
  5   CONTINUE
      DO 10 J=1,60-K
        ANS1(J)=TEMP(J+K)
  10  CONTINUE
      CALL COPYSX(ANS1,ANS)
      RETURN
      END
      SUBROUTINE DKSX(TEMP,ANS,K)
C     TAKE K DERIVATIVES OF SERIES
      DIMENSION TEMP(60),ANS1(60),ANS(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 5 J=1,60
        ANS1(J)=TEMP(J)
  5   CONTINUE
      DO 7 M=1,K
        AK=-8.0D0
        DO 10 J=2,60
          ANS1(J-1)=ANS1(J)*AK
          AK=AK+1.0D0
  10    CONTINUE
        ANS1(60)=0.0D0
  7   CONTINUE
      CALL COPYSX(ANS1,ANS)
      RETURN
      END
      SUBROUTINE INTSX(TEMP,ANS,K)
C     INTEGRATES A SERIES WITH NON-NEGATIVE POWERS ON (0,X). THE
C     INDEX FOR THE ZERO POWER TERM IS 10. ORDER OF DERIVATIVES IS
C     (INDEX-10) (50 TERMS MAX FOR ARRAYS DIMENSIONED BY 60).
      DIMENSION TEMP(60),ANS1(60),ANS(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 5 J=1,60
        ANS1(J)=TEMP(J)
  5   CONTINUE
      DO 7 M=1,K
        JJ=59
        AK=50.0D0
        DO 10 J=10,59          
          ANS1(JJ+1)=ANS1(JJ)/AK
          JJ=JJ-1
          AK=AK-1.0D0
  10    CONTINUE
        ANS1(10)=0.0D0
  7   CONTINUE
      CALL COPYSX(ANS1,ANS)
      RETURN
      END
      SUBROUTINE CMSX(TEMP,ANS,CON)
C     MULTIPY SERIES BY CONSTANT
      DIMENSION TEMP(60),ANS(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 10 J=1,60
        ANS(J)=CON*TEMP(J)
  10  CONTINUE
      RETURN
      END
      SUBROUTINE CDSX(TEMP,ANS,CON)
C     DIVIDE SERIES BY CONSTANT
      DIMENSION TEMP(60),ANS(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 10 J=1,60
        ANS(J)=TEMP(J)/CON
  10  CONTINUE
      RETURN
      END
      SUBROUTINE ADDSX(TEMP1,TEMP2,ANS)
C     ADD TWO SERIES ANS=TEMP1 + TEMP2
      DIMENSION TEMP1(60),TEMP2(60),ANS1(60),ANS(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 10 J=1,60
        ANS1(J)=TEMP1(J)+TEMP2(J)
  10  CONTINUE
      CALL COPYSX(ANS1,ANS)
      RETURN
      END
      SUBROUTINE SUBSX(TEMP1,TEMP2,ANS)
C     SUBTRACT TWO SERIES ANS=TEMP1-TEMP2
      DIMENSION TEMP1(60),TEMP2(60),ANS1(60),ANS(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 10 J=1,60
        ANS1(J)=TEMP1(J)-TEMP2(J)
  10  CONTINUE
      CALL COPYSX(ANS1,ANS)
      RETURN
      END
      SUBROUTINE COPYSX(TEMP1,TEMP2)
C     COPY SERIES TEMP1 TO TEMP2
      DIMENSION TEMP1(60),TEMP2(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DO 10 J=1,60
        TEMP2(J)=TEMP1(J)
  10  CONTINUE
      RETURN
      END
      SUBROUTINE WRTSX(TEMP,NUNIT)
C     WRITE TEMP TO UNIT NUMBER NUNIT IN THE FORM OF A DATA
C     STATEMENT. THE MAIN PROGRAM MUST HAVE THE OPEN STATEMENT
C                OPEN(NUNIT,FILE='FILENAME')
C     WHICH ASSOCIATES THE UNIT NUMBER WITH THE OUTPUT FILE.
C
      DIMENSION TEMP(60)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      WRITE (NUNIT,100) (TEMP(J),J=1,60)
 100  FORMAT('     C',D21.15,',',D21.15,',',D21.15,',')
      RETURN
      END
************************************************************************
      PROGRAM MPCHEBY
C 
C     WRITTEN BY D.E.AMOS, DECEMBER,1972; MODIFIED MAY, 1990.
C 
C     REFERENCE SANDIA NATIONAL LABORATORIES REPORT SC-DR-72 0917
C 
C     ABSTRACT
C
C     PROGRAM MPCHEBY CALCULATES THE CHEBYSHEV COEFFICIENTS FOR A
C     MULTIPLE PRECISION FUNCTION MPGOFX(X) ON AN INTERVAL [A,B].
C
C     FINAL EXPANSION IS  S = 0.5*A(1)+SUM( A(J)*T(J,XX), J=2,N) WHERE
C
C          XX=(2*X-(A+B))/(B-A)   FOR    A.LE.X.LE.B
C                                 OR
C          XX=2*A/X-1.0           FOR    A.LE.X.LT.INFINITY,
C 
C     T(J,XX) ARE CHEBYSHEV POLYNOMIALS AND A(J) ARE THE COEFFICIENTS
C     GENERATED BY THIS PROGRAM AND STORED IN A DATA STATEMENT IN A
C     FILE CALLED MPCHEBY.DAT.
C
C     DESCRIPTION OF ARGUMENTS
C
C     INPUT 
C       A, B    = INTERVAL [A,B] END POINTS. IF A=B, THE EXPANSION
C                 IS TAKEN ON A TO INFINITY USING T(J,2*A/X-1)
C       N       = THE NUMBER OF COEFFICIENTS DESIRED (USUALLY N < 26)
C       NCHEBY  = NUMBER OF CHEBYSHEV NODES FOR THE APPROXIMATION
C                 NCHEBY IS MUCH LARGER THAN N  (USUALLY NCHEBY > 40)
C       IERR    = 1 GIVES A RELATIVE ERROR PRINT
C               = 0 GIVES AN ABSOLUTE ERROR PRINT
C       DNAME   = NAME OF THE COEFFICIENT ARRAY IN THE DATA STATEMENT
C       MPGOFX  = MULTIPLE PRECISION SUBROUTINE FOR THE FUNCTION BEING
C                 APPROXIMATED. SUBROUTINE MPGOFX(MX,MGOFX)
C     OUTPUT
C       FILE MPCHEBY.FIT SHOWING THE COEFFICIENTS AND ERRORS
C       FILE MPCHEBY.DAT CONTAINING THE DATA STATEMENT FOR THE
C                      COEFFICIENTS 0.5*A(1) THRU A(N)
C       FILE MPCHEBY.DAT IS CREATED AND USED AS A SCRATCH FILE
C
C     PROGRAM MPCHEBY USES MPGOFX AND DATAS PLUS ANY OTHERS RELATED TO
C     THE COMPUTATION OF GOFX
C-----------------------------------------------------------------------
      CHARACTER*2 NK,NAB
      CHARACTER*10 ILAB(2)
      CHARACTER*12 FNAME,DNAME,FWORK
C      CHARACTER*25 LINEA
      CHARACTER*80 LINE,LINEA
      CHARACTER*160 LTEMP,LBLANK
      DOUBLE PRECISION T,PI,DN,THETA,VK,VONE,TK,TONE,APB,BMA,X 
      DOUBLE PRECISION Z,TZ,B1,B2,F,AA,BB,AR,TEMP,TEMP1 
C
C-----------------------------------------------------------------------
      DOUBLE PRECISION DTOL,DH
      DOUBLE PRECISION DPMAX
C     SINGLE PRECISION SPMAX

      PARAMETER ( MXNDIG=256 , NBITS=32 ,
     *          LPACK  = (MXNDIG+1)/2 + 1 , LUNPCK = (6*MXNDIG)/5 + 20,
     *          LMWA   = 2*LUNPCK         , LJSUMS = 8*LUNPCK,
     *          LMBUFF = ((LUNPCK+3)*(NBITS-1)*301)/2000 + 6  )
C
C
      DIMENSION MTEMP1(200),MTEMP2(200),MAPB(200),MBMA(200),MAA(200),
     &MBB(200),MVK(200),MVONE(200),MTK(200),MTHETA(200),MZ(200),
     &MTZ(200),MB1(200),MB2(200)
      DIMENSION T(201),F(201),X(201),AR(201)
      DIMENSION MT(200,100),MF(200,100),MX(200,100),MAR(200,100),
     &MTEMP(200)

C
      COMMON /FMUSER/ NDIG,JBASE,JFORM1,JFORM2,KRAD,
     *                KW,NTRACE,LVLTRC,KFLAG,KWARN,KROUND
C
      COMMON /FM/ MWA(LMWA),NCALL,MXEXP,MXEXP2,KARGSW,KEXPUN,KEXPOV,
     *            KUNKNO,IUNKNO,RUNKNO,MXBASE,MXNDG2,KSTACK(19),
     *            MAXINT,SPMAX,DPMAX
C
      COMMON /FMSAVE/ NDIGPI,NJBPI,NDIGE,NJBE,NDIGLB,NJBLB,NDIGLI,NJBLI,
     *                MPISAV(LUNPCK),MESAV(LUNPCK),MLBSAV(LUNPCK),
     *                MLN1(LUNPCK),MLN2(LUNPCK),MLN3(LUNPCK),
     *                MLN4(LUNPCK)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)
C
      DATA (ILAB(I),I=1,2)/' RELATIVE ',' ABSOLUTE '/

C-----------------------------------------------------------------------
      PRINT 950 
  950 FORMAT(' APPROXIMATION INTERVAL A, B= ')
      READ *,AA,BB
      PRINT 951
  951 FORMAT(' NUMBER OF CHEBYSHEV COEFFICIENTS DESIRED (N~25), N= ')
      READ *,NAR
      PRINT 952
  952 FORMAT(' NUMBER OF CHEBYSHEV NODES (NCHEBY~40), NCHEBY= ')
      READ *,NCHEBY
      PRINT 953
  953 FORMAT(' NAME OF THE COEFFICIENTS IN THE DATA STATEMENT, DNAME= ')
      READ 900,DNAME
  900 FORMAT(A8)
      PRINT 954
  954 FORMAT(' 1 FOR RELATIVE ERROR; 2 FOR ABSOLUTE ERROR, IERR= ')
      READ *,IERR
      IF (IERR.NE.1 .AND. IERR.NE.2) THEN
        IERR=2
      ENDIF
C-----------------------------------------------------------------------
C     END INPUT 
C-----------------------------------------------------------------------
      INIT=0
      IF (INIT.EQ.0) THEN
        NPREC=65
        CALL FPSET(NPREC)
C       FOR JBASE=10000, NDIG=NPREC/4+2
        NDIG=INT(NPREC/4)+5
        NCDIG=NDIG
        K=0
        CALL FPI2M(K,MZERO)
        K=1
        CALL FPI2M(K,MONE)
        K=2        
        CALL FPI2M(K,MTWO)
        K=9
        CALL FPI2M(K,MNINE)
        CALL FPPI(MPI)
        CALL FPM2DP(MPI,PI)
        CALL FPSQRT(MPI,MRTPI)
C       MRTPI=SQRT(PI)
        CALL FPDIV(MONE,MRTPI,MTEMP)
        CALL FPADD(MTEMP,MTEMP,MTRTPI)
C       MTRTPI=2/RTPI
        DTOL=0.50D-19
        CALL FPDP2M(DTOL,MTOL)
        I=2
        CALL FPDIVI(MONE,I,MDH)
C       INIT=1
C       INIT IS SET TO 1 AFTER THE INITIALIZATION OF MQUAD8
      ENDIF
      N1 = NCHEBY-1
      N2 = NCHEBY-1
      APB=AA+BB 
      BMA=BB-AA
      CALL FPDP2M(AA,MAA)
      CALL FPDP2M(BB,MBB)
      CALL FPADD(MAA,MBB,MAPB)
      CALL FPSUB(MBB,MAA,MBMA)
      C1= SNGL(APB)
      C2= SNGL(BMA)
      IF(BB.EQ.AA) C1=SNGL(AA+AA) 
      OPEN(10,FILE='MPCHEBY.FIT',STATUS='UNKNOWN')
      DO 500 N=N1,N2 
        IF (NAR.GT.N) THEN
          NAR=N+1
        ENDIF
        NP1=N+1
        DN=DBLE(FLOAT(N))
        THETA=PI/DN
        CALL FPDIVI(MPI,N,MTHETA)
        T(1)=1.0D0
        CALL COPYTO(MONE,1,MT)  
        T(2)=DCOS(THETA)
        CALL FPCOS(MTHETA,MTEMP)
        CALL COPYTO(MTEMP,2,MT)
        CALL FPEQU(MTEMP,MTONE,NDIG,NDIG)
        TONE=T(2)
        VONE=DSIN(THETA)
        CALL FPSIN(MTHETA,MVONE)
        VK = VONE
        CALL FPEQU(MVONE,MVK,NDIG,NDIG)
C-----------------------------------------------------------------------
C     GET CHEBYSHEV NODES ON [A,B] IN X ARRAY
C     TRIG IDENTITIES SIN((K+1)*X) = SIN(K*X)*COS(X)+COS(K*X)*SIN(X)
C                     COS((K+1)*X) = COS(K*X)*COS(X)-SIN(K*X)*SIN(X)
C     FOR CHEBYSHEV NODES T(K)=COS((K-1)*PI/N), K=1,N+1 BY RECURRENCE
C-----------------------------------------------------------------------
        DO 5 K=2,N
          TK=T(K)
          CALL COPYFR(MT,K,MTK)          
          T(K+1)=TONE*TK-VK*VONE
          CALL FPMPY(MTONE,MTK,MTEMP)
          CALL FPMPY(MVK,MVONE,MTEMP1)
          CALL FPSUB(MTEMP,MTEMP1,MTEMP)
          CALL COPYTO(MTEMP,K+1,MT)
          VK=VK*TONE+TK*VONE
          CALL FPMPY(MVK,MTONE,MTEMP)
          CALL FPMPY(MTK,MVONE,MTEMP1)
          CALL FPADD(MTEMP,MTEMP1,MVK)
          X(K)=(TK*BMA+APB)/2.0D0
          CALL FPMPY(MTK,MBMA,MTEMP)
          CALL FPADD(MTEMP,MAPB,MTEMP)
          CALL FPDIVI(MTEMP,2,MTEMP)
          CALL COPYTO(MTEMP,K,MX)
          CALL FPM2DP(MTEMP,TEMP)
          PRINT *,'K, X(K) ',K,TEMP
C          IF(AA.NE.BB) GO TO 5
C          IF(TK.NE.-1.0D0) GO TO 10
C          X(K)=1.0D+16
C          GO TO 5
C   10     CONTINUE
C          X(K)=(AA+AA)/(1.0D0+TK)
    5   CONTINUE
        X(1)=(T(1)*BMA+APB)/2.0D0
        CALL COPYFR(MT,1,MTEMP)
        CALL FPMPY(MTEMP,MBMA,MTEMP)
        CALL FPADD(MTEMP,MAPB,MTEMP)
        CALL FPDIVI(MTEMP,2,MTEMP)
        CALL COPYTO(MTEMP,1,MX)
        X(NP1)=(T(NP1)*BMA+APB)/2.0D0
        CALL COPYFR(MT,NP1,MTEMP)
        CALL FPMPY(MTEMP,MBMA,MTEMP)
        CALL FPADD(MTEMP,MAPB,MTEMP)
        CALL FPDIVI(MTEMP,2,MTEMP)
        CALL COPYTO(MTEMP,NP1,MX)
C        IF(AA.NE.BB) GO TO 20
C        X(1)=(AA+AA)/(1.0D0+T(1))
C        X(NP1)=1.0D+16
C   20   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 52)NP1,AA,BB
   52 FORMAT(' NODES='I5/' A    ='D23.14/' B    ='D23.14/)
        WRITE(10, 51)(X(I),I=1,NP1)
   51 FORMAT(' CHEBYSHEV NODES  X(J)'/(1X,3D23.14))
C-----------------------------------------------------------------------
C     FUNCTION EVALUATION AT CHEBYSHEV NODES X(I), I=1,NP1
C-----------------------------------------------------------------------
        DO 7 I=1,NP1
          CALL COPYFR(MX,I,MTEMP)
          CALL FPM2DP(MTEMP,TEMP1)
          CALL MPGOFX(MTEMP,MTEMP1)
          CALL COPYTO(MTEMP1,I,MF)
          CALL FPM2DP(MTEMP1,TEMP)
          F(I)=TEMP
          PRINT *,'I,X(I),F(I)',I,TEMP1,TEMP
    7   CONTINUE
        F(NP1)=TEMP/2.0D0
        CALL FPDIVI(MTEMP1,2,MTEMP)
        CALL COPYTO(MTEMP,NP1,MF)
C-----------------------------------------------------------------------
C     END OF FUNCTION EVALUATION
C-----------------------------------------------------------------------
        DO 100 IR=1,NP1
C          Z=T(IR)
          CALL COPYFR(MT,IR,MZ)
C          TZ=Z+Z
          CALL FPMPYI(MZ,2,MTZ)
C          B2=0.0D0
          CALL FPEQU(MZERO,MB2,NDIG,NDIG)
C          B1= 0.0D0
          CALL FPEQU(MZERO,MB1,NDIG,NDIG)
          DO 15 J=1,N
            K=NP1-J+1
            TEMP=B1
            CALL FPEQU(MB1,MTEMP,NDIG,NDIG)
C            B1=TZ*B1-B2+F(K)
            CALL COPYFR(MF,K,MTEMP1)
            CALL FPMPY(MTZ,MB1,MTEMP2)
            CALL FPSUB(MTEMP2,MB2,MTEMP2)
            CALL FPADD(MTEMP2,MTEMP1,MB1)
C            B2=TEMP
            CALL FPEQU(MTEMP,MB2,NDIG,NDIG)
   15     CONTINUE
C          ANS=Z*B1-B2+F(1)/2.0D0
          CALL COPYFR(MF,1,MTEMP)
          CALL FPDIVI(MTEMP,2,MTEMP)
          CALL FPMPY(MZ,MB1,MTEMP1)
          CALL FPSUB(MTEMP1,MB2,MTEMP1)
          CALL FPADD(MTEMP1,MTEMP,MTEMP)
C          AR(IR)=(ANS+ANS)/DN
          CALL FPMPYI(MTEMP,2,MTEMP)
          CALL FPDIVI(MTEMP,N,MTEMP)
          CALL COPYTO(MTEMP,IR,MAR)
          CALL FPM2DP(MTEMP,AR(IR))
  100   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10,153) (AR(K),K=1,NP1)
  153 FORMAT(' CHEBYSHEV COEFFICIENTS A(J)'/(1X,3D23.14))
C-----------------------------------------------------------------------
        F(NP1)=F(NP1)+F(NP1)
        CALL COPYFR(MF,NP1,MTEMP)
        CALL FPMPYI(MTEMP,2,MTEMP)
        CALL COPYTO(MTEMP,NP1,MF)
C-----------------------------------------------------------------------
        WRITE(10, 54)(F(I),I=1,NP1)
   54 FORMAT(' FUNCTION VALUES AT CHEBYSHEV NODES F(X(J))'/(1X,3D23.14))
C-----------------------------------------------------------------------
        NARM=NAR-1
        DO 30 I=1,NP1
          XX=SNGL(X(I))
          IF(AA.NE.BB) GO TO 26
          SZ=C1/XX-1.0E0
          GO TO 27
   26     CONTINUE
          SZ=(XX+XX-C1)/C2
   27     CONTINUE
          STZ=SZ+SZ
          S2=0.0E0
          S1=0.0E0
          DO 25 J=1,NARM
            K=NAR-J+1
            ST=S1
            SA=AR(K)
            S1=STZ*S1-S2+SA
            S2=ST
   25     CONTINUE
          SA=AR(1)
          FX=SZ*S1-S2+SA/2.0
          T(I)=DBLE(FX)-F(I)
          IF(IERR.EQ.1) THEN
            IF(F(I).NE.0.0D0) THEN
              T(I)=T(I)/F(I)
            ENDIF
          ENDIF
   30   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 53)ILAB(IERR),NAR,AA,BB,(T(I),I=1,NP1)
   53   FORMAT(/' SINGLE PRECISION',A10,'ERRORS AT CHEBYSHEV NODES USING
     &',I5,' TERMS'/' OF THE CHEBYSHEV SUM ON',2D23.14//(1X,3D23.14))
C-----------------------------------------------------------------------
        DO 35 I=1,NP1
          IF(AA.NE.BB) GO TO 36
          Z=(AA+AA)/X(I)-1.0D0
          GO TO 37
   36     CONTINUE
          Z=(X(I)+X(I)-APB)/BMA
   37     CONTINUE
          TZ=Z+Z
          B2=0.0D0
          B1=0.0D0
          DO 40 J=1,NARM
            K=NAR-J+1
            TEMP=B1
            CALL COPYFR(MAR,K,MTEMP)
            CALL FPM2DP(MTEMP,TEMP1)
C            B1=TZ*B1-B2+AR(K)
            B1=TZ*B1-B2+TEMP1
            B2=TEMP
   40     CONTINUE
          CALL COPYFR(MAR,1,MTEMP)
          CALL FPM2DP(MTEMP,TEMP)
C          TEMP=Z*B1-B2+AR(1)/2.0D0
          TEMP=Z*B1-B2+TEMP/2.0D0
          CALL COPYFR(MF,I,MTEMP)
          CALL FPM2DP(MTEMP,TEMP1)
C          T(I)=TEMP-F(I)
          T(I)=TEMP-TEMP1
          IF(IERR.EQ.1) THEN
            IF(F(I).NE.0.0D0) THEN
C              T(I)=T(I)/F(I)
               T(I)=T(I)/TEMP1
            ENDIF
          ENDIF
   35   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 55)ILAB(IERR),NAR,AA,BB,(T(I),I=1,NP1)
   55   FORMAT(/' DOUBLE PRECISION',A10,'ERRORS AT CHEBYSHEV NODES USING
     &',I5,' TERMS'/' OF THE CHEBYSHEV SUM ON',2D23.14//(1X,3D23.14))
C-----------------------------------------------------------------------
  500 CONTINUE
      CLOSE(10)
      FNAME='MPCHEBY.DAT'
      FWORK='MPCHEBZ.DAT'
      AR(1)=AR(1)*0.5D0
      CALL COPYFR(MAR,1,MTEMP)
      CALL FPDIVI(MTEMP,2,MTEMP)
      CALL COPYTO(MTEMP,1,MAR)
      OPEN(10,FILE='MPCHEBY.DAT',STATUS='UNKNOWN')
      REWIND(10)
      WRITE(10,531)NP1
 531  FORMAT('NODES= ',I3)
      WRITE(10,530) AA,BB
 530  FORMAT('A= 'D30.18/'B= ',D30.18/)
      CLOSE(10)
C-----------------------------------------------------------------------
C     WRITE NORMAL DATA STATEMENT WITH 14 DIGITS, 3 ON A LINE
C-----------------------------------------------------------------------
      CALL DATAS(NAR,AR,DNAME,FNAME,FWORK,KERR)
C-----------------------------------------------------------------------
C     WRITE DATA STATEMENT IN 2 COLUMNS OF 18 DIGITS
C-----------------------------------------------------------------------
      DO 515 I=1,10
        IF(DNAME(I:I).NE.' ') GO TO 520
  515 CONTINUE
      PRINT *, 'THE DATA NAME IS BLANK'
      STOP
  520 CONTINUE
      K1=I
      DO 501 I=K1,10
        IF(DNAME(I:I).EQ.' ') GO TO 502
 501  CONTINUE
      I=11
 502  CONTINUE
      K2=I-1
C     READ TO THE BOTTOM OF THE FILE (TO SKIP WHAT IS ALREADY THERE)
      OPEN(15,FILE=FNAME,STATUS='UNKNOWN')
      REWIND(15)
      DO 511 LOOP=1,400
        READ(15,'( )',END=512)
 511  CONTINUE
 512  CONTINUE
      ENDFILE(15)
      BACKSPACE(15)
      OPEN(20,FILE=FWORK,STATUS='UNKNOWN')
      WRITE(20,509) NAR
 509  FORMAT(I2)
      REWIND(20)
      READ(20,510) NAB
 510  FORMAT(A2)
      REWIND (20)
      LINE='      DATA ('//DNAME(K1:K2)//'(I), I=1,'//NAB//')/'
      WRITE(15,506)LINE
 506  FORMAT(A72)
      LBLANK='                                                         '
      LL=72
      JFORM1=0
      K=1
      NMOD=MOD(NAR,2)
      IF(NMOD.EQ.0) THEN
        NJ=NAR-1
      ELSE
        NJ=NAR-2
      ENDIF
      DO 300 J=1,NJ,2
        WRITE(20,509) K
        REWIND(20)
        READ(20,510) NK
        REWIND(20)
        CALL COPYFR(MAR,J,MTEMP)
        LTEMP='    '//NK//'    ' 
        CALL FPOUT(MTEMP,LINEA,LL)
        I1=NPREC+4
        I2=I1+3
        LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//',  '
        LTEMP=LTEMP(1:8)//LINEA(1:30)
        CALL COPYFR(MAR,J+1,MTEMP)
        CALL FPOUT(MTEMP,LINEA,LL)
        IF (J.EQ.NJ) THEN
          IF(NMOD.EQ.0) THEN
            LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//'/'
          ELSE
            LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//','
          ENDIF
        ELSE
          LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//','
        ENDIF
        LTEMP=LTEMP(1:38)//LINEA(1:28)//LBLANK
        LINE=LTEMP(1:72)
        WRITE(15,301)LINE
 301    FORMAT(A72)
        K=K+1
 300  CONTINUE
      IF(NMOD.NE.0) THEN
        WRITE(20,509) K
        REWIND(20)
        READ(20,510) NK
        REWIND(20)
        CALL COPYFR(MAR,NAR,MTEMP)
        LTEMP='    '//NK//'    ' 
        CALL FPOUT(MTEMP,LINEA,LL)
        LINEA=LINEA(1:22)//'D'//LINEA(64:67)//'/  '
        LTEMP=LTEMP(1:8)//LINEA(1:28)//LBLANK
        LINE=LTEMP(1:72)
        WRITE(15,301)LINE
      ENDIF
      CLOSE(15)
C-----------------------------------------------------------------------
      IF (KERR.NE.0) THEN
        PRINT *,' ERROR IN DATAS, IERR = ',KERR
      ELSE
        PRINT *,'  '
        PRINT *,' APPROXIMATION USING ',NP1,' CHEBYSHEV NODES'
        PRINT *,'  '
        PRINT *,' FILES WRITTEN:'
        PRINT *,'  '
        PRINT *,' MPCHEBY.FIT:    THE COEFFIENTS, NODES, AND ERRORS,'
        PRINT *,' MPCHEBY.DAT:    THE DATA STATEMENT,'
        PRINT *,' MPCHEBZ.DAT:    SCRATCH FILE.'
      ENDIF
      PAUSE
      STOP 
      END
      SUBROUTINE COPYTO(MA,K,MB)
C     COPY VECTOR MA to KTH COLUMN OF MATRIX MB 
      DIMENSION MB(200,100),MA(200)
      DO 10 J=1,200
        MB(J,K)=MA(J)
 10   CONTINUE
      RETURN
      END
      SUBROUTINE COPYFR(MB,K,MA)
C     COPY from KTH COLUMN OF MATRIX MB TO VECTOR MA 
      DIMENSION MB(200,100),MA(200)
      DO 10 J=1,200
        MA(J)=MB(J,K)
 10   CONTINUE
      RETURN
      END
      SUBROUTINE MPGOFXX(MX,MGOFX)
      DIMENSION MX(200),MGOFX(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      CALL FPDIV(MONE,MX,MGOFX)
      RETURN
      END
      SUBROUTINE MPGOFXY(MT,MFUN)
C     COMPUTES FOR ERF(X)
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
C     CALCULATE ERF ON [0,2]
C
      NDIG=NCDIG
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
C        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          IF (FPCOMP(MTRMA,'LE',MTOL)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
        CALL FPMPY(MRTPI,MT,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)        
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
      RETURN
      END
      SUBROUTINE MPGOFX(MX,MGOFX)
C
C     COMPUTES INT ON (X,INF) OF ERFC(X)/X IN MP ARITHMETIC USING
C     GAUS8 FORMULA ON REPEATED SUBDIVIDED INTERVALS
C
C      DOUBLE PRECISION DSUM
      LOGICAL FPCOMP
      DIMENSION MSUM(200),MSUMA(200),MANS(200),MANSA(200),MTOLA(200),
     &MTEMP(200),MA(200),MB(200),MX(200),MSUM1(200),MDX(200)
C
      DIMENSION MGOFX(200)
C
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      KL=10
      CALL FPMPYI(MTOL,KL,MTOLA)
      CALL FPEQU(MDH,MDX,NDIG,NDIG)
      CALL FPEQU(MZERO,MSUM1,NDIG,NDIG)
      KL=100
      DO 10 J=1,100
        CALL FPDIVI(MDX,2,MDX)
        KL=KL+KL
        CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
        CALL FPEQU(MX,MB,NDIG,NDIG)
        DO 20 K=1,KL
          CALL FPEQU(MB,MA,NDIG,NDIG)
          CALL FPADD(MA,MDX,MB)
          CALL MQUAD8(MA,MB,MANS)
          CALL FPADD(MSUM,MANS,MSUM)
          IF(J.EQ.1) THEN
            CALL FPABS(MANS,MANSA)
            CALL FPABS(MSUM,MSUMA)
            CALL FPMPY(MSUMA,MTOL,MSUMA)
            IF (FPCOMP(MANSA,'LT',MSUMA)) GOTO 25
          ENDIF          
 20     CONTINUE
 25     CONTINUE
        IF(J.EQ.1) THEN
          KL=K
          CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
          GOTO 10
        ENDIF
        CALL FPSUB(MSUM,MSUM1,MTEMP)
        CALL FPABS(MTEMP,MTEMP)
        CALL FPABS(MSUM,MSUMA)
        CALL FPMPY(MSUMA,MTOLA,MSUMA)
        IF (FPCOMP(MTEMP,'LE',MSUMA)) GOTO 30
        CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
C        CALL FPM2DP(MSUM,DSUM)
C        PRINT *,KL,DSUM
 10   CONTINUE
 30   CONTINUE
      CALL FPMPY(MX,MX,MTEMP)
      CALL FPEXP(MTEMP,MB)
C      CALL FPMPY(MTEMP,MX,MA)
C      CALL FPMPY(MA,MB,MTEMP)
      CALL FPMPY(MTEMP,MB,MTEMP)
      CALL FPMPY(MTEMP,MSUM,MGOFX)
      RETURN
      END
      SUBROUTINE MQUAD8(MA,MB,MQUAD)
      DIMENSION MA(200),MB(200),MAPB(200),MBMA(200),MTEMP1(200),
     &MTEMP2(200),MTEMP3(200),MTEMP4(200),MTEMP5(200),MTEMP6(200),
     &MTEMP7(200),MTEMP8(200),MSUM(200),MFUN(200),MQUAD(200)
      DIMENSION IX1(4),IX2(4),IX3(4),IX4(4),IW1(4),IW2(4),IW3(4),IW4(4)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)
C
C     GAUSS ABSCISSAS AND WEIGHTS IN MP
      DATA (IX1(J),J=1,4)/0,18343464,24956498,05000000/
      DATA (IX2(J),J=1,4)/0,52553240,99163289,86000000/
      DATA (IX3(J),J=1,4)/0,79666647,74136267,40000000/
      DATA (IX4(J),J=1,4)/0,96028985,64975362,32000000/
      DATA (IW1(J),J=1,4)/0,36268378,33783619,83000000/
      DATA (IW2(J),J=1,4)/0,31370664,58778872,87000000/
      DATA (IW3(J),J=1,4)/0,22238103,44533744,71000000/
      DATA (IW4(J),J=1,4)/0,10122853,62903762,59000000/
C
      NDIG=NCDIG
      IF (INIT.EQ.0) THEN
        CALL FPEQU(MZERO,MX1,NDIG,NDIG)
        MX1(1)=IX1(1)
        MX1(2)=IX1(2)
        MX1(3)=IX1(3)
        MX1(4)=IX1(4)
        CALL FPEQU(MZERO,MX2,NDIG,NDIG)
        MX2(1)=IX2(1)
        MX2(2)=IX2(2)
        MX2(3)=IX2(3)
        MX2(4)=IX2(4)
        CALL FPEQU(MZERO,MX3,NDIG,NDIG)
        MX3(1)=IX3(1)
        MX3(2)=IX3(2)
        MX3(3)=IX3(3)
        MX3(4)=IX3(4)
        CALL FPEQU(MZERO,MX4,NDIG,NDIG)
        MX4(1)=IX4(1)
        MX4(2)=IX4(2)
        MX4(3)=IX4(3)
        MX4(4)=IX4(4)
        CALL FPEQU(MZERO,MW1,NDIG,NDIG)
        MW1(1)=IW1(1)
        MW1(2)=IW1(2)
        MW1(3)=IW1(3)
        MW1(4)=IW1(4)
        CALL FPEQU(MZERO,MW2,NDIG,NDIG)
        MW2(1)=IW2(1)
        MW2(2)=IW2(2)
        MW2(3)=IW2(3)
        MW2(4)=IW2(4)
        CALL FPEQU(MZERO,MW3,NDIG,NDIG)
        MW3(1)=IW3(1)
        MW3(2)=IW3(2)
        MW3(3)=IW3(3)
        MW3(4)=IW3(4)
        CALL FPEQU(MZERO,MW4,NDIG,NDIG)
        MW4(1)=IW4(1)
        MW4(2)=IW4(2)
        MW4(3)=IW4(3)
        MW4(4)=IW4(4)
        PRINT *,'GAUSSIAN ABSCISSAS AND WEIGHTS '
        CALL FPPRNT(MX1)
        CALL FPPRNT(MX2)
        CALL FPPRNT(MX3)
        CALL FPPRNT(MX4)
        CALL FPPRNT(MW1)
        CALL FPPRNT(MW2)
        CALL FPPRNT(MW3)
        CALL FPPRNT(MW4)
        PAUSE
        INIT=1
      ENDIF
      CALL FPADD(MA,MB,MAPB)
      CALL FPDIVI(MAPB,2,MAPB)
      CALL FPSUB(MB,MA,MBMA)
      CALL FPDIVI(MBMA,2,MBMA)
      CALL FPMPY(MX1,MBMA,MTEMP1)
      CALL FPMPY(MX2,MBMA,MTEMP2)
      CALL FPMPY(MX3,MBMA,MTEMP3)
      CALL FPMPY(MX4,MBMA,MTEMP4)
      I=-1
      CALL FPMPYI(MTEMP1,I,MTEMP5)
      CALL FPMPYI(MTEMP2,I,MTEMP6)
      CALL FPMPYI(MTEMP3,I,MTEMP7)
      CALL FPMPYI(MTEMP4,I,MTEMP8)
C
      CALL FPADD(MTEMP1,MAPB,MTEMP1)
      CALL FPADD(MTEMP2,MAPB,MTEMP2)
      CALL FPADD(MTEMP3,MAPB,MTEMP3)
      CALL FPADD(MTEMP4,MAPB,MTEMP4)
      CALL FPADD(MTEMP5,MAPB,MTEMP5)
      CALL FPADD(MTEMP6,MAPB,MTEMP6)
      CALL FPADD(MTEMP7,MAPB,MTEMP7)
      CALL FPADD(MTEMP8,MAPB,MTEMP8)
C
      CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
      CALL MFOFX(MTEMP1,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP2,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP3,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP4,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP5,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP6,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP7,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP8,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL FPMPY(MSUM,MBMA,MQUAD)
      RETURN
      END
      SUBROUTINE MFOFX(MT,MFUN)
C      DOUBLE PRECISION DSUM,DT
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
C        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MTEMP)
          CALL FPMPY(MTEMP,MT,MTEMP)
C          CALL FPSUB(MTRM1,MTEMP,MSUMT)
          CALL FPSUB(MONE,MTEMP,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          CALL FPABS(MSUM,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LE',MSUMA)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
C        CALL FPMPY(MRTPI,MT2,MAK)
        CALL FPMPY(MRTPI,MT,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
C      CALL FPM2DP(MSUMT,DSUM)
C      CALL FPM2DP(MT,DT)
C      PRINT *,I,DT,DSUM
C      PAUSE
      RETURN
      END
      SUBROUTINE DATAS(N,A,DNAME,FNAME,FWORK,IERR)
C 
C     WRITTEN BY D.E.AMOS, MAY, 1990.
C 
C     REFERENCE
C 
C     ABSTRACT
C        DATAS FORMATS A DOUBLE PRECISION SINGLY DIMENSIONED ARRAY
C        A(I), I=1,N INTO A DATA STATEMENT IN FILE FNAME. THE A
C        VECTOR IS FORMATTED BY 3E21.14 ACROSS A LINE AND GIVEN THE
C        NAME SPECIFIED BY THE PARAMETER DNAME. DATAS ALSO CREATES
C        A TEMPORARY FILE TEMPX.DAT.
C
C     DESCRIPTION OF ARGUMENTS
C 
C        INPUT
C           N      -  NUMBER OF ELEMENTS OF A, 1.LE.N.LE.99
C           A      -  DOUBLE PRECISION ARRAY OF DIMENSION AT LEAST N
C           DNAME  -  CHARACTER*K VARIABLE FOR THE NAME OF THE DATA
C                     IN THE FILE DESIGNATED BY FNAME, 1.LE.K.LE.10
C           FNAME  -  CHARACTER*K VARIABLE FOR THE FILE NAME WHERE THE
C                     DATA STATEMENT IS TO BE WRITTEN, 1.LE.K.LE.12
C           FWORK  -  CHARACTER*K VARIABLE FOR A SCRATCH FILE NAME,
C                     1.LE.K.LE.12
C
C
C        OUTPUT
C           ****   -  FILE WITH ALIAS FNAME WHICH CONTAINS THE DATA
C                     STATEMENT
C           ****      FILE WITH ALIAS FWORK USED AS A SCRATCH FILE
C           IERR   -  ERROR INDICATOR
C                     IERR=0  NORMAL RETURN, NO ERRORS
C                     IERR=1  N IS LESS THAN 1
C                     IERR=2  N IS LARGER THAN 99
C                     IERR=3  DNAME IS BLANK
C                     IERR=4  FNAME IS BLANK
C                     IERR=5  FWORK IS BLANK
C 
C     ERROR CONDITIONS
C        N LESS THAN 1 OR GREATER THAN 99 IS AN ERROR
C        EITHER DNAME OR FNAME BLANK IS AN ERROR
C-----------------------------------------------------------------------
      CHARACTER*(*) DNAME,FNAME,FWORK
      CHARACTER*72 LINE
      CHARACTER*2 NC
      CHARACTER*21 B(99)
      DOUBLE PRECISION A(N)
      IERR=0
      IF (N.GT.99) THEN
        IERR=2
        RETURN
      ENDIF
      IF (N.LT.1) THEN
        IERR=1
        RETURN
      ENDIF
      DO 5 I=1,12
        IF(FNAME(I:I).NE.' ') GO TO 6
    5 CONTINUE
      IERR=4
      RETURN
    6 CONTINUE
      DO 7 I=1,12
        IF(FWORK(I:I).NE.' ') GO TO 8
    7 CONTINUE
      IERR=5
      RETURN
    8 CONTINUE
      DO 15 I=1,10
        IF(DNAME(I:I).NE.' ') GO TO 20
   15 CONTINUE
      IERR=3
      RETURN
   20 CONTINUE
      K1=I
      DO 25 I=K1,10
        IF(DNAME(I:I).EQ.' ') GO TO 30
   25 CONTINUE
      I=11
   30 CONTINUE
      K2=I-1
      OPEN(10,FILE=FNAME,STATUS='UNKNOWN')
      REWIND(10)
C     POSITION FILE AT END OF LAST LINE
      DO 31 LOOP=1,400
        READ(10,'( )',END=32)
 31   CONTINUE
 32   CONTINUE
      ENDFILE(10)
      BACKSPACE(10)
C
      OPEN(20,FILE=FWORK,STATUS='UNKNOWN')
      WRITE(20,800) N
      REWIND(20)
      READ(20,801) NC
      REWIND(20)
      LINE='      DATA ('//DNAME(K1:K2)//'(I), I=1,'//NC//')/'
      WRITE(10,900)LINE
      DO 35 I=1,N
        WRITE(20,901) A(I)
   35 CONTINUE
      REWIND(20)
      DO 40 I=1,N
        READ(20,902) B(I)
   40 CONTINUE
      K=0
      DO 45 I=1,N
        LINE='     &'
        M=6
        DO 50 J=1,3
          K=K+1
          LINE=LINE(1:M)//B(K)(1:21)
          M=M+21
          IF(K.EQ.N) THEN
            LINE=LINE(1:M)//'/'
            GO TO 60
          ELSE
            LINE=LINE(1:M)//','
          ENDIF
          M=M+1
   50   CONTINUE
   60   CONTINUE
        WRITE(10,900)LINE
        IF (K.EQ.N) GO TO 70
   45 CONTINUE
   70 CONTINUE
      CLOSE(10)
C-----------------------------------------------------------------------
C     CLEAN UP THE TEMPORARY FILE
C-----------------------------------------------------------------------
      LINE='  '
      REWIND(20)
      WRITE(20,900) LINE
      CLOSE(20)
  800 FORMAT(I2)
  801 FORMAT(A2)
  900 FORMAT(A72)
  901 FORMAT(D21.14)
  902 FORMAT(A21)
      RETURN
      END
************************************************************************
      PROGRAM MCHEBYEV
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*72 LINE
      COMMON /CINIT/INIT
      DIMENSION COEF(100)
      OPEN(7,FILE='MCHEBYEV.TXT',STATUS='UNKNOWN')
C-----------------------------------------------------------------------
C     READ A, B AND 18 DIGIT COEFFICIENTS FROM FILE CHEBY.DAT
C-----------------------------------------------------------------------
      OPEN(10,FILE='MPCHEBY.DAT',STATUS='UNKNOWN')
      REWIND(10)
      READ(10,196)NODES
 196  FORMAT(7X,I3)
      READ(10,199) A
 199  FORMAT(3X,D30.18)
      READ(10,199) B
      DO 4 J=1,3      
C       SKIP OVER THE FIRST DATA STATEMENT 
        READ(10,200) LINE
 4    CONTINUE
      DO 5 J=1,400
        READ(10,200) LINE
C        PRINT *,LINE
 200    FORMAT(A72)
        IF(LINE(7:10).EQ.'DATA') GOTO 201
 5    CONTINUE
      STOP
 201  CONTINUE
      M=1
      DO 6 J=1,400
        READ(10,202,IOSTAT=K) COEF(M),COEF(M+1)
C        PRINT *,J,K,COEF(M),COEF(M+1)
        IF(K.NE.0) GOTO 7
        M=M+2
 202    FORMAT(8X,D26.20,4X,D26.20)
 6    CONTINUE
 7    CONTINUE
C     K=-1 ONLY IF THE READ PRODUCES TWO ZERO ENTRIES 
      CLOSE(10)
C
      IF(COEF(M).EQ.0.0D0.AND.COEF(M+1).EQ.0.0D0) THEN
        M=M-1
        IF(COEF(M).EQ.0.0D0) THEN
          M=M-1
        ENDIF
      ELSE       
        IF(COEF(M).EQ.0.0D0) THEN
          M=M-1
        ENDIF
      ENDIF
C
      INIT=0
 400  CONTINUE
C      PRINT 195, NODES
      PRINT 198, NODES,A,B
      PRINT 194
      K=1
      MF=INT(M/2)
      DO 18 MM=1,MF
        PRINT 203, MM,COEF(K),COEF(K+1)
        K=K+2
 18   CONTINUE
      IF(MOD(M,2).NE.0) THEN
        PRINT 203, MM,COEF(M)
      ENDIF
 9    CONTINUE 
C      PRINT *, ' '
      PRINT *, M,' COEFFICIENTS READ FROM FILE MPCHEBY.DAT'
C      PRINT *, ' '
      PRINT *,'INPUT THE NUMBER OF COEFFICIENTS TO BE USED IN THE SUM (S
     &TOP=0):  '
      READ *,N
      IF (N.GT.M) GOTO 9
      IF (N.EQ.0) STOP
      REWIND(7)
C      WRITE(7,195) NODES
C 195  FORMAT(' NODES= ',I3)
      WRITE(7,198) NODES,A,B
 198  FORMAT('  NODES= ',I3,'  A=',D25.18,' B=',D25.18)
      WRITE(7,194)
 194  FORMAT(7X,'CHEBYSHEV COEFFICIENTS')
      K=1
      MF=INT(M/2)
      DO 8 MM=1,MF
        WRITE(7,203) MM,COEF(K),COEF(K+1)
        K=K+2
 8    CONTINUE
      IF(MOD(M,2).NE.0) THEN
        WRITE(7,203) MM,COEF(M)
      ENDIF
 203  FORMAT(2X,I5,2D30.18)
      WRITE(7,193)N
 193  FORMAT(7X,'NUMBER OF COEFFICIENTS USED IN THE SUM: ',I3)
      WRITE(7,197)
 197  FORMAT(/'               X            Y         REL ERR')
      PRINT 197
      APB=A+B
      BMA=B-A
      AJ=1.0D0
      JL=51
      AJL=DBLE(FLOAT(JL))
      DO 10 J=1,JL
        X=((AJ-1.0D0)*B+(AJL-AJ)*A)/(AJL-1.0D0)
C        X2=X*X
        CALL DCHBYS(APB,BMA,X,N,COEF,ANS)
C        X3=X2*X
C        ANS=ANS*DEXP(-X2)/X3
C         Y=MPGOFX(X)
        Y=DPGOFX(X)
C        Y=1.0D0/X
        ERR=DABS(ANS-Y)/DABS(Y)
        WRITE(7,300) J,X,Y,ERR
 300    FORMAT(4X,I5,3D13.5)
        PRINT 300, J,X,Y,ERR
        IF(MOD(J,20).EQ.0) PAUSE
        AJ=AJ+1.0D0
 10   CONTINUE
      PAUSE
      DO 401 J=1,25
        PRINT *
 401  CONTINUE
      GOTO 400
      END
      SUBROUTINE DCHBYS(APB,BMA,X,N,COEF,SUM)
C-----------------------------------------------------------------------
C     CHBYS SUMS A CHEBYSHEV SERIES WITH COEFFICIENTS COEF(I), I=1,N
C     ON [A,B] AT X. APB=A+B , BMA=B-A
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION COEF(N)
      T=(X+X-APB)/BMA
      TT=T+T
      F1=0.0D0
      F2=0.0D0
      NM=N-1
      DO 160 I=1,NM
        J=N-I+1
        TEMP=F1
        F1=TT*F1-F2+COEF(J)
        F2=TEMP
  160 CONTINUE
      SUM=T*F1-F2+COEF(1)
      RETURN
      END
      DOUBLE PRECISION FUNCTION GOFX(X)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFOFX/ CX      
C      DATA RTPI /1.772453850905516D0/
      EXTERNAL FOFX
      CX=X
      INIT=0
      X1=X
      SIG=2.0D0
      REL=0.50D-15
C      PRINT *,"X=",X
C      PAUSE
      CALL DQUAD8(FOFX,INIT,X1,SIG,REL,X2,QANS,IERR)
      SIG=6.0D0
      CALL DQUAD8(FOFX,INIT,X1,SIG,REL,X2,QANS,IERR)
      XSQ=X*X
      GOFX=QANS*XSQ*X
      GOFX=GOFX*DEXP(XSQ)
C      GOFX=QANS
C     PRINT *,"GOFX=",X,GOFX
C      PAUSE
      RETURN
      END
      DOUBLE PRECISION FUNCTION FOFX(X)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      KODE=2
      FOFX=DRERF(X,KODE,NZ)/X
      RETURN
      END
      DOUBLE PRECISION FUNCTION DPGOFX(X)
      DOUBLE PRECISION X,PI,DH,DTOL
      DOUBLE PRECISION DPMAX
C     SINGLE PRECISION SPMAX

      PARAMETER ( MXNDIG=256 , NBITS=32 ,
     *          LPACK  = (MXNDIG+1)/2 + 1 , LUNPCK = (6*MXNDIG)/5 + 20,
     *          LMWA   = 2*LUNPCK         , LJSUMS = 8*LUNPCK,
     *          LMBUFF = ((LUNPCK+3)*(NBITS-1)*301)/2000 + 6  )
C
C
      DIMENSION MX(200),MGOFX(200)

C
      COMMON /FMUSER/ NDIG,JBASE,JFORM1,JFORM2,KRAD,
     *                KW,NTRACE,LVLTRC,KFLAG,KWARN,KROUND
C
      COMMON /FM/ MWA(LMWA),NCALL,MXEXP,MXEXP2,KARGSW,KEXPUN,KEXPOV,
     *            KUNKNO,IUNKNO,RUNKNO,MXBASE,MXNDG2,KSTACK(19),
     *            MAXINT,SPMAX,DPMAX
C
      COMMON /FMSAVE/ NDIGPI,NJBPI,NDIGE,NJBE,NDIGLB,NJBLB,NDIGLI,NJBLI,
     *                MPISAV(LUNPCK),MESAV(LUNPCK),MLBSAV(LUNPCK),
     *                MLN1(LUNPCK),MLN2(LUNPCK),MLN3(LUNPCK),
     *                MLN4(LUNPCK)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)
C
C-----------------------------------------------------------------------
      IF (INIT.EQ.0) THEN
        NPREC=60
        CALL FPSET(NPREC)
C       FOR JBASE=10000, NDIG=NPREC/4+2
        NDIG=INT(NPREC/4)+5
        NCDIG=NDIG
        K=0
        CALL FPI2M(K,MZERO)
        K=1
        CALL FPI2M(K,MONE)
        K=2        
        CALL FPI2M(K,MTWO)
        K=9
        CALL FPI2M(K,MNINE)
        CALL FPPI(MPI)
        CALL FPM2DP(MPI,PI)
        CALL FPSQRT(MPI,MRTPI)
        CALL FPDIV(MONE,MRTPI,MTEMP)
        CALL FPADD(MTEMP,MTEMP,MTRTPI)
C       MTRTPI=2/RTPI
        DTOL=0.50D-16
        CALL FPDP2M(DTOL,MTOL)
        DH=0.5D0
        CALL FPDP2M(DH,MDH)
C       INIT=1
C       INIT IS SET TO 1 AFTER THE INITIALIZATION OF MQUAD8
      ENDIF
      CALL FPDP2M(X,MX)
      CALL MPGOFX(MX,MGOFX)
C      CALL MFOFXA(MX,MGOFX)
      CALL FPM2DP(MGOFX,DPGOFX)
      END
      SUBROUTINE MPGOFX(MX,MGOFX)
C
C     COMPUTES INT ON (X,INF) OF ERFC(X)/X OR ERFC(X) IN MP ARITHMETIC
C     USING GAUS8 FORMULA ON REPEATED SUBDIVIDED INTERVALS
C
      DOUBLE PRECISION DSUM
      LOGICAL FPCOMP
C
      DIMENSION MX(200),MGOFX(200)
C
      DIMENSION MSUM(200),MSUMA(200),MANS(200),MANSA(200),
     &MTEMP(200),MSUM1(200),MDX(200),MTEMP1(200),MA(200),MB(200)
C
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPEQU(MDH,MDX,NDIG,NDIG)
      CALL FPEQU(MZERO,MSUM1,NDIG,NDIG)
      KL=100
      DO 10 J=1,100
        CALL FPDIVI(MDX,2,MDX)
        KL=KL+KL
        CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
        CALL FPEQU(MX,MB,NDIG,NDIG)
        DO 20 K=1,KL
          CALL FPEQU(MB,MA,NDIG,NDIG)
          CALL FPADD(MA,MDX,MB)
          CALL MQUAD8(MA,MB,MANS)
          CALL FPADD(MSUM,MANS,MSUM)
          CALL FPM2DP(MSUM,DSUM)
          IF(J.EQ.1) THEN
            CALL FPABS(MANS,MANSA)
            CALL FPABS(MSUM,MSUMA)
            CALL FPMPY(MSUMA,MTOL,MSUMA)
            IF (FPCOMP(MANSA,'LT',MSUMA)) GOTO 25
          ENDIF          
 20     CONTINUE
 25     CONTINUE
        IF(J.EQ.1) THEN
          KL=K
          CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
          GOTO 10
        ENDIF
        CALL FPSUB(MSUM,MSUM1,MTEMP)
        CALL FPABS(MTEMP,MTEMP)
        CALL FPABS(MSUM,MSUMA)
        CALL FPMPY(MSUMA,MTOL,MSUMA)
        IF (FPCOMP(MTEMP,'LT',MSUMA)) GOTO 30
        CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
 10   CONTINUE
 30   CONTINUE
      CALL FPMPY(MX,MX,MTEMP)
      CALL FPEXP(MTEMP,MTEMP1)
C      CALL FPMPY(MTEMP,MX,MTEMP)
      CALL FPMPY(MTEMP1,MTEMP,MTEMP)
      CALL FPMPY(MTEMP,MSUM,MGOFX)
      RETURN
      END
      SUBROUTINE MQUAD8(MA,MB,MQUAD)
      DOUBLE PRECISION DQUAD
      DIMENSION MA(200),MB(200),MAPB(200),MBMA(200),MTEMP1(200),
     &MTEMP2(200),MTEMP3(200),MTEMP4(200),MTEMP5(200),MTEMP6(200),
     &MTEMP7(200),MTEMP8(200),MSUM(200),MFUN(200),MQUAD(200)
      DIMENSION IX1(4),IX2(4),IX3(4),IX4(4),IW1(4),IW2(4),IW3(4),IW4(4)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)

C
C     GAUSS ABSCISSAS AND WEIGHTS IN MP
      DATA (IX1(J),J=1,4)/0,18343464,24956498,05000000/
      DATA (IX2(J),J=1,4)/0,52553240,99163289,86000000/
      DATA (IX3(J),J=1,4)/0,79666647,74136267,40000000/
      DATA (IX4(J),J=1,4)/0,96028985,64975362,32000000/
      DATA (IW1(J),J=1,4)/0,36268378,33783619,83000000/
      DATA (IW2(J),J=1,4)/0,31370664,58778872,87000000/
      DATA (IW3(J),J=1,4)/0,22238103,44533744,71000000/
      DATA (IW4(J),J=1,4)/0,10122853,62903762,59000000/
C
      NDIG=NCDIG
      IF (INIT.EQ.0) THEN
        CALL FPEQU(MZERO,MX1,NDIG,NDIG)
        MX1(1)=IX1(1)
        MX1(2)=IX1(2)
        MX1(3)=IX1(3)
        MX1(4)=IX1(4)
        CALL FPEQU(MZERO,MX2,NDIG,NDIG)
        MX2(1)=IX2(1)
        MX2(2)=IX2(2)
        MX2(3)=IX2(3)
        MX2(4)=IX2(4)
        CALL FPEQU(MZERO,MX3,NDIG,NDIG)
        MX3(1)=IX3(1)
        MX3(2)=IX3(2)
        MX3(3)=IX3(3)
        MX3(4)=IX3(4)
        CALL FPEQU(MZERO,MX4,NDIG,NDIG)
        MX4(1)=IX4(1)
        MX4(2)=IX4(2)
        MX4(3)=IX4(3)
        MX4(4)=IX4(4)
        CALL FPEQU(MZERO,MW1,NDIG,NDIG)
        MW1(1)=IW1(1)
        MW1(2)=IW1(2)
        MW1(3)=IW1(3)
        MW1(4)=IW1(4)
        CALL FPEQU(MZERO,MW2,NDIG,NDIG)
        MW2(1)=IW2(1)
        MW2(2)=IW2(2)
        MW2(3)=IW2(3)
        MW2(4)=IW2(4)
        CALL FPEQU(MZERO,MW3,NDIG,NDIG)
        MW3(1)=IW3(1)
        MW3(2)=IW3(2)
        MW3(3)=IW3(3)
        MW3(4)=IW3(4)
        CALL FPEQU(MZERO,MW4,NDIG,NDIG)
        MW4(1)=IW4(1)
        MW4(2)=IW4(2)
        MW4(3)=IW4(3)
        MW4(4)=IW4(4)
        PRINT *,'GAUSSIAN ABSCISSAS AND WEIGHTS '
        CALL FPPRNT(MX1)
        CALL FPPRNT(MX2)
        CALL FPPRNT(MX3)
        CALL FPPRNT(MX4)
        CALL FPPRNT(MW1)
        CALL FPPRNT(MW2)
        CALL FPPRNT(MW3)
        CALL FPPRNT(MW4)
        PAUSE
        INIT=1
      ENDIF
      CALL FPADD(MA,MB,MAPB)
      CALL FPDIVI(MAPB,2,MAPB)
      CALL FPSUB(MB,MA,MBMA)
      CALL FPDIVI(MBMA,2,MBMA)
      CALL FPMPY(MX1,MBMA,MTEMP1)
      CALL FPMPY(MX2,MBMA,MTEMP2)
      CALL FPMPY(MX3,MBMA,MTEMP3)
      CALL FPMPY(MX4,MBMA,MTEMP4)
      I=-1
      CALL FPMPYI(MTEMP1,I,MTEMP5)
      CALL FPMPYI(MTEMP2,I,MTEMP6)
      CALL FPMPYI(MTEMP3,I,MTEMP7)
      CALL FPMPYI(MTEMP4,I,MTEMP8)
C
      CALL FPADD(MTEMP1,MAPB,MTEMP1)
      CALL FPADD(MTEMP2,MAPB,MTEMP2)
      CALL FPADD(MTEMP3,MAPB,MTEMP3)
      CALL FPADD(MTEMP4,MAPB,MTEMP4)
      CALL FPADD(MTEMP5,MAPB,MTEMP5)
      CALL FPADD(MTEMP6,MAPB,MTEMP6)
      CALL FPADD(MTEMP7,MAPB,MTEMP7)
      CALL FPADD(MTEMP8,MAPB,MTEMP8)
C
      CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
      CALL MFOFX(MTEMP1,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP2,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP3,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP4,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP5,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP6,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP7,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP8,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL FPM2DP(MFUN,DQUAD)
C      PRINT*,'MFUN:',DQUAD
      CALL FPMPY(MSUM,MBMA,MQUAD)
C      CALL FPM2DP(MQUAD,DQUAD)
C      PRINT *,'IN MQUAD, ANS=',DQUAD
C      PAUSE
      RETURN
      END
      SUBROUTINE MFOFXA(MT,MFUN)
      DOUBLE PRECISION DSUM,DMT
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPM2DP(MT,DMT)
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
C        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          CALL FPM2DP(MSUM,DSUM)
C          PRINT*,'IN MFOFX, X,SUM ',DMT,DSUM
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MSUMT)
C          CALL FPSUB(MTRM1,MTEMP,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          IF (FPCOMP(MTRMA,'LE',MTOL)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
        CALL FPMPY(MRTPI,MT2,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
      RETURN
      END
      SUBROUTINE MFOFX(MT,MFUN)
      DOUBLE PRECISION DSUM,DMT
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPM2DP(MT,DMT)
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          CALL FPM2DP(MSUM,DSUM)
C          PRINT*,'IN MFOFX, X,SUM ',DMT,DSUM
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MTEMP)
C          CALL FPSUB(MTRM1,MTEMP,MSUMT)
          CALL FPMPY(MTEMP,MT,MTEMP)
          CALL FPSUB(MONE,MTEMP,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          IF (FPCOMP(MTRMA,'LE',MTOL)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
C        CALL FPMPY(MRTPI,MT2,MAK)
        CALL FPMPY(MRTPI,MT,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
      RETURN
      END
************************************************************************
      PROGRAM DPCHEBY
C 
C     WRITTEN BY D.E.AMOS, DECEMBER,1972; MODIFIED MAY, 1990.
C 
C     REFERENCE SANDIA NATIONAL LABORATORIES REPORT SC-DR-72 0917
C 
C     ABSTRACT
C
C     PROGRAM DPCHEBY CALCULATES THE CHEBYSHEV COEFFICIENTS FOR A
C     DP FUNCTION GOFX(X) ON AN INTERVAL [A,B].
C
C     FINAL EXPANSION IS  S = 0.5*A(1)+SUM( A(J)*CT(J,XX), J=2,N) WHERE
C
C          XX=(2*X-(A+B))/(B-A)   FOR    A.LE.X.LE.B
C                                 OR
C          XX=2*A/X-1.0           FOR    A.LE.X.LT.INFINITY,
C 
C     T(J,XX) ARE CHEBYSHEV POLYNOMIALS AND A(J) ARE THE COEFFICIENTS
C     GENERATED BY THIS PROGRAM AND STORED IN A DATA STATEMENT IN A
C     FILE CALLED DPCHEBY.DAT.
C
C     DESCRIPTION OF ARGUMENTS
C
C     INPUT 
C       A, B    = INTERVAL [A,B] END POINTS. IF A=B, THE EXPANSION
C                 IS TAKEN ON A TO INFINITY USING T(J,2*A/X-1)
C       N       = THE NUMBER OF COEFFICIENTS DESIRED (USUALLY N < 26)
C       NCHEBY  = NUMBER OF CHEBYSHEV NODES FOR THE APPROXIMATION
C                 NCHEBY IS MUCH LARGER THAN N  (USUALLY NCHEBY > 40)
C       IERR    = 1 GIVES A RELATIVE ERROR PRINT
C               = 0 GIVES AN ABSOLUTE ERROR PRINT
C       DNAME   = NAME OF THE COEFFICIENT ARRAY IN THE DATA STATEMENT
C       GOFX(X) = DOUBLE PRECISION FUNCTION BEING APPROXIMATED. X MUST
C                 BE DECLARED DOUBLE AND GOFX PROGRAMMED BY THE USER
C                 AS DOUBLE PRECISION FUNCTION GOFX(X)
C     OUTPUT
C       FILE DPCHEBY.FIT SHOWING THE COEFFICIENTS AND ERRORS
C       FILE DPCHEBY.DAT CONTAINING THE DATA STATEMENT FOR THE
C                      COEFFICIENTS 0.5*A(1) THRU A(N)
C       FILE CHEBZ.DAT IS CREATED AND USED AS A SCRATCH FILE
C
C     PROGRAM DPCHEBY USES GOFX AND DATAS PLUS ANY OTHERS RELATED TO THE
C     COMPUTATION OF GOFX
C-----------------------------------------------------------------------
      CHARACTER*12 FNAME,DNAME,FWORK
      CHARACTER*10 ILAB(2)
C      CHARACTER*1 CA,CB
      CHARACTER*72 LINE
      CHARACTER*2 CAB
      DOUBLE PRECISION T,PI,DN,THET,VK,VONE,TK,TONE,APB,BMA,X 
      DOUBLE PRECISION Z,TZ,B1,B2,F,TEMP,ANS,AA,BB,AR 
      DOUBLE PRECISION GOFX
      DATA (ILAB(I),I=1,2)/' RELATIVE ',' ABSOLUTE '/
      DIMENSION T(301),F(301),X(301),AR(301)
C
      COMMON /CINIT/INIT
C-----------------------------------------------------------------------
      PRINT 950 
  950 FORMAT(' APPROXIMATION INTERVAL A, B= ')
      READ *,AA,BB
      PRINT 951
  951 FORMAT(' NUMBER OF CHEBYSHEV COEFFICIENTS DESIRED (N~25), N= ')
      READ *,NAR
      PRINT 952
  952 FORMAT(' NUMBER OF CHEBYSHEV NODES (NCHEBY~40), NCHEBY= ')
      READ *,NCHEBY
      PRINT 953
  953 FORMAT(' NAME OF THE COEFFICIENTS IN THE DATA STATEMENT, DNAME= ')
      READ 900,DNAME
  900 FORMAT(A8)
      PRINT 954
  954 FORMAT(' 1 FOR RELATIVE ERROR; 2 FOR ABSOLUTE ERROR, IERR= ')
      READ *,IERR
      IF (IERR.NE.1 .AND. IERR.NE.2) THEN
        IERR=2
      ENDIF
C-----------------------------------------------------------------------
C     END INPUT 
C-----------------------------------------------------------------------
      INIT=0
      N1 = NCHEBY-1
      N2 = NCHEBY-1
      PI=3.1415926535897932384626433D0
      APB=AA+BB 
      BMA=BB-AA 
      C1= SNGL(APB)
      C2= SNGL(BMA)
      IF(BB.EQ.AA) C1=SNGL(AA+AA) 
      OPEN(10,FILE='DPCHEBY.FIT',STATUS='UNKNOWN')
      DO 500 N=N1,N2 
        IF (NAR.GT.N) THEN
          NAR=N+1
        ENDIF
        NP1=N+1
        DN=DBLE(FLOAT(N))
        THET=PI/DN
        T(1)=1.0D0
        T(2)=DCOS(THET)
        TONE =T(2)
        VONE=DSIN(THET)
        VK = VONE
C-----------------------------------------------------------------------
C     GET CHEBYSHEV NODES ON [A,B] IN X ARRAY
C     TRIG IDENTITIES SIN((K+1)*X) = SIN(K*X)*COS(X)+COS(K*X)*SIN(X)
C                     COS((K+1)*X) = COS(K*X)*COS(X)-SIN(K*X)*SIN(X)
C     FOR CHEBYSHEV NODES T(K)=COS((K-1)*PI/N), K=1,N+1 BY RECURRENCE
C-----------------------------------------------------------------------
        DO 5 K=2,N
          TK=T(K)
          T(K+1)=TONE*TK-VK*VONE
          VK=VK*TONE+TK*VONE
          X(K)=(TK*BMA+APB)/2.0D0
          IF(AA.NE.BB) GO TO 5
          IF(TK.NE.-1.0D0) GO TO 10
          X(K)=1.0D+16
          GO TO 5
   10     CONTINUE
          X(K)=(AA+AA)/(1.0D0+TK)
    5   CONTINUE
        X(1)=(T(1)*BMA+APB)/2.0D0
        X(NP1)=(T(NP1)*BMA+APB)/2.0D0
        IF(AA.NE.BB) GO TO 20
        X(1)=(AA+AA)/(1.0D0+T(1))
        X(NP1)=1.0D+16
   20   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 52)NP1,AA,BB
   52 FORMAT(' NODES='I5/' A    ='D23.14/' B    ='D23.14/)
        WRITE(10, 51)(X(I),I=1,NP1)
   51 FORMAT(' CHEBYSHEV NODES  X(J)'/(1X,3D23.14))
C-----------------------------------------------------------------------
C     FUNCTION EVALUATION AT CHEBYSHEV NODES X(I), I=1,NP1
C-----------------------------------------------------------------------
        DO 7 I=1,NP1
          F(I) = GOFX(X(I))
    7   CONTINUE
        F(NP1)=F(NP1)/2.0D0
C-----------------------------------------------------------------------
C     END OF FUNCTION EVALUATION
C-----------------------------------------------------------------------
        DO 100 IR=1,NP1
          Z=T(IR)
          TZ=Z+Z
          B2=0.0D0
          B1= 0.0D0
          DO 15 J=1,N
            K=NP1-J+1
            TEMP=B1
            B1=TZ*B1-B2+F(K)
            B2=TEMP
   15     CONTINUE
          ANS=Z*B1-B2+F(1)/2.0D0
          AR(IR)=(ANS+ANS)/DN
  100   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10,153) (AR(K),K=1,NP1)
  153 FORMAT(' CHEBYSHEV COEFFICIENTS A(J)'/(1X,3D23.14))
        F(NP1)=F(NP1)+F(NP1)
        WRITE(10, 54)(F(I),I=1,NP1)
   54 FORMAT(' FUNCTION VALUES AT CHEBYSHEV NODES F(X(J))'/(1X,3D23.14))
C-----------------------------------------------------------------------
        NARM=NAR-1
        DO 30 I=1,NP1
          XX=SNGL(X(I))
          IF(AA.NE.BB) GO TO 26
          SZ=C1/XX-1.0E0
          GO TO 27
   26     CONTINUE
          SZ=(XX+XX-C1)/C2
   27     CONTINUE
          STZ=SZ+SZ
          S2=0.0E0
          S1=0.0E0
          DO 25 J=1,NARM
            K=NAR-J+1
            ST=S1
            SA=AR(K)
            S1=STZ*S1-S2+SA
            S2=ST
   25     CONTINUE
          SA=AR(1)
          FX=SZ*S1-S2+SA/2.0
          T(I)=DBLE(FX)-F(I)
          IF(IERR.EQ.1) THEN
            IF(F(I).NE.0.0D0) THEN
              T(I)=T(I)/F(I)
            ENDIF
          ENDIF
   30   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 53)ILAB(IERR),NAR,AA,BB,(T(I),I=1,NP1)
   53   FORMAT(/' SINGLE PRECISION',A10,'ERRORS AT CHEBYSHEV NODES USING
     &',I5,' TERMS'/' OF THE CHEBYSHEV SUM ON',2D23.14//(1X,3D23.14))
C-----------------------------------------------------------------------
        DO 35 I=1,NP1
          IF(AA.NE.BB) GO TO 36
          Z=(AA+AA)/X(I)-1.0D0
          GO TO 37
   36     CONTINUE
          Z=(X(I)+X(I)-APB)/BMA
   37     CONTINUE
          TZ=Z+Z
          B2=0.0D0
          B1=0.0D0
          DO 40 J=1,NARM
            K=NAR-J+1
            TEMP=B1
            B1=TZ*B1-B2+AR(K)
            B2=TEMP
   40     CONTINUE
          TEMP=Z*B1-B2+AR(1)/2.0D0
          T(I)=TEMP-F(I)
          IF(IERR.EQ.1) THEN
            IF(F(I).NE.0.0D0) THEN
              T(I)=T(I)/F(I)
            ENDIF
          ENDIF
   35   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 55)ILAB(IERR),NAR,AA,BB,(T(I),I=1,NP1)
   55   FORMAT(/' DOUBLE PRECISION',A10,'ERRORS AT CHEBYSHEV NODES USING
     &',I5,' TERMS'/' OF THE CHEBYSHEV SUM ON',2D23.14//(1X,3D23.14))
C-----------------------------------------------------------------------
  500 CONTINUE
      CLOSE(10)
      FNAME='DPCHEBY.DAT'
      FWORK='DPCHEBZ.DAT'
      AR(1)=AR(1)*0.5D0
      OPEN(10,FILE='DPCHEBY.DAT',STATUS='UNKNOWN')
      REWIND(10)
      WRITE(10,531)NP1
 531  FORMAT('NODES= ',I3)
      WRITE(10,530) AA,BB
 530  FORMAT('A= 'D30.18/'B= ',D30.18/)
      CLOSE(10)
C-----------------------------------------------------------------------
C     WRITE NORMAL DATA STATEMENT WITH 14 DIGITS, 3 ON A LINE
C-----------------------------------------------------------------------
      CALL DATAS(NAR,AR,DNAME,FNAME,FWORK,KERR)
C-----------------------------------------------------------------------
C     WRITE DATA STATEMENT IN 2 COLUMNS OF 18 DIGITS
C-----------------------------------------------------------------------
      DO 515 I=1,10
        IF(DNAME(I:I).NE.' ') GO TO 520
  515 CONTINUE
      PRINT *, 'THE DATA NAME IS BLANK'
      STOP
  520 CONTINUE
      K1=I
      DO 501 I=K1,10
        IF(DNAME(I:I).EQ.' ') GO TO 502
 501  CONTINUE
      I=11
 502  CONTINUE
      K2=I-1
C     READ TO THE BOTTOM OF THE FILE (TO SKIP WHAT IS ALREADY THERE)
      OPEN(15,FILE=FNAME,STATUS='UNKNOWN')
      REWIND(15)
      DO 511 LOOP=1,400
        READ(15,'( )',END=512)
 511  CONTINUE
 512  CONTINUE
      ENDFILE(15)
      BACKSPACE(15)
      OPEN(20,FILE=FWORK,STATUS='UNKNOWN')
      WRITE(20,509) NAR
 509  FORMAT(I2)
      REWIND(20)
      READ(20,510) CAB
 510  FORMAT(A2)
      CLOSE(20)
      LINE='      DATA ('//DNAME(K1:K2)//'(I), I=1,'//CAB//')/'
      WRITE(15,506)LINE
 506  FORMAT(A72)
      WRITE(15,507) (AR(J),J=1,NAR)
 507  FORMAT(5X,'&',D30.18,',',D30.18,',')
      CLOSE(15)
C-----------------------------------------------------------------------
      IF (KERR.NE.0) THEN
        PRINT *,' ERROR IN DATAS, IERR = ',KERR
      ELSE
        PRINT *,'  '
        PRINT *,' APPROXIMATION USING ',NP1,' CHEBYSHEV NODES'
        PRINT *,'  '
        PRINT *,' FILES WRITTEN:'
        PRINT *,'  '
        PRINT *,' DPCHEBY.FIT:    THE COEFFIENTS, NODES, AND ERRORS,'
        PRINT *,' DPCHEBY.DAT:    THE DATA STATEMENT,'
        PRINT *,' DPCHEBZ.DAT:    SCRATCH FILE.'
      ENDIF
      PAUSE
      STOP
      END
      SUBROUTINE DATAS(N,A,DNAME,FNAME,FWORK,IERR)
C 
C     WRITTEN BY D.E.AMOS, MAY, 1990.
C 
C     REFERENCE
C 
C     ABSTRACT
C        DATAS FORMATS A DOUBLE PRECISION SINGLY DIMENSIONED ARRAY
C        A(I), I=1,N INTO A DATA STATEMENT IN FILE FNAME. THE A
C        VECTOR IS FORMATTED BY 3E21.14 ACROSS A LINE AND GIVEN THE
C        NAME SPECIFIED BY THE PARAMETER DNAME. DATAS ALSO CREATES
C        A TEMPORARY FILE TEMPX.DAT.
C
C     DESCRIPTION OF ARGUMENTS
C 
C        INPUT
C           N      -  NUMBER OF ELEMENTS OF A, 1.LE.N.LE.99
C           A      -  DOUBLE PRECISION ARRAY OF DIMENSION AT LEAST N
C           DNAME  -  CHARACTER*K VARIABLE FOR THE NAME OF THE DATA
C                     IN THE FILE DESIGNATED BY FNAME, 1.LE.K.LE.10
C           FNAME  -  CHARACTER*K VARIABLE FOR THE FILE NAME WHERE THE
C                     DATA STATEMENT IS TO BE WRITTEN, 1.LE.K.LE.12
C           FWORK  -  CHARACTER*K VARIABLE FOR A SCRATCH FILE NAME,
C                     1.LE.K.LE.12
C
C
C        OUTPUT
C           ****   -  FILE WITH ALIAS FNAME WHICH CONTAINS THE DATA
C                     STATEMENT
C           ****      FILE WITH ALIAS FWORK USED AS A SCRATCH FILE
C           IERR   -  ERROR INDICATOR
C                     IERR=0  NORMAL RETURN, NO ERRORS
C                     IERR=1  N IS LESS THAN 1
C                     IERR=2  N IS LARGER THAN 99
C                     IERR=3  DNAME IS BLANK
C                     IERR=4  FNAME IS BLANK
C                     IERR=5  FWORK IS BLANK
C 
C     ERROR CONDITIONS
C        N LESS THAN 1 OR GREATER THAN 99 IS AN ERROR
C        EITHER DNAME OR FNAME BLANK IS AN ERROR
C-----------------------------------------------------------------------
      CHARACTER*(*) DNAME,FNAME,FWORK
      CHARACTER*72 LINE
      CHARACTER*2 NC
      CHARACTER*21 B(99)
      DOUBLE PRECISION A(N)
      IERR=0
      IF (N.GT.99) THEN
        IERR=2
        RETURN
      ENDIF
      IF (N.LT.1) THEN
        IERR=1
        RETURN
      ENDIF
      DO 5 I=1,12
        IF(FNAME(I:I).NE.' ') GO TO 6
    5 CONTINUE
      IERR=4
      RETURN
    6 CONTINUE
      DO 7 I=1,12
        IF(FWORK(I:I).NE.' ') GO TO 8
    7 CONTINUE
      IERR=5
      RETURN
    8 CONTINUE
      DO 15 I=1,10
        IF(DNAME(I:I).NE.' ') GO TO 20
   15 CONTINUE
      IERR=3
      RETURN
   20 CONTINUE
      K1=I
      DO 25 I=K1,10
        IF(DNAME(I:I).EQ.' ') GO TO 30
   25 CONTINUE
      I=11
   30 CONTINUE
      K2=I-1
      OPEN(10,FILE=FNAME,STATUS='UNKNOWN')
      REWIND(10)
C     POSITION FILE AT END OF LAST LINE
      DO 31 LOOP=1,400
        READ(10,'( )',END=32)
 31   CONTINUE
 32   CONTINUE
      ENDFILE(10)
      BACKSPACE(10)
C
      OPEN(20,FILE=FWORK,STATUS='UNKNOWN')
      WRITE(20,800) N
      REWIND(20)
      READ(20,801) NC
      REWIND(20)
      LINE='      DATA ('//DNAME(K1:K2)//'(I), I=1,'//NC//')/'
      WRITE(10,900)LINE
      DO 35 I=1,N
        WRITE(20,901) A(I)
   35 CONTINUE
      REWIND(20)
      DO 40 I=1,N
        READ(20,902) B(I)
   40 CONTINUE
      K=0
      DO 45 I=1,N
        LINE='     &'
        M=6
        DO 50 J=1,3
          K=K+1
          LINE=LINE(1:M)//B(K)(1:21)
          M=M+21
          IF(K.EQ.N) THEN
            LINE=LINE(1:M)//'/'
            GO TO 60
          ELSE
            LINE=LINE(1:M)//','
          ENDIF
          M=M+1
   50   CONTINUE
   60   CONTINUE
        WRITE(10,900)LINE
        IF (K.EQ.N) GO TO 70
   45 CONTINUE
   70 CONTINUE
      CLOSE(10)
C-----------------------------------------------------------------------
C     CLEAN UP THE TEMPORARY FILE
C-----------------------------------------------------------------------
      LINE='  '
      REWIND(20)
      WRITE(20,900) LINE
      CLOSE(20)
  800 FORMAT(I2)
  801 FORMAT(A2)
  900 FORMAT(A72)
  901 FORMAT(D21.14)
  902 FORMAT(A21)
      RETURN
      END
      DOUBLE PRECISION FUNCTION GOFX(X)
      DOUBLE PRECISION X 
      GOFX=1.0D0/X
      END
************************************************************************
      PROGRAM DCHEBYEV
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*72 LINE
      DIMENSION COEF(100)
      OPEN(7,FILE='DCHEBYEV.TXT',STATUS='UNKNOWN')
C-----------------------------------------------------------------------
C     READ A, B AND 18 DIGIT COEFFICIENTS FROM FILE CHEBY.DAT
C-----------------------------------------------------------------------
      OPEN(10,FILE='DPCHEBY.DAT',STATUS='UNKNOWN')
      REWIND(10)
      READ(10,196)NODES
 196  FORMAT(7X,I3)
      READ(10,199) A
 199  FORMAT(3X,D30.18)
      READ(10,199) B
      DO 4 J=1,3      
C       SKIP OVER THE FIRST DATA STATEMENT 
        READ(10,200) LINE
 4    CONTINUE
      DO 5 J=1,400
        READ(10,200) LINE
        PRINT *,LINE
 200    FORMAT(A72)
        IF(LINE(7:10).EQ.'DATA') GOTO 201
 5    CONTINUE
      STOP
 201  CONTINUE
      M=1
      DO 6 J=1,400
        READ(10,202,IOSTAT=K) COEF(M),COEF(M+1)
C        PRINT *,J,K,COEF(M),COEF(M+1)
        IF(K.NE.0) GOTO 7
        M=M+2
 202    FORMAT(6X,D30.18,1X,D30.18)
 6    CONTINUE
 7    CONTINUE
C     K=-1 ONLY IF THE READ PRODUCES TWO ZERO ENTRIES 
      CLOSE(10)
C
      IF(COEF(M).EQ.0.0D0.AND.COEF(M+1).EQ.0.0D0) THEN
        M=M-1
        IF(COEF(M).EQ.0.0D0) THEN
          M=M-1
        ENDIF
      ELSE       
        IF(COEF(M).EQ.0.0D0) THEN
          M=M-1
        ENDIF
      ENDIF
C
 400  CONTINUE
C      PRINT 195, NODES
      PRINT 198, NODES,A,B
      PRINT 194
      K=1
      MF=INT(M/2)
      DO 18 MM=1,MF
        PRINT 203, MM,COEF(K),COEF(K+1)
        K=K+2
 18   CONTINUE
      IF(MOD(M,2).NE.0) THEN
        PRINT 203, MM,COEF(M)
      ENDIF
 9    CONTINUE 
C      PRINT *, ' '
      PRINT *, M,' COEFFICIENTS READ FROM FILE DPCHEBY.DAT'
C      PRINT *, ' '
      PRINT *,'INPUT THE NUMBER OF COEFFICIENTS TO BE USED IN THE SUM (S
     &TOP=0):  '
      READ *,N
      IF (N.GT.M) GOTO 9
      IF (N.EQ.0) STOP
      REWIND(7)
C      WRITE(7,195) NODES
C 195  FORMAT(' NODES= ',I3)
      WRITE(7,198) NODES,A,B
 198  FORMAT('  NODES= ',I3,'  A=',D25.18,' B=',D25.18)
      WRITE(7,194)
 194  FORMAT(7X,'CHEBYSHEV COEFFICIENTS')
      K=1
      MF=INT(M/2)
      DO 8 MM=1,MF
        WRITE(7,203) MM,COEF(K),COEF(K+1)
        K=K+2
 8    CONTINUE
      IF(MOD(M,2).NE.0) THEN
        WRITE(7,203) MM,COEF(M)
      ENDIF
 203  FORMAT(2X,I5,2D30.18)
      WRITE(7,193)N
 193  FORMAT(7X,'NUMBER OF COEFFICIENTS USED IN THE SUM: ',I3)
      WRITE(7,197)
 197  FORMAT(/'               X            Y         REL ERR')
      PRINT 197
      APB=A+B
      BMA=B-A
      AJ=1.0D0
      JL=51
      AJL=DBLE(FLOAT(JL))
      DO 10 J=1,JL
        X=((AJ-1.0D0)*B+(AJL-AJ)*A)/(AJL-1.0D0)
        X2=X*X
        CALL DCHBYS(APB,BMA,X,N,COEF,ANS)
C        X3=X2*X
C        ANS=ANS*DEXP(-X2)/X3
C        Y=GOFX(X)
        Y=1.0D0/X
        ERR=DABS(ANS-Y)/DABS(Y)
        WRITE(7,300) J,X,Y,ERR
 300    FORMAT(4X,I5,3D13.5)
        PRINT 300, J,X,Y,ERR
        IF(MOD(J,20).EQ.0) PAUSE
        AJ=AJ+1.0D0
 10   CONTINUE
      PAUSE
      DO 401 J=1,25
        PRINT *
 401  CONTINUE
      GOTO 400
      END
      SUBROUTINE DCHBYS(APB,BMA,X,N,COEF,SUM)
C-----------------------------------------------------------------------
C     CHBYS SUMS A CHEBYSHEV SERIES WITH COEFFICIENTS COEF(I), I=1,N
C     ON [A,B] AT X. APB=A+B , BMA=B-A
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION COEF(N)
      T=(X+X-APB)/BMA
      TT=T+T
      F1=0.0D0
      F2=0.0D0
      NM=N-1
      DO 160 I=1,NM
        J=N-I+1
        TEMP=F1
        F1=TT*F1-F2+COEF(J)
        F2=TEMP
  160 CONTINUE
      SUM=T*F1-F2+COEF(1)
      RETURN
      END
************************************************************************
      PROGRAM MPCHEBY
C 
C     WRITTEN BY D.E.AMOS, DECEMBER,1972; MODIFIED MAY, 1990.
C 
C     REFERENCE SANDIA NATIONAL LABORATORIES REPORT SC-DR-72 0917
C 
C     ABSTRACT
C
C     PROGRAM MPCHEBY CALCULATES THE CHEBYSHEV COEFFICIENTS FOR A
C     MULTIPLE PRECISION FUNCTION MPGOFX(X) ON AN INTERVAL [A,B].
C
C     FINAL EXPANSION IS  S = 0.5*A(1)+SUM( A(J)*T(J,XX), J=2,N) WHERE
C
C          XX=(2*X-(A+B))/(B-A)   FOR    A.LE.X.LE.B
C                                 OR
C          XX=2*A/X-1.0           FOR    A.LE.X.LT.INFINITY,
C 
C     T(J,XX) ARE CHEBYSHEV POLYNOMIALS AND A(J) ARE THE COEFFICIENTS
C     GENERATED BY THIS PROGRAM AND STORED IN A DATA STATEMENT IN A
C     FILE CALLED MPCHEBY.DAT.
C
C     DESCRIPTION OF ARGUMENTS
C
C     INPUT 
C       A, B    = INTERVAL [A,B] END POINTS. IF A=B, THE EXPANSION
C                 IS TAKEN ON A TO INFINITY USING T(J,2*A/X-1)
C       N       = THE NUMBER OF COEFFICIENTS DESIRED (USUALLY N < 26)
C       NCHEBY  = NUMBER OF CHEBYSHEV NODES FOR THE APPROXIMATION
C                 NCHEBY IS MUCH LARGER THAN N  (USUALLY NCHEBY > 40)
C       IERR    = 1 GIVES A RELATIVE ERROR PRINT
C               = 0 GIVES AN ABSOLUTE ERROR PRINT
C       DNAME   = NAME OF THE COEFFICIENT ARRAY IN THE DATA STATEMENT
C       MPGOFX  = MULTIPLE PRECISION SUBROUTINE FOR THE FUNCTION BEING
C                 APPROXIMATED. SUBROUTINE MPGOFX(MX,MGOFX)
C     OUTPUT
C       FILE MPCHEBY.FIT SHOWING THE COEFFICIENTS AND ERRORS
C       FILE MPCHEBY.DAT CONTAINING THE DATA STATEMENT FOR THE
C                      COEFFICIENTS 0.5*A(1) THRU A(N)
C       FILE MPCHEBY.DAT IS CREATED AND USED AS A SCRATCH FILE
C
C     PROGRAM MPCHEBY USES MPGOFX AND DATAS PLUS ANY OTHERS RELATED TO
C     THE COMPUTATION OF GOFX
C-----------------------------------------------------------------------
      CHARACTER*2 NK,NAB
      CHARACTER*10 ILAB(2)
      CHARACTER*12 FNAME,DNAME,FWORK
C      CHARACTER*25 LINEA
      CHARACTER*80 LINE,LINEA
      CHARACTER*160 LTEMP,LBLANK
      DOUBLE PRECISION T,PI,DN,THETA,VK,VONE,TK,TONE,APB,BMA,X 
      DOUBLE PRECISION Z,TZ,B1,B2,F,AA,BB,AR,TEMP,TEMP1 
C
C-----------------------------------------------------------------------
      DOUBLE PRECISION DTOL,DH
      DOUBLE PRECISION DPMAX
C     SINGLE PRECISION SPMAX

      PARAMETER ( MXNDIG=256 , NBITS=32 ,
     *          LPACK  = (MXNDIG+1)/2 + 1 , LUNPCK = (6*MXNDIG)/5 + 20,
     *          LMWA   = 2*LUNPCK         , LJSUMS = 8*LUNPCK,
     *          LMBUFF = ((LUNPCK+3)*(NBITS-1)*301)/2000 + 6  )
C
C
      DIMENSION MTEMP1(200),MTEMP2(200),MAPB(200),MBMA(200),MAA(200),
     &MBB(200),MVK(200),MVONE(200),MTK(200),MTHETA(200),MZ(200),
     &MTZ(200),MB1(200),MB2(200)
      DIMENSION T(201),F(201),X(201),AR(201)
      DIMENSION MT(200,100),MF(200,100),MX(200,100),MAR(200,100),
     &MTEMP(200)

C
      COMMON /FMUSER/ NDIG,JBASE,JFORM1,JFORM2,KRAD,
     *                KW,NTRACE,LVLTRC,KFLAG,KWARN,KROUND
C
      COMMON /FM/ MWA(LMWA),NCALL,MXEXP,MXEXP2,KARGSW,KEXPUN,KEXPOV,
     *            KUNKNO,IUNKNO,RUNKNO,MXBASE,MXNDG2,KSTACK(19),
     *            MAXINT,SPMAX,DPMAX
C
      COMMON /FMSAVE/ NDIGPI,NJBPI,NDIGE,NJBE,NDIGLB,NJBLB,NDIGLI,NJBLI,
     *                MPISAV(LUNPCK),MESAV(LUNPCK),MLBSAV(LUNPCK),
     *                MLN1(LUNPCK),MLN2(LUNPCK),MLN3(LUNPCK),
     *                MLN4(LUNPCK)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)
C
      DATA (ILAB(I),I=1,2)/' RELATIVE ',' ABSOLUTE '/

C-----------------------------------------------------------------------
      PRINT 950 
  950 FORMAT(' APPROXIMATION INTERVAL A, B= ')
      READ *,AA,BB
      PRINT 951
  951 FORMAT(' NUMBER OF CHEBYSHEV COEFFICIENTS DESIRED (N~25), N= ')
      READ *,NAR
      PRINT 952
  952 FORMAT(' NUMBER OF CHEBYSHEV NODES (NCHEBY~40), NCHEBY= ')
      READ *,NCHEBY
      PRINT 953
  953 FORMAT(' NAME OF THE COEFFICIENTS IN THE DATA STATEMENT, DNAME= ')
      READ 900,DNAME
  900 FORMAT(A8)
      PRINT 954
  954 FORMAT(' 1 FOR RELATIVE ERROR; 2 FOR ABSOLUTE ERROR, IERR= ')
      READ *,IERR
      IF (IERR.NE.1 .AND. IERR.NE.2) THEN
        IERR=2
      ENDIF
C-----------------------------------------------------------------------
C     END INPUT 
C-----------------------------------------------------------------------
      INIT=0
      IF (INIT.EQ.0) THEN
        NPREC=65
        CALL FPSET(NPREC)
C       FOR JBASE=10000, NDIG=NPREC/4+2
        NDIG=INT(NPREC/4)+5
        NCDIG=NDIG
        K=0
        CALL FPI2M(K,MZERO)
        K=1
        CALL FPI2M(K,MONE)
        K=2        
        CALL FPI2M(K,MTWO)
        K=9
        CALL FPI2M(K,MNINE)
        CALL FPPI(MPI)
        CALL FPM2DP(MPI,PI)
        CALL FPSQRT(MPI,MRTPI)
C       MRTPI=SQRT(PI)
        CALL FPDIV(MONE,MRTPI,MTEMP)
        CALL FPADD(MTEMP,MTEMP,MTRTPI)
C       MTRTPI=2/RTPI
        DTOL=0.50D-19
        CALL FPDP2M(DTOL,MTOL)
        I=2
        CALL FPDIVI(MONE,I,MDH)
C       INIT=1
C       INIT IS SET TO 1 AFTER THE INITIALIZATION OF MQUAD8
      ENDIF
      N1 = NCHEBY-1
      N2 = NCHEBY-1
      APB=AA+BB 
      BMA=BB-AA
      CALL FPDP2M(AA,MAA)
      CALL FPDP2M(BB,MBB)
      CALL FPADD(MAA,MBB,MAPB)
      CALL FPSUB(MBB,MAA,MBMA)
      C1= SNGL(APB)
      C2= SNGL(BMA)
      IF(BB.EQ.AA) C1=SNGL(AA+AA) 
      OPEN(10,FILE='MPCHEBY.FIT',STATUS='UNKNOWN')
      DO 500 N=N1,N2 
        IF (NAR.GT.N) THEN
          NAR=N+1
        ENDIF
        NP1=N+1
        DN=DBLE(FLOAT(N))
        THETA=PI/DN
        CALL FPDIVI(MPI,N,MTHETA)
        T(1)=1.0D0
        CALL COPYTO(MONE,1,MT)  
        T(2)=DCOS(THETA)
        CALL FPCOS(MTHETA,MTEMP)
        CALL COPYTO(MTEMP,2,MT)
        CALL FPEQU(MTEMP,MTONE,NDIG,NDIG)
        TONE=T(2)
        VONE=DSIN(THETA)
        CALL FPSIN(MTHETA,MVONE)
        VK = VONE
        CALL FPEQU(MVONE,MVK,NDIG,NDIG)
C-----------------------------------------------------------------------
C     GET CHEBYSHEV NODES ON [A,B] IN X ARRAY
C     TRIG IDENTITIES SIN((K+1)*X) = SIN(K*X)*COS(X)+COS(K*X)*SIN(X)
C                     COS((K+1)*X) = COS(K*X)*COS(X)-SIN(K*X)*SIN(X)
C     FOR CHEBYSHEV NODES T(K)=COS((K-1)*PI/N), K=1,N+1 BY RECURRENCE
C-----------------------------------------------------------------------
        DO 5 K=2,N
          TK=T(K)
          CALL COPYFR(MT,K,MTK)          
          T(K+1)=TONE*TK-VK*VONE
          CALL FPMPY(MTONE,MTK,MTEMP)
          CALL FPMPY(MVK,MVONE,MTEMP1)
          CALL FPSUB(MTEMP,MTEMP1,MTEMP)
          CALL COPYTO(MTEMP,K+1,MT)
          VK=VK*TONE+TK*VONE
          CALL FPMPY(MVK,MTONE,MTEMP)
          CALL FPMPY(MTK,MVONE,MTEMP1)
          CALL FPADD(MTEMP,MTEMP1,MVK)
          X(K)=(TK*BMA+APB)/2.0D0
          CALL FPMPY(MTK,MBMA,MTEMP)
          CALL FPADD(MTEMP,MAPB,MTEMP)
          CALL FPDIVI(MTEMP,2,MTEMP)
          CALL COPYTO(MTEMP,K,MX)
          CALL FPM2DP(MTEMP,TEMP)
          PRINT *,'K, X(K) ',K,TEMP
C          IF(AA.NE.BB) GO TO 5
C          IF(TK.NE.-1.0D0) GO TO 10
C          X(K)=1.0D+16
C          GO TO 5
C   10     CONTINUE
C          X(K)=(AA+AA)/(1.0D0+TK)
    5   CONTINUE
        X(1)=(T(1)*BMA+APB)/2.0D0
        CALL COPYFR(MT,1,MTEMP)
        CALL FPMPY(MTEMP,MBMA,MTEMP)
        CALL FPADD(MTEMP,MAPB,MTEMP)
        CALL FPDIVI(MTEMP,2,MTEMP)
        CALL COPYTO(MTEMP,1,MX)
        X(NP1)=(T(NP1)*BMA+APB)/2.0D0
        CALL COPYFR(MT,NP1,MTEMP)
        CALL FPMPY(MTEMP,MBMA,MTEMP)
        CALL FPADD(MTEMP,MAPB,MTEMP)
        CALL FPDIVI(MTEMP,2,MTEMP)
        CALL COPYTO(MTEMP,NP1,MX)
C        IF(AA.NE.BB) GO TO 20
C        X(1)=(AA+AA)/(1.0D0+T(1))
C        X(NP1)=1.0D+16
C   20   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 52)NP1,AA,BB
   52 FORMAT(' NODES='I5/' A    ='D23.14/' B    ='D23.14/)
        WRITE(10, 51)(X(I),I=1,NP1)
   51 FORMAT(' CHEBYSHEV NODES  X(J)'/(1X,3D23.14))
C-----------------------------------------------------------------------
C     FUNCTION EVALUATION AT CHEBYSHEV NODES X(I), I=1,NP1
C-----------------------------------------------------------------------
        DO 7 I=1,NP1
          CALL COPYFR(MX,I,MTEMP)
          CALL FPM2DP(MTEMP,TEMP1)
          CALL MPGOFX(MTEMP,MTEMP1)
          CALL COPYTO(MTEMP1,I,MF)
          CALL FPM2DP(MTEMP1,TEMP)
          F(I)=TEMP
          PRINT *,'I,X(I),F(I)',I,TEMP1,TEMP
    7   CONTINUE
        F(NP1)=TEMP/2.0D0
        CALL FPDIVI(MTEMP1,2,MTEMP)
        CALL COPYTO(MTEMP,NP1,MF)
C-----------------------------------------------------------------------
C     END OF FUNCTION EVALUATION
C-----------------------------------------------------------------------
        DO 100 IR=1,NP1
C          Z=T(IR)
          CALL COPYFR(MT,IR,MZ)
C          TZ=Z+Z
          CALL FPMPYI(MZ,2,MTZ)
C          B2=0.0D0
          CALL FPEQU(MZERO,MB2,NDIG,NDIG)
C          B1= 0.0D0
          CALL FPEQU(MZERO,MB1,NDIG,NDIG)
          DO 15 J=1,N
            K=NP1-J+1
            TEMP=B1
            CALL FPEQU(MB1,MTEMP,NDIG,NDIG)
C            B1=TZ*B1-B2+F(K)
            CALL COPYFR(MF,K,MTEMP1)
            CALL FPMPY(MTZ,MB1,MTEMP2)
            CALL FPSUB(MTEMP2,MB2,MTEMP2)
            CALL FPADD(MTEMP2,MTEMP1,MB1)
C            B2=TEMP
            CALL FPEQU(MTEMP,MB2,NDIG,NDIG)
   15     CONTINUE
C          ANS=Z*B1-B2+F(1)/2.0D0
          CALL COPYFR(MF,1,MTEMP)
          CALL FPDIVI(MTEMP,2,MTEMP)
          CALL FPMPY(MZ,MB1,MTEMP1)
          CALL FPSUB(MTEMP1,MB2,MTEMP1)
          CALL FPADD(MTEMP1,MTEMP,MTEMP)
C          AR(IR)=(ANS+ANS)/DN
          CALL FPMPYI(MTEMP,2,MTEMP)
          CALL FPDIVI(MTEMP,N,MTEMP)
          CALL COPYTO(MTEMP,IR,MAR)
          CALL FPM2DP(MTEMP,AR(IR))
  100   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10,153) (AR(K),K=1,NP1)
  153 FORMAT(' CHEBYSHEV COEFFICIENTS A(J)'/(1X,3D23.14))
C-----------------------------------------------------------------------
        F(NP1)=F(NP1)+F(NP1)
        CALL COPYFR(MF,NP1,MTEMP)
        CALL FPMPYI(MTEMP,2,MTEMP)
        CALL COPYTO(MTEMP,NP1,MF)
C-----------------------------------------------------------------------
        WRITE(10, 54)(F(I),I=1,NP1)
   54 FORMAT(' FUNCTION VALUES AT CHEBYSHEV NODES F(X(J))'/(1X,3D23.14))
C-----------------------------------------------------------------------
        NARM=NAR-1
        DO 30 I=1,NP1
          XX=SNGL(X(I))
          IF(AA.NE.BB) GO TO 26
          SZ=C1/XX-1.0E0
          GO TO 27
   26     CONTINUE
          SZ=(XX+XX-C1)/C2
   27     CONTINUE
          STZ=SZ+SZ
          S2=0.0E0
          S1=0.0E0
          DO 25 J=1,NARM
            K=NAR-J+1
            ST=S1
            SA=AR(K)
            S1=STZ*S1-S2+SA
            S2=ST
   25     CONTINUE
          SA=AR(1)
          FX=SZ*S1-S2+SA/2.0
          T(I)=DBLE(FX)-F(I)
          IF(IERR.EQ.1) THEN
            IF(F(I).NE.0.0D0) THEN
              T(I)=T(I)/F(I)
            ENDIF
          ENDIF
   30   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 53)ILAB(IERR),NAR,AA,BB,(T(I),I=1,NP1)
   53   FORMAT(/' SINGLE PRECISION',A10,'ERRORS AT CHEBYSHEV NODES USING
     &',I5,' TERMS'/' OF THE CHEBYSHEV SUM ON',2D23.14//(1X,3D23.14))
C-----------------------------------------------------------------------
        DO 35 I=1,NP1
          IF(AA.NE.BB) GO TO 36
          Z=(AA+AA)/X(I)-1.0D0
          GO TO 37
   36     CONTINUE
          Z=(X(I)+X(I)-APB)/BMA
   37     CONTINUE
          TZ=Z+Z
          B2=0.0D0
          B1=0.0D0
          DO 40 J=1,NARM
            K=NAR-J+1
            TEMP=B1
            CALL COPYFR(MAR,K,MTEMP)
            CALL FPM2DP(MTEMP,TEMP1)
C            B1=TZ*B1-B2+AR(K)
            B1=TZ*B1-B2+TEMP1
            B2=TEMP
   40     CONTINUE
          CALL COPYFR(MAR,1,MTEMP)
          CALL FPM2DP(MTEMP,TEMP)
C          TEMP=Z*B1-B2+AR(1)/2.0D0
          TEMP=Z*B1-B2+TEMP/2.0D0
          CALL COPYFR(MF,I,MTEMP)
          CALL FPM2DP(MTEMP,TEMP1)
C          T(I)=TEMP-F(I)
          T(I)=TEMP-TEMP1
          IF(IERR.EQ.1) THEN
            IF(F(I).NE.0.0D0) THEN
C              T(I)=T(I)/F(I)
               T(I)=T(I)/TEMP1
            ENDIF
          ENDIF
   35   CONTINUE
C-----------------------------------------------------------------------
        WRITE(10, 55)ILAB(IERR),NAR,AA,BB,(T(I),I=1,NP1)
   55   FORMAT(/' DOUBLE PRECISION',A10,'ERRORS AT CHEBYSHEV NODES USING
     &',I5,' TERMS'/' OF THE CHEBYSHEV SUM ON',2D23.14//(1X,3D23.14))
C-----------------------------------------------------------------------
  500 CONTINUE
      CLOSE(10)
      FNAME='MPCHEBY.DAT'
      FWORK='MPCHEBZ.DAT'
      AR(1)=AR(1)*0.5D0
      CALL COPYFR(MAR,1,MTEMP)
      CALL FPDIVI(MTEMP,2,MTEMP)
      CALL COPYTO(MTEMP,1,MAR)
      OPEN(10,FILE='MPCHEBY.DAT',STATUS='UNKNOWN')
      REWIND(10)
      WRITE(10,531)NP1
 531  FORMAT('NODES= ',I3)
      WRITE(10,530) AA,BB
 530  FORMAT('A= 'D30.18/'B= ',D30.18/)
      CLOSE(10)
C-----------------------------------------------------------------------
C     WRITE NORMAL DATA STATEMENT WITH 14 DIGITS, 3 ON A LINE
C-----------------------------------------------------------------------
      CALL DATAS(NAR,AR,DNAME,FNAME,FWORK,KERR)
C-----------------------------------------------------------------------
C     WRITE DATA STATEMENT IN 2 COLUMNS OF 18 DIGITS
C-----------------------------------------------------------------------
      DO 515 I=1,10
        IF(DNAME(I:I).NE.' ') GO TO 520
  515 CONTINUE
      PRINT *, 'THE DATA NAME IS BLANK'
      STOP
  520 CONTINUE
      K1=I
      DO 501 I=K1,10
        IF(DNAME(I:I).EQ.' ') GO TO 502
 501  CONTINUE
      I=11
 502  CONTINUE
      K2=I-1
C     READ TO THE BOTTOM OF THE FILE (TO SKIP WHAT IS ALREADY THERE)
      OPEN(15,FILE=FNAME,STATUS='UNKNOWN')
      REWIND(15)
      DO 511 LOOP=1,400
        READ(15,'( )',END=512)
 511  CONTINUE
 512  CONTINUE
      ENDFILE(15)
      BACKSPACE(15)
      OPEN(20,FILE=FWORK,STATUS='UNKNOWN')
      WRITE(20,509) NAR
 509  FORMAT(I2)
      REWIND(20)
      READ(20,510) NAB
 510  FORMAT(A2)
      REWIND (20)
      LINE='      DATA ('//DNAME(K1:K2)//'(I), I=1,'//NAB//')/'
      WRITE(15,506)LINE
 506  FORMAT(A72)
      LBLANK='                                                         '
      LL=72
      JFORM1=0
      K=1
      NMOD=MOD(NAR,2)
      IF(NMOD.EQ.0) THEN
        NJ=NAR-1
      ELSE
        NJ=NAR-2
      ENDIF
      DO 300 J=1,NJ,2
        WRITE(20,509) K
        REWIND(20)
        READ(20,510) NK
        REWIND(20)
        CALL COPYFR(MAR,J,MTEMP)
        LTEMP='    '//NK//'    ' 
        CALL FPOUT(MTEMP,LINEA,LL)
        I1=NPREC+4
        I2=I1+3
        LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//',  '
        LTEMP=LTEMP(1:8)//LINEA(1:30)
        CALL COPYFR(MAR,J+1,MTEMP)
        CALL FPOUT(MTEMP,LINEA,LL)
        IF (J.EQ.NJ) THEN
          IF(NMOD.EQ.0) THEN
            LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//'/'
          ELSE
            LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//','
          ENDIF
        ELSE
          LINEA=LINEA(1:22)//'D'//LINEA(I1:I2)//','
        ENDIF
        LTEMP=LTEMP(1:38)//LINEA(1:28)//LBLANK
        LINE=LTEMP(1:72)
        WRITE(15,301)LINE
 301    FORMAT(A72)
        K=K+1
 300  CONTINUE
      IF(NMOD.NE.0) THEN
        WRITE(20,509) K
        REWIND(20)
        READ(20,510) NK
        REWIND(20)
        CALL COPYFR(MAR,NAR,MTEMP)
        LTEMP='    '//NK//'    ' 
        CALL FPOUT(MTEMP,LINEA,LL)
        LINEA=LINEA(1:22)//'D'//LINEA(64:67)//'/  '
        LTEMP=LTEMP(1:8)//LINEA(1:28)//LBLANK
        LINE=LTEMP(1:72)
        WRITE(15,301)LINE
      ENDIF
      CLOSE(15)
C-----------------------------------------------------------------------
      IF (KERR.NE.0) THEN
        PRINT *,' ERROR IN DATAS, IERR = ',KERR
      ELSE
        PRINT *,'  '
        PRINT *,' APPROXIMATION USING ',NP1,' CHEBYSHEV NODES'
        PRINT *,'  '
        PRINT *,' FILES WRITTEN:'
        PRINT *,'  '
        PRINT *,' MPCHEBY.FIT:    THE COEFFIENTS, NODES, AND ERRORS,'
        PRINT *,' MPCHEBY.DAT:    THE DATA STATEMENT,'
        PRINT *,' MPCHEBZ.DAT:    SCRATCH FILE.'
      ENDIF
      PAUSE
      STOP 
      END
      SUBROUTINE COPYTO(MA,K,MB)
C     COPY VECTOR MA to KTH COLUMN OF MATRIX MB 
      DIMENSION MB(200,100),MA(200)
      DO 10 J=1,200
        MB(J,K)=MA(J)
 10   CONTINUE
      RETURN
      END
      SUBROUTINE COPYFR(MB,K,MA)
C     COPY from KTH COLUMN OF MATRIX MB TO VECTOR MA 
      DIMENSION MB(200,100),MA(200)
      DO 10 J=1,200
        MA(J)=MB(J,K)
 10   CONTINUE
      RETURN
      END
      SUBROUTINE MPGOFXX(MX,MGOFX)
      DIMENSION MX(200),MGOFX(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      CALL FPDIV(MONE,MX,MGOFX)
      RETURN
      END
      SUBROUTINE MPGOFXY(MT,MFUN)
C     COMPUTES FOR ERF(X)
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
C     CALCULATE ERF ON [0,2]
C
      NDIG=NCDIG
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
C        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          IF (FPCOMP(MTRMA,'LE',MTOL)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
        CALL FPMPY(MRTPI,MT,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)        
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
      RETURN
      END
      SUBROUTINE MPGOFX(MX,MGOFX)
C
C     COMPUTES INT ON (X,INF) OF ERFC(X)/X IN MP ARITHMETIC USING
C     GAUS8 FORMULA ON REPEATED SUBDIVIDED INTERVALS
C
C      DOUBLE PRECISION DSUM
      LOGICAL FPCOMP
      DIMENSION MSUM(200),MSUMA(200),MANS(200),MANSA(200),MTOLA(200),
     &MTEMP(200),MA(200),MB(200),MX(200),MSUM1(200),MDX(200)
C
      DIMENSION MGOFX(200)
C
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      KL=10
      CALL FPMPYI(MTOL,KL,MTOLA)
      CALL FPEQU(MDH,MDX,NDIG,NDIG)
      CALL FPEQU(MZERO,MSUM1,NDIG,NDIG)
      KL=100
      DO 10 J=1,100
        CALL FPDIVI(MDX,2,MDX)
        KL=KL+KL
        CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
        CALL FPEQU(MX,MB,NDIG,NDIG)
        DO 20 K=1,KL
          CALL FPEQU(MB,MA,NDIG,NDIG)
          CALL FPADD(MA,MDX,MB)
          CALL MQUAD8(MA,MB,MANS)
          CALL FPADD(MSUM,MANS,MSUM)
          IF(J.EQ.1) THEN
            CALL FPABS(MANS,MANSA)
            CALL FPABS(MSUM,MSUMA)
            CALL FPMPY(MSUMA,MTOL,MSUMA)
            IF (FPCOMP(MANSA,'LT',MSUMA)) GOTO 25
          ENDIF          
 20     CONTINUE
 25     CONTINUE
        IF(J.EQ.1) THEN
          KL=K
          CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
          GOTO 10
        ENDIF
        CALL FPSUB(MSUM,MSUM1,MTEMP)
        CALL FPABS(MTEMP,MTEMP)
        CALL FPABS(MSUM,MSUMA)
        CALL FPMPY(MSUMA,MTOLA,MSUMA)
        IF (FPCOMP(MTEMP,'LE',MSUMA)) GOTO 30
        CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
C        CALL FPM2DP(MSUM,DSUM)
C        PRINT *,KL,DSUM
 10   CONTINUE
 30   CONTINUE
      CALL FPMPY(MX,MX,MTEMP)
      CALL FPEXP(MTEMP,MB)
C      CALL FPMPY(MTEMP,MX,MA)
C      CALL FPMPY(MA,MB,MTEMP)
      CALL FPMPY(MTEMP,MB,MTEMP)
      CALL FPMPY(MTEMP,MSUM,MGOFX)
      RETURN
      END
      SUBROUTINE MQUAD8(MA,MB,MQUAD)
      DIMENSION MA(200),MB(200),MAPB(200),MBMA(200),MTEMP1(200),
     &MTEMP2(200),MTEMP3(200),MTEMP4(200),MTEMP5(200),MTEMP6(200),
     &MTEMP7(200),MTEMP8(200),MSUM(200),MFUN(200),MQUAD(200)
      DIMENSION IX1(4),IX2(4),IX3(4),IX4(4),IW1(4),IW2(4),IW3(4),IW4(4)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)
C
C     GAUSS ABSCISSAS AND WEIGHTS IN MP
      DATA (IX1(J),J=1,4)/0,18343464,24956498,05000000/
      DATA (IX2(J),J=1,4)/0,52553240,99163289,86000000/
      DATA (IX3(J),J=1,4)/0,79666647,74136267,40000000/
      DATA (IX4(J),J=1,4)/0,96028985,64975362,32000000/
      DATA (IW1(J),J=1,4)/0,36268378,33783619,83000000/
      DATA (IW2(J),J=1,4)/0,31370664,58778872,87000000/
      DATA (IW3(J),J=1,4)/0,22238103,44533744,71000000/
      DATA (IW4(J),J=1,4)/0,10122853,62903762,59000000/
C
      NDIG=NCDIG
      IF (INIT.EQ.0) THEN
        CALL FPEQU(MZERO,MX1,NDIG,NDIG)
        MX1(1)=IX1(1)
        MX1(2)=IX1(2)
        MX1(3)=IX1(3)
        MX1(4)=IX1(4)
        CALL FPEQU(MZERO,MX2,NDIG,NDIG)
        MX2(1)=IX2(1)
        MX2(2)=IX2(2)
        MX2(3)=IX2(3)
        MX2(4)=IX2(4)
        CALL FPEQU(MZERO,MX3,NDIG,NDIG)
        MX3(1)=IX3(1)
        MX3(2)=IX3(2)
        MX3(3)=IX3(3)
        MX3(4)=IX3(4)
        CALL FPEQU(MZERO,MX4,NDIG,NDIG)
        MX4(1)=IX4(1)
        MX4(2)=IX4(2)
        MX4(3)=IX4(3)
        MX4(4)=IX4(4)
        CALL FPEQU(MZERO,MW1,NDIG,NDIG)
        MW1(1)=IW1(1)
        MW1(2)=IW1(2)
        MW1(3)=IW1(3)
        MW1(4)=IW1(4)
        CALL FPEQU(MZERO,MW2,NDIG,NDIG)
        MW2(1)=IW2(1)
        MW2(2)=IW2(2)
        MW2(3)=IW2(3)
        MW2(4)=IW2(4)
        CALL FPEQU(MZERO,MW3,NDIG,NDIG)
        MW3(1)=IW3(1)
        MW3(2)=IW3(2)
        MW3(3)=IW3(3)
        MW3(4)=IW3(4)
        CALL FPEQU(MZERO,MW4,NDIG,NDIG)
        MW4(1)=IW4(1)
        MW4(2)=IW4(2)
        MW4(3)=IW4(3)
        MW4(4)=IW4(4)
        PRINT *,'GAUSSIAN ABSCISSAS AND WEIGHTS '
        CALL FPPRNT(MX1)
        CALL FPPRNT(MX2)
        CALL FPPRNT(MX3)
        CALL FPPRNT(MX4)
        CALL FPPRNT(MW1)
        CALL FPPRNT(MW2)
        CALL FPPRNT(MW3)
        CALL FPPRNT(MW4)
        PAUSE
        INIT=1
      ENDIF
      CALL FPADD(MA,MB,MAPB)
      CALL FPDIVI(MAPB,2,MAPB)
      CALL FPSUB(MB,MA,MBMA)
      CALL FPDIVI(MBMA,2,MBMA)
      CALL FPMPY(MX1,MBMA,MTEMP1)
      CALL FPMPY(MX2,MBMA,MTEMP2)
      CALL FPMPY(MX3,MBMA,MTEMP3)
      CALL FPMPY(MX4,MBMA,MTEMP4)
      I=-1
      CALL FPMPYI(MTEMP1,I,MTEMP5)
      CALL FPMPYI(MTEMP2,I,MTEMP6)
      CALL FPMPYI(MTEMP3,I,MTEMP7)
      CALL FPMPYI(MTEMP4,I,MTEMP8)
C
      CALL FPADD(MTEMP1,MAPB,MTEMP1)
      CALL FPADD(MTEMP2,MAPB,MTEMP2)
      CALL FPADD(MTEMP3,MAPB,MTEMP3)
      CALL FPADD(MTEMP4,MAPB,MTEMP4)
      CALL FPADD(MTEMP5,MAPB,MTEMP5)
      CALL FPADD(MTEMP6,MAPB,MTEMP6)
      CALL FPADD(MTEMP7,MAPB,MTEMP7)
      CALL FPADD(MTEMP8,MAPB,MTEMP8)
C
      CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
      CALL MFOFX(MTEMP1,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP2,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP3,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP4,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP5,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP6,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP7,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP8,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL FPMPY(MSUM,MBMA,MQUAD)
      RETURN
      END
      SUBROUTINE MFOFX(MT,MFUN)
C      DOUBLE PRECISION DSUM,DT
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
C        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MTEMP)
          CALL FPMPY(MTEMP,MT,MTEMP)
C          CALL FPSUB(MTRM1,MTEMP,MSUMT)
          CALL FPSUB(MONE,MTEMP,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          CALL FPABS(MSUM,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LE',MSUMA)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
C        CALL FPMPY(MRTPI,MT2,MAK)
        CALL FPMPY(MRTPI,MT,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
C      CALL FPM2DP(MSUMT,DSUM)
C      CALL FPM2DP(MT,DT)
C      PRINT *,I,DT,DSUM
C      PAUSE
      RETURN
      END
      SUBROUTINE DATAS(N,A,DNAME,FNAME,FWORK,IERR)
C 
C     WRITTEN BY D.E.AMOS, MAY, 1990.
C 
C     REFERENCE
C 
C     ABSTRACT
C        DATAS FORMATS A DOUBLE PRECISION SINGLY DIMENSIONED ARRAY
C        A(I), I=1,N INTO A DATA STATEMENT IN FILE FNAME. THE A
C        VECTOR IS FORMATTED BY 3E21.14 ACROSS A LINE AND GIVEN THE
C        NAME SPECIFIED BY THE PARAMETER DNAME. DATAS ALSO CREATES
C        A TEMPORARY FILE TEMPX.DAT.
C
C     DESCRIPTION OF ARGUMENTS
C 
C        INPUT
C           N      -  NUMBER OF ELEMENTS OF A, 1.LE.N.LE.99
C           A      -  DOUBLE PRECISION ARRAY OF DIMENSION AT LEAST N
C           DNAME  -  CHARACTER*K VARIABLE FOR THE NAME OF THE DATA
C                     IN THE FILE DESIGNATED BY FNAME, 1.LE.K.LE.10
C           FNAME  -  CHARACTER*K VARIABLE FOR THE FILE NAME WHERE THE
C                     DATA STATEMENT IS TO BE WRITTEN, 1.LE.K.LE.12
C           FWORK  -  CHARACTER*K VARIABLE FOR A SCRATCH FILE NAME,
C                     1.LE.K.LE.12
C
C
C        OUTPUT
C           ****   -  FILE WITH ALIAS FNAME WHICH CONTAINS THE DATA
C                     STATEMENT
C           ****      FILE WITH ALIAS FWORK USED AS A SCRATCH FILE
C           IERR   -  ERROR INDICATOR
C                     IERR=0  NORMAL RETURN, NO ERRORS
C                     IERR=1  N IS LESS THAN 1
C                     IERR=2  N IS LARGER THAN 99
C                     IERR=3  DNAME IS BLANK
C                     IERR=4  FNAME IS BLANK
C                     IERR=5  FWORK IS BLANK
C 
C     ERROR CONDITIONS
C        N LESS THAN 1 OR GREATER THAN 99 IS AN ERROR
C        EITHER DNAME OR FNAME BLANK IS AN ERROR
C-----------------------------------------------------------------------
      CHARACTER*(*) DNAME,FNAME,FWORK
      CHARACTER*72 LINE
      CHARACTER*2 NC
      CHARACTER*21 B(99)
      DOUBLE PRECISION A(N)
      IERR=0
      IF (N.GT.99) THEN
        IERR=2
        RETURN
      ENDIF
      IF (N.LT.1) THEN
        IERR=1
        RETURN
      ENDIF
      DO 5 I=1,12
        IF(FNAME(I:I).NE.' ') GO TO 6
    5 CONTINUE
      IERR=4
      RETURN
    6 CONTINUE
      DO 7 I=1,12
        IF(FWORK(I:I).NE.' ') GO TO 8
    7 CONTINUE
      IERR=5
      RETURN
    8 CONTINUE
      DO 15 I=1,10
        IF(DNAME(I:I).NE.' ') GO TO 20
   15 CONTINUE
      IERR=3
      RETURN
   20 CONTINUE
      K1=I
      DO 25 I=K1,10
        IF(DNAME(I:I).EQ.' ') GO TO 30
   25 CONTINUE
      I=11
   30 CONTINUE
      K2=I-1
      OPEN(10,FILE=FNAME,STATUS='UNKNOWN')
      REWIND(10)
C     POSITION FILE AT END OF LAST LINE
      DO 31 LOOP=1,400
        READ(10,'( )',END=32)
 31   CONTINUE
 32   CONTINUE
      ENDFILE(10)
      BACKSPACE(10)
C
      OPEN(20,FILE=FWORK,STATUS='UNKNOWN')
      WRITE(20,800) N
      REWIND(20)
      READ(20,801) NC
      REWIND(20)
      LINE='      DATA ('//DNAME(K1:K2)//'(I), I=1,'//NC//')/'
      WRITE(10,900)LINE
      DO 35 I=1,N
        WRITE(20,901) A(I)
   35 CONTINUE
      REWIND(20)
      DO 40 I=1,N
        READ(20,902) B(I)
   40 CONTINUE
      K=0
      DO 45 I=1,N
        LINE='     &'
        M=6
        DO 50 J=1,3
          K=K+1
          LINE=LINE(1:M)//B(K)(1:21)
          M=M+21
          IF(K.EQ.N) THEN
            LINE=LINE(1:M)//'/'
            GO TO 60
          ELSE
            LINE=LINE(1:M)//','
          ENDIF
          M=M+1
   50   CONTINUE
   60   CONTINUE
        WRITE(10,900)LINE
        IF (K.EQ.N) GO TO 70
   45 CONTINUE
   70 CONTINUE
      CLOSE(10)
C-----------------------------------------------------------------------
C     CLEAN UP THE TEMPORARY FILE
C-----------------------------------------------------------------------
      LINE='  '
      REWIND(20)
      WRITE(20,900) LINE
      CLOSE(20)
  800 FORMAT(I2)
  801 FORMAT(A2)
  900 FORMAT(A72)
  901 FORMAT(D21.14)
  902 FORMAT(A21)
      RETURN
      END
************************************************************************
      PROGRAM MCHEBYEV
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*72 LINE
      COMMON /CINIT/INIT
      DIMENSION COEF(100)
      OPEN(7,FILE='MCHEBYEV.TXT',STATUS='UNKNOWN')
C-----------------------------------------------------------------------
C     READ A, B AND 18 DIGIT COEFFICIENTS FROM FILE CHEBY.DAT
C-----------------------------------------------------------------------
      OPEN(10,FILE='MPCHEBY.DAT',STATUS='UNKNOWN')
      REWIND(10)
      READ(10,196)NODES
 196  FORMAT(7X,I3)
      READ(10,199) A
 199  FORMAT(3X,D30.18)
      READ(10,199) B
      DO 4 J=1,3      
C       SKIP OVER THE FIRST DATA STATEMENT 
        READ(10,200) LINE
 4    CONTINUE
      DO 5 J=1,400
        READ(10,200) LINE
C        PRINT *,LINE
 200    FORMAT(A72)
        IF(LINE(7:10).EQ.'DATA') GOTO 201
 5    CONTINUE
      STOP
 201  CONTINUE
      M=1
      DO 6 J=1,400
        READ(10,202,IOSTAT=K) COEF(M),COEF(M+1)
C        PRINT *,J,K,COEF(M),COEF(M+1)
        IF(K.NE.0) GOTO 7
        M=M+2
 202    FORMAT(8X,D26.20,4X,D26.20)
 6    CONTINUE
 7    CONTINUE
C     K=-1 ONLY IF THE READ PRODUCES TWO ZERO ENTRIES 
      CLOSE(10)
C
      IF(COEF(M).EQ.0.0D0.AND.COEF(M+1).EQ.0.0D0) THEN
        M=M-1
        IF(COEF(M).EQ.0.0D0) THEN
          M=M-1
        ENDIF
      ELSE       
        IF(COEF(M).EQ.0.0D0) THEN
          M=M-1
        ENDIF
      ENDIF
C
      INIT=0
 400  CONTINUE
C      PRINT 195, NODES
      PRINT 198, NODES,A,B
      PRINT 194
      K=1
      MF=INT(M/2)
      DO 18 MM=1,MF
        PRINT 203, MM,COEF(K),COEF(K+1)
        K=K+2
 18   CONTINUE
      IF(MOD(M,2).NE.0) THEN
        PRINT 203, MM,COEF(M)
      ENDIF
 9    CONTINUE 
C      PRINT *, ' '
      PRINT *, M,' COEFFICIENTS READ FROM FILE MPCHEBY.DAT'
C      PRINT *, ' '
      PRINT *,'INPUT THE NUMBER OF COEFFICIENTS TO BE USED IN THE SUM (S
     &TOP=0):  '
      READ *,N
      IF (N.GT.M) GOTO 9
      IF (N.EQ.0) STOP
      REWIND(7)
C      WRITE(7,195) NODES
C 195  FORMAT(' NODES= ',I3)
      WRITE(7,198) NODES,A,B
 198  FORMAT('  NODES= ',I3,'  A=',D25.18,' B=',D25.18)
      WRITE(7,194)
 194  FORMAT(7X,'CHEBYSHEV COEFFICIENTS')
      K=1
      MF=INT(M/2)
      DO 8 MM=1,MF
        WRITE(7,203) MM,COEF(K),COEF(K+1)
        K=K+2
 8    CONTINUE
      IF(MOD(M,2).NE.0) THEN
        WRITE(7,203) MM,COEF(M)
      ENDIF
 203  FORMAT(2X,I5,2D30.18)
      WRITE(7,193)N
 193  FORMAT(7X,'NUMBER OF COEFFICIENTS USED IN THE SUM: ',I3)
      WRITE(7,197)
 197  FORMAT(/'               X            Y         REL ERR')
      PRINT 197
      APB=A+B
      BMA=B-A
      AJ=1.0D0
      JL=51
      AJL=DBLE(FLOAT(JL))
      DO 10 J=1,JL
        X=((AJ-1.0D0)*B+(AJL-AJ)*A)/(AJL-1.0D0)
C        X2=X*X
        CALL DCHBYS(APB,BMA,X,N,COEF,ANS)
C        X3=X2*X
C        ANS=ANS*DEXP(-X2)/X3
C         Y=MPGOFX(X)
        Y=DPGOFX(X)
C        Y=1.0D0/X
        ERR=DABS(ANS-Y)/DABS(Y)
        WRITE(7,300) J,X,Y,ERR
 300    FORMAT(4X,I5,3D13.5)
        PRINT 300, J,X,Y,ERR
        IF(MOD(J,20).EQ.0) PAUSE
        AJ=AJ+1.0D0
 10   CONTINUE
      PAUSE
      DO 401 J=1,25
        PRINT *
 401  CONTINUE
      GOTO 400
      END
      SUBROUTINE DCHBYS(APB,BMA,X,N,COEF,SUM)
C-----------------------------------------------------------------------
C     CHBYS SUMS A CHEBYSHEV SERIES WITH COEFFICIENTS COEF(I), I=1,N
C     ON [A,B] AT X. APB=A+B , BMA=B-A
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION COEF(N)
      T=(X+X-APB)/BMA
      TT=T+T
      F1=0.0D0
      F2=0.0D0
      NM=N-1
      DO 160 I=1,NM
        J=N-I+1
        TEMP=F1
        F1=TT*F1-F2+COEF(J)
        F2=TEMP
  160 CONTINUE
      SUM=T*F1-F2+COEF(1)
      RETURN
      END
      DOUBLE PRECISION FUNCTION GOFX(X)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CFOFX/ CX      
C      DATA RTPI /1.772453850905516D0/
      EXTERNAL FOFX
      CX=X
      INIT=0
      X1=X
      SIG=2.0D0
      REL=0.50D-15
C      PRINT *,"X=",X
C      PAUSE
      CALL DQUAD8(FOFX,INIT,X1,SIG,REL,X2,QANS,IERR)
      SIG=6.0D0
      CALL DQUAD8(FOFX,INIT,X1,SIG,REL,X2,QANS,IERR)
      XSQ=X*X
      GOFX=QANS*XSQ*X
      GOFX=GOFX*DEXP(XSQ)
C      GOFX=QANS
C     PRINT *,"GOFX=",X,GOFX
C      PAUSE
      RETURN
      END
      DOUBLE PRECISION FUNCTION FOFX(X)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      KODE=2
      FOFX=DRERF(X,KODE,NZ)/X
      RETURN
      END
      DOUBLE PRECISION FUNCTION DPGOFX(X)
      DOUBLE PRECISION X,PI,DH,DTOL
      DOUBLE PRECISION DPMAX
C     SINGLE PRECISION SPMAX

      PARAMETER ( MXNDIG=256 , NBITS=32 ,
     *          LPACK  = (MXNDIG+1)/2 + 1 , LUNPCK = (6*MXNDIG)/5 + 20,
     *          LMWA   = 2*LUNPCK         , LJSUMS = 8*LUNPCK,
     *          LMBUFF = ((LUNPCK+3)*(NBITS-1)*301)/2000 + 6  )
C
C
      DIMENSION MX(200),MGOFX(200)

C
      COMMON /FMUSER/ NDIG,JBASE,JFORM1,JFORM2,KRAD,
     *                KW,NTRACE,LVLTRC,KFLAG,KWARN,KROUND
C
      COMMON /FM/ MWA(LMWA),NCALL,MXEXP,MXEXP2,KARGSW,KEXPUN,KEXPOV,
     *            KUNKNO,IUNKNO,RUNKNO,MXBASE,MXNDG2,KSTACK(19),
     *            MAXINT,SPMAX,DPMAX
C
      COMMON /FMSAVE/ NDIGPI,NJBPI,NDIGE,NJBE,NDIGLB,NJBLB,NDIGLI,NJBLI,
     *                MPISAV(LUNPCK),MESAV(LUNPCK),MLBSAV(LUNPCK),
     *                MLN1(LUNPCK),MLN2(LUNPCK),MLN3(LUNPCK),
     *                MLN4(LUNPCK)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)
C
C-----------------------------------------------------------------------
      IF (INIT.EQ.0) THEN
        NPREC=60
        CALL FPSET(NPREC)
C       FOR JBASE=10000, NDIG=NPREC/4+2
        NDIG=INT(NPREC/4)+5
        NCDIG=NDIG
        K=0
        CALL FPI2M(K,MZERO)
        K=1
        CALL FPI2M(K,MONE)
        K=2        
        CALL FPI2M(K,MTWO)
        K=9
        CALL FPI2M(K,MNINE)
        CALL FPPI(MPI)
        CALL FPM2DP(MPI,PI)
        CALL FPSQRT(MPI,MRTPI)
        CALL FPDIV(MONE,MRTPI,MTEMP)
        CALL FPADD(MTEMP,MTEMP,MTRTPI)
C       MTRTPI=2/RTPI
        DTOL=0.50D-16
        CALL FPDP2M(DTOL,MTOL)
        DH=0.5D0
        CALL FPDP2M(DH,MDH)
C       INIT=1
C       INIT IS SET TO 1 AFTER THE INITIALIZATION OF MQUAD8
      ENDIF
      CALL FPDP2M(X,MX)
      CALL MPGOFX(MX,MGOFX)
C      CALL MFOFXA(MX,MGOFX)
      CALL FPM2DP(MGOFX,DPGOFX)
      END
      SUBROUTINE MPGOFX(MX,MGOFX)
C
C     COMPUTES INT ON (X,INF) OF ERFC(X)/X OR ERFC(X) IN MP ARITHMETIC
C     USING GAUS8 FORMULA ON REPEATED SUBDIVIDED INTERVALS
C
      DOUBLE PRECISION DSUM
      LOGICAL FPCOMP
C
      DIMENSION MX(200),MGOFX(200)
C
      DIMENSION MSUM(200),MSUMA(200),MANS(200),MANSA(200),
     &MTEMP(200),MSUM1(200),MDX(200),MTEMP1(200),MA(200),MB(200)
C
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPEQU(MDH,MDX,NDIG,NDIG)
      CALL FPEQU(MZERO,MSUM1,NDIG,NDIG)
      KL=100
      DO 10 J=1,100
        CALL FPDIVI(MDX,2,MDX)
        KL=KL+KL
        CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
        CALL FPEQU(MX,MB,NDIG,NDIG)
        DO 20 K=1,KL
          CALL FPEQU(MB,MA,NDIG,NDIG)
          CALL FPADD(MA,MDX,MB)
          CALL MQUAD8(MA,MB,MANS)
          CALL FPADD(MSUM,MANS,MSUM)
          CALL FPM2DP(MSUM,DSUM)
          IF(J.EQ.1) THEN
            CALL FPABS(MANS,MANSA)
            CALL FPABS(MSUM,MSUMA)
            CALL FPMPY(MSUMA,MTOL,MSUMA)
            IF (FPCOMP(MANSA,'LT',MSUMA)) GOTO 25
          ENDIF          
 20     CONTINUE
 25     CONTINUE
        IF(J.EQ.1) THEN
          KL=K
          CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
          GOTO 10
        ENDIF
        CALL FPSUB(MSUM,MSUM1,MTEMP)
        CALL FPABS(MTEMP,MTEMP)
        CALL FPABS(MSUM,MSUMA)
        CALL FPMPY(MSUMA,MTOL,MSUMA)
        IF (FPCOMP(MTEMP,'LT',MSUMA)) GOTO 30
        CALL FPEQU(MSUM,MSUM1,NDIG,NDIG)
 10   CONTINUE
 30   CONTINUE
      CALL FPMPY(MX,MX,MTEMP)
      CALL FPEXP(MTEMP,MTEMP1)
C      CALL FPMPY(MTEMP,MX,MTEMP)
      CALL FPMPY(MTEMP1,MTEMP,MTEMP)
      CALL FPMPY(MTEMP,MSUM,MGOFX)
      RETURN
      END
      SUBROUTINE MQUAD8(MA,MB,MQUAD)
      DOUBLE PRECISION DQUAD
      DIMENSION MA(200),MB(200),MAPB(200),MBMA(200),MTEMP1(200),
     &MTEMP2(200),MTEMP3(200),MTEMP4(200),MTEMP5(200),MTEMP6(200),
     &MTEMP7(200),MTEMP8(200),MSUM(200),MFUN(200),MQUAD(200)
      DIMENSION IX1(4),IX2(4),IX3(4),IX4(4),IW1(4),IW2(4),IW3(4),IW4(4)
C
      COMMON /CINIT/INIT
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
      COMMON /GAUSS/ MX1(200),MX2(200),MX3(200),MX4(200),MW1(200),
     &MW2(200),MW3(200),MW4(200)

C
C     GAUSS ABSCISSAS AND WEIGHTS IN MP
      DATA (IX1(J),J=1,4)/0,18343464,24956498,05000000/
      DATA (IX2(J),J=1,4)/0,52553240,99163289,86000000/
      DATA (IX3(J),J=1,4)/0,79666647,74136267,40000000/
      DATA (IX4(J),J=1,4)/0,96028985,64975362,32000000/
      DATA (IW1(J),J=1,4)/0,36268378,33783619,83000000/
      DATA (IW2(J),J=1,4)/0,31370664,58778872,87000000/
      DATA (IW3(J),J=1,4)/0,22238103,44533744,71000000/
      DATA (IW4(J),J=1,4)/0,10122853,62903762,59000000/
C
      NDIG=NCDIG
      IF (INIT.EQ.0) THEN
        CALL FPEQU(MZERO,MX1,NDIG,NDIG)
        MX1(1)=IX1(1)
        MX1(2)=IX1(2)
        MX1(3)=IX1(3)
        MX1(4)=IX1(4)
        CALL FPEQU(MZERO,MX2,NDIG,NDIG)
        MX2(1)=IX2(1)
        MX2(2)=IX2(2)
        MX2(3)=IX2(3)
        MX2(4)=IX2(4)
        CALL FPEQU(MZERO,MX3,NDIG,NDIG)
        MX3(1)=IX3(1)
        MX3(2)=IX3(2)
        MX3(3)=IX3(3)
        MX3(4)=IX3(4)
        CALL FPEQU(MZERO,MX4,NDIG,NDIG)
        MX4(1)=IX4(1)
        MX4(2)=IX4(2)
        MX4(3)=IX4(3)
        MX4(4)=IX4(4)
        CALL FPEQU(MZERO,MW1,NDIG,NDIG)
        MW1(1)=IW1(1)
        MW1(2)=IW1(2)
        MW1(3)=IW1(3)
        MW1(4)=IW1(4)
        CALL FPEQU(MZERO,MW2,NDIG,NDIG)
        MW2(1)=IW2(1)
        MW2(2)=IW2(2)
        MW2(3)=IW2(3)
        MW2(4)=IW2(4)
        CALL FPEQU(MZERO,MW3,NDIG,NDIG)
        MW3(1)=IW3(1)
        MW3(2)=IW3(2)
        MW3(3)=IW3(3)
        MW3(4)=IW3(4)
        CALL FPEQU(MZERO,MW4,NDIG,NDIG)
        MW4(1)=IW4(1)
        MW4(2)=IW4(2)
        MW4(3)=IW4(3)
        MW4(4)=IW4(4)
        PRINT *,'GAUSSIAN ABSCISSAS AND WEIGHTS '
        CALL FPPRNT(MX1)
        CALL FPPRNT(MX2)
        CALL FPPRNT(MX3)
        CALL FPPRNT(MX4)
        CALL FPPRNT(MW1)
        CALL FPPRNT(MW2)
        CALL FPPRNT(MW3)
        CALL FPPRNT(MW4)
        PAUSE
        INIT=1
      ENDIF
      CALL FPADD(MA,MB,MAPB)
      CALL FPDIVI(MAPB,2,MAPB)
      CALL FPSUB(MB,MA,MBMA)
      CALL FPDIVI(MBMA,2,MBMA)
      CALL FPMPY(MX1,MBMA,MTEMP1)
      CALL FPMPY(MX2,MBMA,MTEMP2)
      CALL FPMPY(MX3,MBMA,MTEMP3)
      CALL FPMPY(MX4,MBMA,MTEMP4)
      I=-1
      CALL FPMPYI(MTEMP1,I,MTEMP5)
      CALL FPMPYI(MTEMP2,I,MTEMP6)
      CALL FPMPYI(MTEMP3,I,MTEMP7)
      CALL FPMPYI(MTEMP4,I,MTEMP8)
C
      CALL FPADD(MTEMP1,MAPB,MTEMP1)
      CALL FPADD(MTEMP2,MAPB,MTEMP2)
      CALL FPADD(MTEMP3,MAPB,MTEMP3)
      CALL FPADD(MTEMP4,MAPB,MTEMP4)
      CALL FPADD(MTEMP5,MAPB,MTEMP5)
      CALL FPADD(MTEMP6,MAPB,MTEMP6)
      CALL FPADD(MTEMP7,MAPB,MTEMP7)
      CALL FPADD(MTEMP8,MAPB,MTEMP8)
C
      CALL FPEQU(MZERO,MSUM,NDIG,NDIG)
      CALL MFOFX(MTEMP1,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP2,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP3,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP4,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP5,MFUN)
      CALL FPMPY(MW1,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP6,MFUN)
      CALL FPMPY(MW2,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP7,MFUN)
      CALL FPMPY(MW3,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL MFOFX(MTEMP8,MFUN)
      CALL FPMPY(MW4,MFUN,MFUN)
      CALL FPADD(MSUM,MFUN,MSUM)
      CALL FPM2DP(MFUN,DQUAD)
C      PRINT*,'MFUN:',DQUAD
      CALL FPMPY(MSUM,MBMA,MQUAD)
C      CALL FPM2DP(MQUAD,DQUAD)
C      PRINT *,'IN MQUAD, ANS=',DQUAD
C      PAUSE
      RETURN
      END
      SUBROUTINE MFOFXA(MT,MFUN)
      DOUBLE PRECISION DSUM,DMT
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPM2DP(MT,DMT)
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
C        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          CALL FPM2DP(MSUM,DSUM)
C          PRINT*,'IN MFOFX, X,SUM ',DMT,DSUM
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MSUMT)
C          CALL FPSUB(MTRM1,MTEMP,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          IF (FPCOMP(MTRMA,'LE',MTOL)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
        CALL FPMPY(MRTPI,MT2,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
      RETURN
      END
      SUBROUTINE MFOFX(MT,MFUN)
      DOUBLE PRECISION DSUM,DMT
      LOGICAL FPCOMP
      DIMENSION MT(200),MT2(200),MTRM(200),MSUM(200),MCI(200),MCNT(200),
     &MTRM1(200),MTRMA(200),MTRMS(200),MTEMP(200),MSUMA(200),MAK(200),
     &MFUN(200),MSUMT(200)
      COMMON /CWORK/MPI(200),MRTPI(200),MONE(200),MTWO(200),MTRTPI(200),
     *MZERO(200),MTOL(200),MDH(200),MNINE(200),NCDIG
C
      NDIG=NCDIG
      CALL FPM2DP(MT,DMT)
      CALL FPMPY(MT,MT,MT2)
      CALL FPEQU(MONE,MTRM,NDIG,NDIG)
      CALL FPEQU(MONE,MSUM,NDIG,NDIG)
      J=-1
      IF(FPCOMP(MT,"LT",MNINE)) THEN
        CALL FPEQU(MONE,MCI,NDIG,NDIG)
        CALL FPEQU(MTWO,MCNT,NDIG,NDIG)
        CALL FPADD(MONE,MCNT,MCNT)
        CALL FPDIV(MONE,MT,MTRM1)
        DO 10 I=1,1000
          CALL FPMPY(MTRM,MT2,MTRM)
          CALL FPDIV(MTRM,MCI,MTRM)
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPDIV(MTRM,MCNT,MTRMS)
          CALL FPADD(MSUM,MTRMS,MSUM)
          CALL FPM2DP(MSUM,DSUM)
C          PRINT*,'IN MFOFX, X,SUM ',DMT,DSUM
          IF(I.LT.5) GOTO 5
          CALL FPMPY(MSUM,MTRTPI,MTEMP)
C          CALL FPSUB(MTRM1,MTEMP,MSUMT)
          CALL FPMPY(MTEMP,MT,MTEMP)
          CALL FPSUB(MONE,MTEMP,MSUMT)
          CALL FPABS(MTRMS,MTRMA)
          CALL FPABS(MSUMT,MSUMA)
          CALL FPMPY(MTOL,MSUMA,MSUMA)
          IF (FPCOMP(MTRMA,'LT',MSUMA)) GOTO 15
  5       CONTINUE
          CALL FPADD(MCI,MONE,MCI)
          CALL FPADD(MCNT,MTWO,MCNT)
  10    CONTINUE
        PRINT*,"DROP THRU SERIES LOOP"
        PAUSE
  15    CONTINUE
      ELSE
        CALL FPEQU(MDH,MAK,NDIG,NDIG)
        DO 20 I=1,25
          CALL FPMPY(MTRM,MAK,MTRM)
          CALL FPDIV(MTRM,MT2,MTRM)
          CALL FPABS(MTRM,MTRMA)
          IF (FPCOMP(MTRMA,'LE',MTOL)) GOTO 25
          CALL FPMPYI(MTRM,J,MTRM)
          CALL FPADD(MSUM,MTRM,MSUM)
          CALL FPADD(MAK,MONE,MAK)
  20    CONTINUE
        PRINT*,"DROP THRU ASYMPTOTOIC SERIES LOOP"
        PAUSE
  25    CONTINUE
        CALL FPMPYI(MT2,J,MAK)
        CALL FPEXP(MAK,MTRM)
        CALL FPMPY(MSUM,MTRM,MSUM)
C        CALL FPMPY(MRTPI,MT2,MAK)
        CALL FPMPY(MRTPI,MT,MAK)
        CALL FPDIV(MSUM,MAK,MSUMT)
      ENDIF
      CALL FPEQU(MSUMT,MFUN,NDIG,NDIG)
      RETURN
      END
